package com.qmth.distributed.print.business.entity;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.base.BaseEntity;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;

/**
 * <p>
 * 考试试卷结构表
 * </p>
 *
 * @author xf
 */
@TableName("exam_paper_structure")
public class ExamPaperStructure extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "学校id")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField("school_id")
    private Long schoolId;

    @ApiModelProperty(value = "学期ID")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(value = "semester_id")
    private Long semesterId;

    /**
     * 学期名称
     */
    @TableField("semester_name")
    private String semesterName;

    @ApiModelProperty(value = "考试id")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField("exam_id")
    private Long examId;

    @ApiModelProperty(value = "云阅卷考试ID")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(value = "third_relate_id")
    private Long thirdRelateId;

    /**
     * 云阅卷考试名称
     */
    @TableField("third_relate_name")
    private String thirdRelateName;

    /**
     * 试卷编号
     */
    @TableField("paper_number")
    private String paperNumber;

    /**
     * 课程代码
     */
    @TableField("course_code")
    private String courseCode;
    /**
     * 课程名称
     */
    @TableField("course_name")
    private String courseName;
    /**
     * 课程创建的任务序号（和exam_task保持一致）
     */
    @TableField("sequence")
    private String sequence;
    /**
     * 试卷类型
     */
    @TableField("paper_type")
    private String paperType;
    /**
     * 状态
     */
    private String status;
    @TableField("is_finish")
    private Boolean isFinish;
    /**
     * 试卷原卷和标答附件ID
     */
    @TableField("paper_answer")
    private String paperAnswer;
    /**
     * 客观题试卷结构JSON
     */
    @TableField("objective_structure")
    private String objectiveStructure;
    /**
     * 主观题试卷结构JSON
     */
    @TableField("subjective_structure")
    private String subjectiveStructure;

    @ApiModelProperty(value = "试卷整体json")
    @TableField("paper_info_json")
    private String paperInfoJson;
    /**
     * 命题老师ID
     */
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField("proposition_teacher_id")
    private Long propositionTeacherId;

    /**
     * 0-禁用，1-启用
     */
    private Boolean enable;

    private String markLeader;

    @ApiModelProperty(value = "云阅卷试卷结构json")
    @TableField("cloud_info_json")
    private String cloudInfoJson;

    @ApiModelProperty(value = "云阅卷试卷结构和知学知考结构不一样（默认false）")
    @TableField("structure_change")
    private Boolean structureChange;


    @ApiModelProperty(value = "客观题答案有提交，需要重新统分（1：有变动，需要统分，0：不统分）（默认0）")
    @TableField("object_answer_change")
    private Boolean objectAnswerChange;

    @ApiModelProperty(value = "是否开启分班阅（1：开启，0：关闭）（默认0）")
    @TableField("open_class_reading")
    private Boolean openClassReading;

    @TableField(exist = false)
    private String taskStatus;

    @TableField(exist = false)
    private String transferId;

    public Long getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Long schoolId) {
        this.schoolId = schoolId;
    }

    public Long getSemesterId() {
        return semesterId;
    }

    public void setSemesterId(Long semesterId) {
        this.semesterId = semesterId;
    }

    public String getSemesterName() {
        return semesterName;
    }

    public void setSemesterName(String semesterName) {
        this.semesterName = semesterName;
    }

    public Long getThirdRelateId() {
        return thirdRelateId;
    }

    public void setThirdRelateId(Long thirdRelateId) {
        this.thirdRelateId = thirdRelateId;
    }

    public String getThirdRelateName() {
        return thirdRelateName;
    }

    public void setThirdRelateName(String thirdRelateName) {
        this.thirdRelateName = thirdRelateName;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getSequence() {
        return sequence;
    }

    public void setSequence(String sequence) {
        this.sequence = sequence;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public Boolean getFinish() {
        return isFinish;
    }

    public void setFinish(Boolean finish) {
        isFinish = finish;
    }

    public String getPaperAnswer() {
        return paperAnswer;
    }

    public void setPaperAnswer(String paperAnswer) {
        this.paperAnswer = paperAnswer;
    }

    public String getObjectiveStructure() {
        return objectiveStructure;
    }

    public void setObjectiveStructure(String objectiveStructure) {
        this.objectiveStructure = objectiveStructure;
    }

    public String getSubjectiveStructure() {
        return subjectiveStructure;
    }

    public void setSubjectiveStructure(String subjectiveStructure) {
        this.subjectiveStructure = subjectiveStructure;
    }

    public Long getPropositionTeacherId() {
        return propositionTeacherId;
    }

    public void setPropositionTeacherId(Long propositionTeacherId) {
        this.propositionTeacherId = propositionTeacherId;
    }

    public Boolean getEnable() {
        return enable;
    }

    public void setEnable(Boolean enable) {
        this.enable = enable;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public String getPaperInfoJson() {
        return paperInfoJson;
    }

    public void setPaperInfoJson(String paperInfoJson) {
        this.paperInfoJson = paperInfoJson;
    }

    public String getTaskStatus() {
        return taskStatus;
    }

    public void setTaskStatus(String taskStatus) {
        this.taskStatus = taskStatus;
    }

    public String getMarkLeader() {
        return markLeader;
    }

    public void setMarkLeader(String markLeader) {
        this.markLeader = markLeader;
    }

    public String getCloudInfoJson() {
        return cloudInfoJson;
    }

    public void setCloudInfoJson(String cloudInfoJson) {
        this.cloudInfoJson = cloudInfoJson;
    }

    public Boolean getStructureChange() {
        return structureChange;
    }

    public void setStructureChange(Boolean structureChange) {
        this.structureChange = structureChange;
    }

    public String getTransferId() {
        return transferId;
    }

    public void setTransferId(String transferId) {
        this.transferId = transferId;
    }

    public Boolean getObjectAnswerChange() {
        return objectAnswerChange;
    }

    public void setObjectAnswerChange(Boolean objectAnswerChange) {
        this.objectAnswerChange = objectAnswerChange;
    }

    public Boolean getOpenClassReading() {
        return openClassReading;
    }

    public void setOpenClassReading(Boolean openClassReading) {
        this.openClassReading = openClassReading;
    }

    /**
     * 状态处理
     *
     * @param status   状态
     * @param type     大类，取ExamPaperStructureStatusTypeEnum
     * @param category 类别，save或者sync
     * @param value    值，true或者false
     * @return
     */
    public static String parseStatus(String status, String type, String category, Boolean value) {
        JSONObject jsonObject = StringUtils.isNotBlank(status) ? JSON.parseObject(status, JSONObject.class) : new JSONObject();
        JSONObject object = jsonObject.containsKey(type) ? jsonObject.getJSONObject(type) : new JSONObject();
        object.put(category, value);
        jsonObject.put(type, object);
        return JSON.toJSONString(jsonObject);
    }
}
