package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.marking.ConvertJpgStorage;
import com.qmth.distributed.print.business.bean.params.ExamCardParams;
import com.qmth.distributed.print.business.bean.params.GenericExamCardParams;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.CardStatusEnum;
import com.qmth.distributed.print.business.enums.CardTypeEnum;
import com.qmth.distributed.print.business.enums.MakeMethodEnum;
import com.qmth.distributed.print.business.mapper.ExamCardMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.util.CreatePdfUtil;
import com.qmth.distributed.print.business.util.HtmlToJpgUtil;
import com.qmth.distributed.print.business.util.HtmlToPdfUtil;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.CardCreateMethodEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PageSizeEnum;
import com.qmth.teachcloud.common.service.BasicAttachmentService;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.FileCopyUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * <p>
 * 题卡 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamCardServiceImpl extends ServiceImpl<ExamCardMapper, ExamCard> implements ExamCardService {

    @Resource
    private BasicExamRuleService basicExamRuleService;

    @Resource
    private BasicCardRuleService basicCardRuleService;

    @Resource
    private ExamTaskDetailService examTaskDetailService;

    @Resource
    private ExamTaskService examTaskService;

    @Resource
    private BasicAttachmentService basicAttachmentService;

    @Resource
    TeachcloudCommonService teachcloudCommonService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    HtmlToJpgUtil htmlToJpgUtil;

    @Resource
    CreatePdfUtil createPdfUtil;

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Transactional
    @Override
    public String saveExamCard(ExamCardParams examCardParams) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser user = (SysUser) ServletUtil.getRequestUser();

        // 新增
        ExamCard examCard;
        if (examCardParams.getTitle().getBytes().length > 80) {
            throw ExceptionResultEnum.ERROR.exception("标题最长只能输入80个字符");
        }
        if (examCardParams.getId() == null) {
            validateCardData(examCardParams);

            QueryWrapper<ExamCard> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamCard::getSchoolId, schoolId)
                    .eq(ExamCard::getCourseCode, examCardParams.getCourseCode())
                    .eq(ExamCard::getTitle, examCardParams.getTitle());
            List<ExamCard> examCardList = this.list(queryWrapper);
            if (!examCardList.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("题卡名称已存在");
            }

            examCard = new ExamCard();
            examCard.setSchoolId(schoolId);
            if (Objects.isNull(examCardParams.getTemplateId())) {
                examCard.setOrgId(basicCourseService.getOrgIdBySchoolIdAndCourseCode(schoolId, examCardParams.getCourseCode()));
            } else {
                examCard.setOrgId(user.getOrgId());
            }
            examCard.setCourseCode(examCardParams.getCourseCode());
            examCard.setCourseName(examCardParams.getCourseName());
            examCard.setTitle(examCardParams.getTitle());
            examCard.setMakeMethod(examCardParams.getMakeMethod());
            examCard.setStatus(examCardParams.getStatus());
            examCard.setType(examCardParams.getType());
            examCard.setAttachmentId(examCardParams.getAttachmentId());
            examCard.setCardRuleId(examCardParams.getCardRuleId());
            examCard.insertInfo(user.getId());

            // 自动保存
            if (CardStatusEnum.SUBMIT.equals(examCardParams.getStatus())) {
                examCard.setContent(examCardParams.getContent());
                examCard.setStageContent(null);
                String htmlContent = examCardParams.getHtmlContent();
                List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertHtmlToJpgByCard(examCard.getTitle(), htmlContent, PageSizeEnum.A3);
                if (convertJpgStorageList.size() > 0) {
                    examCard.setJpgAttachment(JSON.toJSONString(convertJpgStorageList));
                }
                examCard.setHtmlContent(htmlContent);
            } else {
                examCard.setStageContent(examCardParams.getContent());
            }
            this.save(examCard);
        }
        // 修改
        else {
            QueryWrapper<ExamCard> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamCard::getSchoolId, schoolId)
                    .eq(ExamCard::getCourseCode, examCardParams.getCourseCode())
                    .eq(ExamCard::getTitle, examCardParams.getTitle())
                    .ne(ExamCard::getId, examCardParams.getId());
            List<ExamCard> examCardList = this.list(queryWrapper);
            if (!examCardList.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("题卡名称已存在");
            }

            examCard = this.getById(examCardParams.getId());
            examCard.setTitle(examCardParams.getTitle());
            examCard.setStatus(examCardParams.getStatus());
            examCard.setUpdateId(user.getId());
            examCard.setUpdateTime(System.currentTimeMillis());
            if (CardStatusEnum.SUBMIT.equals(examCardParams.getStatus())) {
                examCard.setContent(examCardParams.getContent());
                examCard.setStageContent(null);
                String htmlContent = examCardParams.getHtmlContent();
                List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertHtmlToJpgByCard(examCard.getTitle(), htmlContent, PageSizeEnum.A3);
                if (convertJpgStorageList.size() > 0) {
                    examCard.setJpgAttachment(JSON.toJSONString(convertJpgStorageList));
                }
                examCard.setHtmlContent(htmlContent);
            } else {
                examCard.setStageContent(examCardParams.getContent());
            }
            this.updateById(examCard);
        }

        return String.valueOf(examCard.getId());
    }

    @Override
    public List<ExamCard> listSelectCard(String courseCode, Long cardRuleId, String paperNumber) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        // 通用题卡
        List<ExamCard> list = new ArrayList<>();
        List<ExamCard> genericCards = this.baseMapper.listGenericCard(schoolId, CardTypeEnum.GENERIC.name(), cardRuleId);
        if (!genericCards.isEmpty()) {
            list.addAll(genericCards);
        }
        // 专卡
        if (SystemConstant.ALL_CARD != cardRuleId) {
            // 只查自己创建的专卡(20230405已改为查询课程下所有题卡，允许复制，不能选择)
            List<ExamCard> customCards = this.baseMapper.listCustom(schoolId, courseCode, CardTypeEnum.CUSTOM.name(), cardRuleId);
            if (!customCards.isEmpty()) {
                list.addAll(customCards);
            }
        }

        // 命题任务未提交的任务，单独查询题卡
        List<ExamCard> customCards = this.baseMapper.listCustomByExamStage(schoolId, courseCode, paperNumber, CardTypeEnum.CUSTOM.name());
        if (!customCards.isEmpty()) {
            for (ExamCard customCard : customCards) {
                long count = list.stream().filter(m -> m.getId().equals(customCard.getId())).count();
                if (count == 0) {
                    list.add(customCard);
                }
            }
        }
        list.sort(Comparator.comparing(ExamCard::getType));
        return list;
    }

    @Override
    public IPage<ExamCard> listPage(String cardType, String title, String createMethod, Boolean enable, Long createStartTime, Long createEndTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<ExamCard> cardPageDtoIPage = this.baseMapper.listPage(new Page<>(pageNumber, pageSize), schoolId, cardType, title, createMethod, enable, createStartTime, createEndTime, dpr);
        for (ExamCard record : cardPageDtoIPage.getRecords()) {
            if (StringUtils.isNotBlank(record.getJpgAttachment())) {
                List<JSONObject> mapList = JSON.parseArray(record.getJpgAttachment(), JSONObject.class);
                List<String> imageUrls = new ArrayList<>();
                // 组装图片访问url
                for (JSONObject object : mapList) {
                    imageUrls.add(teachcloudCommonService.filePreview(object.getLong("attachmentId")));
                }
                record.setImageUrls(imageUrls);
            }
        }
        return cardPageDtoIPage;
    }

    @Transactional
    @Override
    public Long saveGeneric(GenericExamCardParams params) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        QueryWrapper<ExamCard> checkTitleQueryWrapper = new QueryWrapper<>();
        checkTitleQueryWrapper.lambda().eq(ExamCard::getSchoolId, schoolId)
                .eq(ExamCard::getType, CardTypeEnum.GENERIC)
                .eq(ExamCard::getTitle, params.getTitle().trim());
        ExamCard checkTitleExamCardList = this.getOne(checkTitleQueryWrapper);

        ExamCard examCard = new ExamCard();
        // 新增
        if (Objects.isNull(params.getId())) {
            if (checkTitleExamCardList != null) {
                throw ExceptionResultEnum.ERROR.exception("题卡名称已使用");
            }
            // 保存题卡
            examCard.setId(SystemConstant.getDbUuid());
            examCard.setSchoolId(schoolId);
            examCard.setOrgId(sysUser.getOrgId());

            examCard.setTitle(params.getTitle());
            examCard.setMakeMethod(MakeMethodEnum.SELECT); // 默认值
            // 上传题卡，状态为提交
            examCard.setStatus(CardCreateMethodEnum.UPLOAD.equals(params.getCreateMethod()) ? CardStatusEnum.SUBMIT : CardStatusEnum.valueOf(params.getStatus()));
            examCard.setType(CardTypeEnum.GENERIC); // 默认值
            examCard.setCreateMethod(params.getCreateMethod());
            examCard.setAttachmentId(params.getAttachmentId());
            examCard.setEnable(true); // 默认值
            examCard.setCardRuleId(params.getCardRuleId());
            examCard.setRemark(params.getRemark());
            examCard.setCreateId(sysUser.getId());
            examCard.setCreateTime(System.currentTimeMillis());
            this.save(examCard);

            // 方式为上传
            String htmlContent;
            if (CardCreateMethodEnum.UPLOAD.equals(params.getCreateMethod())) {
                if (params.getAttachmentId() == null) {
                    throw ExceptionResultEnum.ERROR.exception("请上传模板文件");
                }
                BasicAttachment attachment = basicAttachmentService.getById(params.getAttachmentId());
                if (attachment == null) {
                    throw ExceptionResultEnum.ERROR.exception("无模板文件记录，请重新上传");
                }
                if (!SystemConstant.HTML_PREFIX.equalsIgnoreCase(attachment.getType())) {
                    throw ExceptionResultEnum.ERROR.exception("只能上传后缀为.html文件");
                }
                htmlContent = teachcloudCommonService.readFileContent(attachment.getPath());
                examCard.setHtmlContent(htmlContent);
            }
            // 方式为自定义
            else if (CardCreateMethodEnum.STANDARD.equals(params.getCreateMethod())
                    || CardCreateMethodEnum.FREE.equals(params.getCreateMethod())) {
                if (CardStatusEnum.SUBMIT.name().equals(params.getStatus())) {
                    examCard.setContent(params.getContent());
                    examCard.setStageContent(null);
                    htmlContent = params.getHtmlContent();
                    examCard.setHtmlContent(htmlContent);
                } else {
                    examCard.setStageContent(params.getContent());
                }
            } else {
                throw ExceptionResultEnum.ERROR.exception("不存在的题卡创建方式");
            }
        }
        // 修改
        else {
            examCard = this.getById(params.getId());
            if (checkTitleExamCardList != null && !Objects.equals(params.getId(), examCard.getId())) {
                throw ExceptionResultEnum.ERROR.exception("题卡名称重复");
            }
            examCard.setTitle(params.getTitle());
            examCard.setAttachmentId(params.getAttachmentId());
            examCard.setCardRuleId(params.getCardRuleId());
            // 上传题卡，状态为提交
            examCard.setStatus(CardCreateMethodEnum.UPLOAD.equals(params.getCreateMethod()) ? CardStatusEnum.SUBMIT : CardStatusEnum.valueOf(params.getStatus()));
            examCard.setRemark(params.getRemark());
            examCard.setUpdateId(sysUser.getId());
            examCard.setUpdateTime(System.currentTimeMillis());

            String htmlContent;
            // 方式为上传
            if (CardCreateMethodEnum.UPLOAD.equals(params.getCreateMethod())) {
                if (params.getAttachmentId() == null) {
                    throw ExceptionResultEnum.ERROR.exception("请上传模板文件");
                }
                BasicAttachment attachment = basicAttachmentService.getById(params.getAttachmentId());
                if (attachment == null) {
                    throw ExceptionResultEnum.ERROR.exception("无模板文件记录，请重新上传");
                }
                if (!SystemConstant.HTML_PREFIX.equalsIgnoreCase(attachment.getType())) {
                    throw ExceptionResultEnum.ERROR.exception("只能上传后缀为.html文件");
                }
                htmlContent = teachcloudCommonService.readFileContent(attachment.getPath());
                examCard.setHtmlContent(htmlContent);
                // 上传题卡，状态为提交
                examCard.setStatus(CardStatusEnum.SUBMIT);
            }
            // 方式为自定义
            else if (CardCreateMethodEnum.STANDARD.equals(params.getCreateMethod()) || CardCreateMethodEnum.FREE.equals(params.getCreateMethod())) {
                if (CardStatusEnum.SUBMIT.name().equals(params.getStatus())) {
                    examCard.setContent(params.getContent());
                    examCard.setStageContent(null);
                    htmlContent = params.getHtmlContent();
                    examCard.setHtmlContent(htmlContent);
                } else {
                    examCard.setStageContent(params.getContent());
                }
            } else {
                throw ExceptionResultEnum.ERROR.exception("不存在的题卡创建方式");
            }
        }
        if (CardStatusEnum.SUBMIT.name().equals(params.getStatus())) {
            // 生成题卡图片
            List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertHtmlToJpgByCard(examCard.getTitle(), examCard.getHtmlContent(), PageSizeEnum.A3);
            if (convertJpgStorageList.size() > 0) {
                examCard.setJpgAttachment(JSON.toJSONString(convertJpgStorageList));
            }
        }
        this.saveOrUpdate(examCard);
        return examCard.getId();
    }

    @Override
    public Boolean deleteGeneric(Long id) {
        // 校验题卡是否被绑定
        List<ExamTaskDetail> examTaskDetails = examTaskDetailService.getExamTaskDetailByCardIdJson(id);
        if (examTaskDetails != null && examTaskDetails.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("题卡已绑定命题任务，不能删除");
        }
        return this.removeById(id);
    }

    @Transactional
    @Override
    public Long copyCard(Long id, String courseCode) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();

        ExamCard examCard = this.getById(id);
        if (examCard == null) {
            throw ExceptionResultEnum.ERROR.exception("题卡不存在");
        }
        if (CardTypeEnum.CUSTOM.equals(examCard.getType()) && !courseCode.equals(examCard.getCourseCode())) {
            throw ExceptionResultEnum.ERROR.exception("复制的题卡不属于当前课程，无法复制");
        }

        // 复制题卡
        ExamCard copyExamCard = new ExamCard();
        BeanUtils.copyProperties(examCard, copyExamCard);
        String title = checkTitle(examCard, courseCode);
        copyExamCard.setTitle(title);
        copyExamCard.insertInfo(requestUser.getId());
        copyExamCard.setSyncStatus(null);
        if (CardTypeEnum.GENERIC.equals(examCard.getType()) && CardCreateMethodEnum.STANDARD.equals(examCard.getCreateMethod())) {
            BasicCourse basicCourse = basicCourseService.findByCourseCode(courseCode, requestUser.getSchoolId());
            copyExamCard.setCourseCode(courseCode);
            copyExamCard.setCourseName(basicCourse != null ? basicCourse.getName() : null);
            copyExamCard.setMakeMethod(MakeMethodEnum.SELF);
            copyExamCard.setCreateMethod(null);
            copyExamCard.setType(CardTypeEnum.CUSTOM);
        }
        copyExamCard.setRemark(String.format("来源题卡:%s(%s)", id, examCard.getTitle()));
        copyExamCard.setUsed(false);
        // 生成图片
        List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertHtmlToJpgByCard(examCard.getTitle(), copyExamCard.getHtmlContent(), PageSizeEnum.A3);
        if (!convertJpgStorageList.isEmpty()) {
            copyExamCard.setJpgAttachment(JSON.toJSONString(convertJpgStorageList));
        } else {
            copyExamCard.setJpgAttachment(null);
        }
        this.save(copyExamCard);
        return copyExamCard.getId();
    }

    @Override
    public void convertImage(Long id) {
        ExamCard examCard = this.getById(id);
        List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertHtmlToJpgByCard(examCard.getTitle(), examCard.getHtmlContent(), PageSizeEnum.A3);
        if (convertJpgStorageList.size() > 0) {
            String jpgAttachmentIds = JSON.toJSONString(convertJpgStorageList);
            UpdateWrapper<ExamCard> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamCard::getJpgAttachment, jpgAttachmentIds)
                    .eq(ExamCard::getId, id);
            this.update(updateWrapper);
        }
    }

    @Override
    public void cardDownload(HttpServletResponse response, String id) {
        String rootPath = null;
        try {
            // 本地保存目录
            File tempFile = SystemConstant.getFileTempDirVar(System.currentTimeMillis() + File.separator + SystemConstant.getNanoId(), SystemConstant.TEMP_PREFIX);
            rootPath = tempFile.getParent();

            // 删除临时目录中创建的临时文件
            if (Objects.nonNull(tempFile)) {
                tempFile.delete();
            }

            // 题卡详细信息
            ExamCard examCard = this.getById(id);

            StringJoiner dirPath = new StringJoiner("");
            dirPath = dirPath.add(rootPath).add(File.separator).add(examCard.getTitle()).add(File.separator);

            String htmlContent;
            // 通用模板
            if (CardTypeEnum.GENERIC.equals(examCard.getType())) {
                if (CardCreateMethodEnum.UPLOAD.equals(examCard.getCreateMethod())) {
                    htmlContent = createPdfUtil.resetHtmlTemplateBar(examCard.getHtmlContent());
                } else {
                    BasicCardRule basicCardRule = basicCardRuleService.getById(examCard.getCardRuleId());
                    htmlContent = createPdfUtil.replaceHtmlCard(examCard, basicCardRule);
                }
            } else {
                List<ExamTask> examTasks = examTaskService.getExamTaskByCourseCodeAndCardId(examCard.getSchoolId(), examCard.getCourseCode(), id);
                BasicCardRule basicCardRule = null;
                if (!CollectionUtils.isEmpty(examTasks)) {
                    basicCardRule = basicCardRuleService.getById(examTasks.get(0).getCardRuleId());
                }
                htmlContent = createPdfUtil.replaceHtmlCard(examCard, basicCardRule);
            }

            // html文件
            String cardHtmlPath = dirPath + examCard.getTitle() + SystemConstant.HTML_PREFIX;
            File htmlFile = new File(cardHtmlPath);
            if (!htmlFile.exists()) {
                htmlFile.getParentFile().mkdirs();
                htmlFile.createNewFile();
            }
            // 生成html文件
            FileCopyUtils.copy(htmlContent.getBytes(StandardCharsets.UTF_8), htmlFile);

            // pdf文件
            String cardPdfPath = dirPath + examCard.getTitle() + SystemConstant.PDF_PREFIX;
            File pdfFile = new File(cardPdfPath);
            if (!pdfFile.exists()) {
                pdfFile.getParentFile().mkdirs();
                pdfFile.createNewFile();
            }
            HtmlToPdfUtil.convert(cardHtmlPath, cardPdfPath, PageSizeEnum.A3);

            // jpg文件
            String jpgAttachmentIds = examCard.getJpgAttachment();
            if (StringUtils.isNotBlank(jpgAttachmentIds)) {
                List<JSONObject> jsonObjectList = JSONObject.parseArray(jpgAttachmentIds, JSONObject.class);
                for (JSONObject jsonObject : jsonObjectList) {
                    String jpgAttachmentId = jsonObject.get("attachmentId").toString();
                    String index = jsonObject.get("index").toString();
                    if (StringUtils.isNotBlank(jpgAttachmentId)) {
                        BasicAttachment attachment = basicAttachmentService.getById(jpgAttachmentId);
                        if (attachment != null) {
                            String fileName = examCard.getTitle() + SystemConstant.HYPHEN + index + attachment.getType();
                            teachcloudCommonService.copyFile(dirPath.toString(), fileName, attachment);
                        }
                    }
                }
            }

            // json文件
            String content = examCard.getContent();
            if (StringUtils.isNotBlank(content)) {
                String jsonPath = dirPath + examCard.getTitle() + SystemConstant.JSON_PREFIX;
                SystemConstant.createJsonFile(jsonPath, content);
            }

            String zipFileName = id + SystemConstant.ZIP_PREFIX;
            teachcloudCommonService.downloadFileAndZip(response, rootPath, rootPath, zipFileName);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(rootPath)) {
                ConvertUtil.delFolder(rootPath);
            }
        }
    }

    /**
     * 生成新的title
     *
     * @param examCard 题卡对象
     */
    private String checkTitle(ExamCard examCard, String courseCode) {
        String title;
        int i = 1;
        ExamCard newExamCard;
        do {
            title = examCard.getTitle() + "副本" + i;
            QueryWrapper<ExamCard> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamCard::getSchoolId, examCard.getSchoolId())
                    .eq(ExamCard::getCourseCode, courseCode)
                    .eq(ExamCard::getTitle, title);
            newExamCard = this.getOne(queryWrapper);
            i++;
        } while (newExamCard != null);
        return title;
    }

    /**
     * 数据验证
     *
     * @param examCardParams
     */
    private void validateCardData(ExamCardParams examCardParams) {
        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId();
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
        }

//        if (Objects.isNull(examCardParams.getMakeMethod())) {
//            throw ExceptionResultEnum.ERROR.exception("题卡制作方式不能为空");
//        }
        if (examCardParams.getStatus() == null) {
            throw ExceptionResultEnum.ERROR.exception("提交方式不能为空");
        }
        if (CardTypeEnum.CUSTOM.name().equals(examCardParams.getType().name())) {
//            if (examCardParams.getExamTaskId() == null) {
//                throw ExceptionResultEnum.ERROR.exception("命题任务ID不能为空");
//            }
            if (StringUtils.isBlank(examCardParams.getCourseCode())) {
                throw ExceptionResultEnum.ERROR.exception("课程代码不能为空");
            }
            if (StringUtils.isBlank(examCardParams.getCourseName())) {
                throw ExceptionResultEnum.ERROR.exception("课程名称不能为空");
            }
            if (MakeMethodEnum.SELECT.name().equals(examCardParams.getMakeMethod().name())) {
                if (examCardParams.getId() == null) {
                    throw ExceptionResultEnum.ERROR.exception("选择已有题卡时，题卡ID不能为空");
                }
                // 当前选择题卡是否已绑定（不再校验是否使用  2023-06-20）
            } else if (MakeMethodEnum.SELF.name().equals(examCardParams.getMakeMethod().name())) {
                if (StringUtils.isBlank(examCardParams.getTitle())) {
                    throw ExceptionResultEnum.ERROR.exception("题卡标题不能为空");
                }
                if (StringUtils.isBlank(examCardParams.getContent())) {
                    throw ExceptionResultEnum.ERROR.exception("题卡内容不能为空");
                }
                if (CardStatusEnum.SUBMIT.name().equals(examCardParams.getStatus().name())) {
                    if (StringUtils.isBlank(examCardParams.getHtmlContent())) {
                        throw ExceptionResultEnum.ERROR.exception("提交题卡时，html内容不能为空");
                    }
                }
            } else {
                throw ExceptionResultEnum.ERROR.exception("不支持的题卡制作方式");
            }
        }
    }
}
