package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamDetailCourseDto;
import com.qmth.distributed.print.business.bean.dto.ExamDetailCourseFieldsDto;
import com.qmth.distributed.print.business.bean.examRule.CodeName;
import com.qmth.distributed.print.business.bean.examRule.CodeNameEnable;
import com.qmth.distributed.print.business.bean.examRule.CodeNameEnableValue;
import com.qmth.distributed.print.business.bean.params.SingleExamStudentAddParam;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamDetailCourseMapper;
import com.qmth.distributed.print.business.service.ExamDetailCourseService;
import com.qmth.distributed.print.business.service.ExamStudentService;
import com.qmth.teachcloud.common.bean.result.TbTaskDetailResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考务数据（考场关联科目） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailCourseServiceImpl extends ServiceImpl<ExamDetailCourseMapper, ExamDetailCourse> implements ExamDetailCourseService {

    @Resource
    private ExamStudentService examStudentService;

    @Override
    public List<Map<String, Object>> listClientByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listClientByExamDetailIdAndStatus(examDetailId, ExamDetailStatusEnum.FINISH.name());
    }

    @Override
    public void updatePaperNumber(List<Long> examDetailCourseIds, String paperNumber, String relatePaperType) {
        UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetailCourse::getPaperNumber, paperNumber)
                .set(ExamDetailCourse::getPaperType, relatePaperType)
                .in(ExamDetailCourse::getId, examDetailCourseIds);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listByExamDetailIdAndStatus(examDetailId, ExamStatusEnum.SUBMIT.name());
    }

    @Override
    public List<BasicCourse> listCoursesByPrintPlanId(String param, List<Long> printPlanIdList) {
        return this.baseMapper.listCoursesByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList);
    }

    @Override
    public List<ExamDetailCourse> listByPrintPlanIdAndCourseCodeAndPaperNumber(Long schoolId, Long printPlanId, String courseCode, String paperNumber) {
        return this.baseMapper.listByPrintPlanIdAndCourseCodeAndPaperNumber(schoolId, printPlanId, courseCode, paperNumber);
    }

    @Override
    public List<ExamDetailCourse> listByCourseCodeAndPaperNumber(Long schoolId, Long examId, String courseCode, String paperNumber) {
        return this.baseMapper.listByAndCourseCodeAndPaperNumber(schoolId, examId, courseCode, paperNumber);
    }

    @Override
    public List<ExamDetailCourseDto> listByPrintPlanIdAndExamTaskId(List<Long> printPlanIds, Long examTaskId, String paperType) {
        return this.baseMapper.listByPrintPlanIdAndExamTaskId(printPlanIds, examTaskId, paperType);
    }

    @Override
    public List<ExamDetailCourse> listExamDetailByExamIdAndPaperNumber(Long schoolId, Long examId, String paperNumber) {
        return this.baseMapper.listExamDetailByExamIdAndPaperNumber(schoolId, examId, paperNumber);
    }

    @Override
    public TbTaskDetailResult getByExamDetailId(Long examDetailId) {
        return this.baseMapper.getByExamDetailId(examDetailId);
    }

    @Override
    public List<String> listPaperNumberByPrintPlanId(String param, List<Long> printPlanIdList, Set<Long> orgIds) {
        return this.baseMapper.listPaperNumberByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList, orgIds);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailId(Long examDetailId) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamDetailCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetailCourse::getSchoolId, schoolId)
                .eq(ExamDetailCourse::getExamDetailId, examDetailId);
        return this.list(queryWrapper);
    }

    @Override
    public List<ExamDetailCourseFieldsDto> listExamDetailPaperNumber(Long examDetailId) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamDetailCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetailCourse::getSchoolId, schoolId)
                .eq(ExamDetailCourse::getExamDetailId, examDetailId)
                .orderByAsc(ExamDetailCourse::getCourseCode);
        List<ExamDetailCourse> examDetailCourseList = this.list(queryWrapper);
        List<ExamDetailCourseFieldsDto> examDetailCourseFieldsDtoList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(examDetailCourseList)) {
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                ExamDetailCourseFieldsDto examDetailCourseFieldsDto = new ExamDetailCourseFieldsDto();
                examDetailCourseFieldsDto.setExamDetailCourse(examDetailCourse);
                // 查询一条考生数据，拿到使用的必填字段和扩展字段
                ExamStudent examStudent = examStudentService.listByExamDetailCourseIdLimit1(schoolId, examDetailCourse.getId());
                List<CodeName> requireds = new ArrayList<>();
                List<CodeName> extendes = new ArrayList<>();
                if (examStudent != null) {
                    List<CodeNameEnableValue> studentRequiredFieldList = JSON.parseArray(examStudent.getRequiredFields(), CodeNameEnableValue.class);
                    if (CollectionUtils.isNotEmpty(studentRequiredFieldList)) {
                        requireds = studentRequiredFieldList.stream().map(m -> {
                            CodeName codeName = new CodeName();
                            codeName.setCode(m.getCode());
                            codeName.setName(m.getName());
                            return codeName;
                        }).collect(Collectors.toList());
                    }
                    List<CodeNameEnableValue> studentExtendFieldList = JSON.parseArray(examStudent.getExtendFields(), CodeNameEnableValue.class);
                    if (CollectionUtils.isNotEmpty(studentExtendFieldList)) {
                        extendes = studentExtendFieldList.stream().map(m -> {
                            CodeName codeName = new CodeName();
                            codeName.setCode(m.getCode());
                            codeName.setName(m.getName());
                            return codeName;
                        }).collect(Collectors.toList());
                    }
                }
                examDetailCourseFieldsDto.setRequiredFields(requireds);
                examDetailCourseFieldsDto.setExtendFields(extendes);
                examDetailCourseFieldsDtoList.add(examDetailCourseFieldsDto);
            }
        }
        return examDetailCourseFieldsDtoList;
    }

    @Override
    public boolean addExamStudent(SingleExamStudentAddParam singleExamStudentAddParam) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        ExamStudent examStudent = new ExamStudent();

        try {
            Field[] examinationImportDtoFields = examStudent.getClass().getDeclaredFields();
            for (CodeNameEnableValue requiredField : singleExamStudentAddParam.getRequiredFields()) {
                for (Field examinationImportDtoField : examinationImportDtoFields) {
                    if(!examinationImportDtoField.getName().equals(requiredField.getCode())){
                        continue;
                    }
                    examinationImportDtoField.setAccessible(true);
                    examinationImportDtoField.set(requiredField.getCode(), requiredField.getValue());
                }
            }
        } catch (IllegalAccessException e) {
            throw ExceptionResultEnum.ERROR.exception("必选字段赋值失败");
        }

        examStudent.setId(SystemConstant.getDbUuid());
        examStudent.setSchoolId(schoolId);
        examStudent.setExamId(singleExamStudentAddParam.getExamId());
        examStudent.setExamDetailCourseId(singleExamStudentAddParam.getExamDetailCourse().getId());
        examStudent.setPaperNumber(singleExamStudentAddParam.getExamDetailCourse().getPaperNumber());
        examStudent.setCoursePaperId(singleExamStudentAddParam.getExamDetailCourse().getCoursePaperId());
        examStudent.setExtendFields(JSON.toJSONString(singleExamStudentAddParam.getExtendFields()));
        examStudent.setCreateId(sysUser.getId());
        examStudent.setCreateTime(System.currentTimeMillis());
        examStudentService.save(examStudent);

        // todo 往exam_student表插入扫描数据
        return false;
    }
}
