package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.ExamPaperStructure;
import com.qmth.distributed.print.business.entity.TBSyncTask;
import com.qmth.distributed.print.business.mapper.TBSyncTaskMapper;
import com.qmth.distributed.print.business.service.ExamPaperStructureService;
import com.qmth.distributed.print.business.service.GradeBatchPaperService;
import com.qmth.distributed.print.business.service.GradeBatchService;
import com.qmth.distributed.print.business.service.TBSyncTaskService;
import com.qmth.distributed.print.business.templete.execute.AsyncTeachCloudReportService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.result.SyncListResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PushTypeEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Date: 2021/5/20.
 */
@Service
public class TBSyncTaskServiceImpl extends ServiceImpl<TBSyncTaskMapper, TBSyncTask> implements TBSyncTaskService {

    @Resource
    ExamPaperStructureService examPaperStructureService;

    @Resource
    AsyncTeachCloudReportService asyncTeachCloudReportService;

    @Resource
    GradeBatchService gradeBatchService;

    @Resource
    GradeBatchPaperService gradeBatchPaperService;

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Transactional
    @Override
    public TBSyncTask saveTask(Long schoolId, Long objectId, PushTypeEnum type, String remark) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBSyncTask::getSchoolId, schoolId)
                .eq(TBSyncTask::getObjectId, objectId)
                .eq(TBSyncTask::getType, type);
        TBSyncTask tbSyncTask = this.getOne(queryWrapper);
        Long id = SystemConstant.getDbUuid();
        if (tbSyncTask != null) {
            if (!TaskStatusEnum.FINISH.equals(tbSyncTask.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("有任务未结束");
            }
            id = tbSyncTask.getId();
            this.removeById(id);
        }
        tbSyncTask = new TBSyncTask();
        tbSyncTask.setId(id);
        tbSyncTask.setSchoolId(schoolId);
        tbSyncTask.setObjectId(objectId);
        tbSyncTask.setType(type);
        tbSyncTask.setRemark(remark);
        tbSyncTask.setStatus(TaskStatusEnum.INIT);
        tbSyncTask.setCreateId(sysUser.getId());
        tbSyncTask.setCreateTime(System.currentTimeMillis());
        this.save(tbSyncTask);
        return tbSyncTask;
    }

    @Transactional
    @Override
    public TBSyncTask saveTaskStructure(Long schoolId, ExamPaperStructure examPaperStructure, Long objectId, PushTypeEnum type, String remark) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBSyncTask::getSchoolId, schoolId)
                .eq(TBSyncTask::getObjectId, objectId)
                .eq(TBSyncTask::getType, type);
        TBSyncTask tbSyncTask = this.getOne(queryWrapper);
        Long id = SystemConstant.getDbUuid();
        if (tbSyncTask != null) {
            if (!TaskStatusEnum.FINISH.equals(tbSyncTask.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("有任务未结束");
            }
            id = tbSyncTask.getId();
            this.removeById(id);
        }
        tbSyncTask = new TBSyncTask();
        tbSyncTask.setId(id);
        tbSyncTask.setSchoolId(schoolId);
        tbSyncTask.setSemesterId(examPaperStructure.getSemesterId());
        tbSyncTask.setExamId(examPaperStructure.getExamId());
        tbSyncTask.setCourseCode(examPaperStructure.getCourseCode());
        tbSyncTask.setCourseName(examPaperStructure.getCourseName());
        tbSyncTask.setPaperNumber(examPaperStructure.getPaperNumber());
        tbSyncTask.setObjectId(objectId);
        tbSyncTask.setType(type);
        tbSyncTask.setRemark(remark);
        tbSyncTask.setStatus(TaskStatusEnum.INIT);
        tbSyncTask.setCreateId(sysUser.getId());
        tbSyncTask.setCreateTime(System.currentTimeMillis());
        this.save(tbSyncTask);
        return tbSyncTask;
    }

    @Transactional
    @Override
    public TBSyncTask saveTaskBaseData(Long schoolId, Long semesterId, Long examId, String courseCode, String courseName, String paperNumber, Long objectId, String orgCode, String thirdRelateName, String examTime, PushTypeEnum type, String remark) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBSyncTask::getSchoolId, schoolId)
                .eq(TBSyncTask::getObjectId, objectId)
                .eq(TBSyncTask::getType, type);
        if (StringUtils.isNotBlank(orgCode)) {
            queryWrapper.lambda().eq(TBSyncTask::getOrgCode, orgCode);
        } else {
            queryWrapper.lambda().isNull(TBSyncTask::getOrgCode);
        }
        TBSyncTask tbSyncTask = this.getOne(queryWrapper);
        Long id = SystemConstant.getDbUuid();
        if (tbSyncTask != null) {
            if (!TaskStatusEnum.FINISH.equals(tbSyncTask.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("有任务未结束");
            }
            id = tbSyncTask.getId();
            this.removeById(id);
        }
        tbSyncTask = new TBSyncTask();
        tbSyncTask.setId(id);
        tbSyncTask.setSchoolId(schoolId);
        tbSyncTask.setSemesterId(semesterId);
        tbSyncTask.setExamId(examId);
        tbSyncTask.setCourseCode(courseCode);
        tbSyncTask.setCourseName(courseName);
        tbSyncTask.setPaperNumber(paperNumber);
        tbSyncTask.setObjectId(objectId);
        tbSyncTask.setOrgCode(orgCode);
        tbSyncTask.setThirdRelateName(thirdRelateName);
        tbSyncTask.setExamTime(examTime);
        tbSyncTask.setType(type);
        tbSyncTask.setRemark(remark);
        tbSyncTask.setStatus(TaskStatusEnum.INIT);
        tbSyncTask.setCreateId(sysUser.getId());
        tbSyncTask.setCreateTime(System.currentTimeMillis());
        this.save(tbSyncTask);
        return tbSyncTask;
    }

    @Override
    public void updateStatusAndResultById(Long id, Long thirdRelateId, TaskStatusEnum status, TaskResultEnum result, String errorMessage) {
        UpdateWrapper<TBSyncTask> tbSyncTaskUpdateWrapper = new UpdateWrapper<>();
        tbSyncTaskUpdateWrapper.lambda().set(TBSyncTask::getStatus, status)
                .set(TBSyncTask::getThirdRelateId, thirdRelateId)
                .set(TBSyncTask::getResult, result)
                .set(TBSyncTask::getErrorMessage, errorMessage)
                .eq(TBSyncTask::getId, id);
        this.update(tbSyncTaskUpdateWrapper);
    }

    @Override
    public void updateStatusAndResultById(Long id, Long thirdRelateId, TaskStatusEnum status, TaskResultEnum result, String errorMessage, String remark) {
        UpdateWrapper<TBSyncTask> tbSyncTaskUpdateWrapper = new UpdateWrapper<>();
        tbSyncTaskUpdateWrapper.lambda().set(TBSyncTask::getStatus, status)
                .set(TBSyncTask::getThirdRelateId, thirdRelateId)
                .set(TBSyncTask::getResult, result)
                .set(TBSyncTask::getErrorMessage, errorMessage)
                .set(TBSyncTask::getRemark, remark)
                .eq(TBSyncTask::getId, id);
        this.update(tbSyncTaskUpdateWrapper);
    }

    @Transactional
    @Override
    public TBSyncTask savePushCommon(PushTypeEnum pushTypeEnum, Map map, SysUser sysUser) {
        SysUser requestUser = Objects.nonNull(sysUser) ? sysUser : (SysUser) ServletUtil.getRequestUser();
        TBSyncTask tbSyncTask = new TBSyncTask(pushTypeEnum,
                TaskStatusEnum.INIT,
                requestUser.getId(),
                requestUser.getSchoolId());

        map.computeIfAbsent(SystemConstant.TB_SYNC_TASK, v -> tbSyncTask);
        map.computeIfAbsent(SystemConstant.USER, v -> requestUser);
        map.computeIfAbsent(SystemConstant.TB_SYNC_TASK_ID, v -> tbSyncTask.getId());
        return tbSyncTask;
    }

    @Transactional
    @Override
    public TBSyncTask savePushCommon(PushTypeEnum pushTypeEnum, Map map, SysUser sysUser, Long objectId) {
        SysUser requestUser = Objects.nonNull(sysUser) ? sysUser : (SysUser) ServletUtil.getRequestUser();
        TBSyncTask tbSyncTask = new TBSyncTask(pushTypeEnum,
                TaskStatusEnum.INIT,
                requestUser.getId(),
                requestUser.getSchoolId(),
                objectId);

        map.computeIfAbsent(SystemConstant.TB_SYNC_TASK, v -> tbSyncTask);
        map.computeIfAbsent(SystemConstant.USER, v -> requestUser);
        map.computeIfAbsent(SystemConstant.TB_SYNC_TASK_ID, v -> tbSyncTask.getId());
        return tbSyncTask;
    }

    @Override
    public IPage<SyncListResult> query(Long schoolId, Long semesterId, Long examId, String courseCode, String paperNumber, TaskStatusEnum status, PushTypeEnum type, TaskResultEnum result, int pageNumber, int pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());

        IPage<SyncListResult> page = this.baseMapper.query(new Page<>(pageNumber, pageSize), schoolId, semesterId, examId, courseCode, paperNumber, status, type, result, dpr);
        List<SyncListResult> list = page.getRecords();
        for (SyncListResult syncListResult : list) {
            syncListResult.setStatus(Objects.nonNull(syncListResult.getStatus()) ? TaskStatusEnum.valueOf(syncListResult.getStatus()).getTitle() : null);
            syncListResult.setType(Objects.nonNull(syncListResult.getType()) ? PushTypeEnum.valueOf(syncListResult.getType()).getTitle() : null);
            syncListResult.setResultStr(Objects.nonNull(syncListResult.getResult()) ? TaskResultEnum.valueOf(syncListResult.getResult()).getTitle() : null);
        }
        return page;
    }

    @Override
    public void resync(Long id) throws IllegalAccessException {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        TBSyncTask tbSyncTask = this.getById(id);
        if (tbSyncTask == null) {
            throw ExceptionResultEnum.ERROR.exception("任务不存在");
        }
        if (!TaskStatusEnum.FINISH.equals(tbSyncTask.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("任务同步中");
        }
        if (TaskResultEnum.SUCCESS.equals(tbSyncTask.getResult())) {
            throw ExceptionResultEnum.ERROR.exception("任务同步成功");
        }
        switch (tbSyncTask.getType()) {
            case GRADE_BATCH_PUSH:
                asyncTeachCloudReportService.syncGradeBatch(gradeBatchService.getById(tbSyncTask.getObjectId()));
                break;
            case PUBLISH:
                asyncTeachCloudReportService.publish(gradeBatchPaperService.getById(tbSyncTask.getObjectId()));
                break;
            case CALCULATE:
                asyncTeachCloudReportService.startCalc(gradeBatchService.getById(tbSyncTask.getObjectId()));
            default:
                break;
        }
    }
}
