package com.qmth.distributed.print.api;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.dto.ExamPaperStructurePageDto;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.entity.ExamPaperStructure;
import com.qmth.distributed.print.business.service.ExamPaperStructureService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

/**
 * <p>
 * 试卷结构 前端控制器
 * </p>
 *
 * @author xf
 */
@Api(tags = "试卷结构Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_EXAM + "/structure")
public class ExamPaperStructureController {

    @Resource
    private ExamPaperStructureService examPaperStructureService;


    /**
     * 查询
     *
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(@ApiParam(value = "学期ID", required = false) @RequestParam(required = false) Long semesterId,
                       @ApiParam(value = "考试ID", required = false) @RequestParam(required = false) Long examId,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<ExamPaperStructurePageDto> examPaperStructureIPage = examPaperStructureService.listByPropositionTeacherId(semesterId, examId, pageNumber, pageSize, null);
        return ResultUtil.ok(examPaperStructureIPage);
    }

    /**
     * 查询最新状态
     *
     * @param id 评卷参数ID
     */
    @ApiOperation(value = "查询最新状态")
    @RequestMapping(value = "/get_structure_status", method = RequestMethod.POST)
    public Result getStructureStatus(@RequestParam("id") String id) {
        ExamPaperStructure examPaperStructure = examPaperStructureService.getById(id);
        return ResultUtil.ok(examPaperStructure == null ? null : examPaperStructure.getStatus(), null);
    }


    /**
     * 上传
     *
     * @param md5   文件md5
     * @param files 文件数组
     * @return Result
     */
    @ApiOperation(value = "上传试卷结构、标答")
    @RequestMapping(value = "/upload", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    public Result upload(@RequestParam("examPaperStructure") String examPaperStructure,
                         @RequestParam("md5") String md5,
                         @RequestParam("keys") String keys,
                         @RequestParam("files") MultipartFile[] files) {
        ExamPaperStructure examPaper = examPaperStructureService.upload(examPaperStructure, md5, keys, files);
        return ResultUtil.ok(String.valueOf(examPaper.getId()), null);
    }

    @ApiOperation(value = "设置评卷参数-提交")
    @RequestMapping(value = "/submit", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "更新成功", response = EditResult.class)})
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    public Result submitExamPaperParams(@RequestBody String evaluationParameters) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        ExamPaperStructure examPaperStructure = examPaperStructureService.submitExamPaperParams(evaluationParameters, sysUser);
        return ResultUtil.ok(String.valueOf(examPaperStructure.getId()), null);
    }

    /**
     * 上传标答文件
     *
     * @param md5  文件md5
     * @param file 文件数组
     * @return Result
     */
    @ApiOperation(value = "上传试卷标答文件")
    @RequestMapping(value = "/upload_answer", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    public Result uploadAnswer(@RequestParam("id") String id,
                               @RequestParam("md5") String md5,
                               @RequestParam("file") MultipartFile file) {
        ExamPaperStructure examPaperStructure = examPaperStructureService.uploadAnswer(id, md5, file);
        return ResultUtil.ok(true);
    }

    /**
     * 设置科组长
     *
     * @param data data
     * @return Result
     */
    @ApiOperation(value = "设置科组长")
    @RequestMapping(value = "/bind_mark_leader", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    public Result bindMarkLeader(@RequestBody String data) {
        ExamPaperStructure examPaperStructure = examPaperStructureService.bindMarkLeader(data);
        return ResultUtil.ok(true);
    }

    /**
     * 绑定评卷员、分班阅
     *
     * @param data
     * @return
     */
    @ApiOperation(value = "绑定评卷员、分班阅")
    @RequestMapping(value = "/bind_marker", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "更新成功", response = EditResult.class)})
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    public Result bindMarker(@RequestBody String data) {
        ExamPaperStructure examPaperStructure = examPaperStructureService.saveMarkerAndClass(data);
        return ResultUtil.ok(true);
    }

    /**
     * 保存客观题答案
     *
     * @param body body
     */
    @ApiOperation(value = "保存客观题答案")
    @RequestMapping(value = "/update_objective_answer", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    public Result updateObjectiveAnswer(@RequestBody String body) {
        JSONObject object = JSONObject.parseObject(body, JSONObject.class);
        Long id = object.getLong("id");
        String objectiveStructure = object.getString("objectiveStructure");
        ExamPaperStructure examPaperStructure = examPaperStructureService.updateObjectiveAnswer(id, objectiveStructure);
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "评卷参数设置-获取题卡图片")
    @RequestMapping(value = "/find_jpg_file", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result findCardJpgFileByPaperNumber(@ApiParam(value = "考试id", required = true) @RequestParam String examId,
                                               @ApiParam(value = "课程代码", required = true) @RequestParam String courseCode,
                                               @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                                               @ApiParam(value = "试卷类型", required = true) @RequestParam String paperType) {
        return ResultUtil.ok(examPaperStructureService.findCardJpgFileByPaperNumber(SystemConstant.convertIdToLong(examId), courseCode, paperNumber, paperType));
    }

    /**
     * 查询教学秘书
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list_secretary", method = RequestMethod.POST)
    public Result listSecretary() {
        return ResultUtil.ok(examPaperStructureService.listSecretary());
    }

    /**
     * 转办/打回
     */
    @ApiOperation(value = "转办/打回")
    @RequestMapping(value = "/transfer", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    public Result transfer(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                           @ApiParam(value = "课程代码", required = true) @RequestParam String courseCode,
                           @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                           @ApiParam(value = "用户ID", required = false) @RequestParam(required = false) Long transferId) {
        return ResultUtil.ok(examPaperStructureService.transfer(examId, courseCode, paperNumber, transferId));
    }

    /**
     * 查询绑定班级
     */
    @ApiOperation(value = "查询绑定班级")
    @RequestMapping(value = "/list_user_class", method = RequestMethod.POST)
    public Result transfer(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                           @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        return ResultUtil.ok(examPaperStructureService.listUserClass(examId, paperNumber));
    }
}

