package com.qmth.distributed.print.api.mark;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkUserGroupProgressDto;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.service.MarkUserGroupService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;

/**
 * <p>
 * 评卷员 前端控制器
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Api(tags = "评卷-评卷员")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_MARK + "/marker")
public class MarkUserGroupController {

    @Resource
    private MarkUserGroupService markUserGroupService;

    /**
     * 评卷员管理列表查询
     */
    @ApiOperation(value = "评卷员管理列表查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result classSummary(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                               @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                               @ApiParam(value = "分组号") @RequestParam(required = false) Integer groupNumber,
                               @ApiParam(value = "评卷员") @RequestParam(required = false) String loginName,
                               @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                               @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<MarkUserGroupProgressDto> markGroupClassProgressDtoIPage = markUserGroupService.pageGroupUserByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber, loginName, pageNumber, pageSize);
        return ResultUtil.ok(markGroupClassProgressDtoIPage);
    }

    /**
     * 绑定评卷员
     */
    @ApiOperation(value = "绑定评卷员")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    public Result addMarker(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                            @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                            @ApiParam(value = "分组号", required = true) @RequestParam Integer groupNumber,
                            @ApiParam(value = "评卷员ID集合", required = true) @RequestParam List<Long> userIds) {
        markUserGroupService.addMarkUserGroup(examId, paperNumber, groupNumber, userIds);
        return ResultUtil.ok(true);
    }

    /**
     * 重置评卷员
     */
    @ApiOperation(value = "重置评卷员")
    @RequestMapping(value = "/reset", method = RequestMethod.POST)
    public Result resetMarker(@ApiParam(value = "考试ID", required = true) @RequestParam Long markUserGroupId) {
        boolean resetMarker = markUserGroupService.resetMarker(markUserGroupId);
        if (resetMarker) {
            return ResultUtil.ok(true);
        } else {
            return ResultUtil.error(ExceptionResultEnum.ERROR.getCode(), "评卷员正在重置");
        }
    }

    /**
     * 解绑评卷员
     */
    @ApiOperation(value = "解绑评卷员")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public Result deleteMarker(@ApiParam(value = "考试ID", required = true) @RequestParam Long markUserGroupId) {
        markUserGroupService.deleteMarker(markUserGroupId);
        return ResultUtil.ok(true);
    }

    /**
     * 回收评卷员正在评卷的试卷
     */
    @ApiOperation(value = "回收正在评卷的试卷")
    @RequestMapping(value = "/release", method = RequestMethod.POST)
    public Result releaseMarker(@ApiParam(value = "评卷员ID集合", required = true) @RequestParam List<Long> markUserGroupIds) {
        markUserGroupService.releaseMarker(markUserGroupIds);
        return ResultUtil.ok(true);
    }

    /**
     * 设置评卷任务
     */
    @ApiOperation(value = "设置评卷任务")
    @RequestMapping(value = "/setTaskCount", method = RequestMethod.POST)
    public Result setTaskCount(@ApiParam(value = "评卷员ID集合", required = true) @RequestParam List<Long> markUserGroupIds,
                               @ApiParam(value = "评卷任务数", required = true) @RequestParam Integer taskCount) {
        markUserGroupService.setTaskCount(markUserGroupIds, taskCount);
        return ResultUtil.ok(true);
    }

}
