package com.qmth.distributed.print.api.mark;


import java.util.concurrent.TimeUnit;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.enums.Platform;
import com.qmth.distributed.print.config.SysProperty;
import com.qmth.teachcloud.common.bean.auth.ExpireTimeBean;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.TBSession;
import com.qmth.teachcloud.common.service.BasicSemesterService;
import com.qmth.teachcloud.common.util.AuthUtil;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.bean.sys.ActiveTimeVo;
import com.qmth.teachcloud.mark.bean.sys.ApiStatusVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 * 扫描系统相关 前端控制器
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Api(tags = "扫描-登录、登出、服务器时间")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_SCAN)
public class ScanController {

    @Resource
    BasicSemesterService basicSemesterService;
    @Resource
    private SysProperty sysProperty;
    @Resource
    private RedisUtil redisUtil;
    
    @Aac(auth = false)
    @ApiOperation(value = "服务端状态")
    @RequestMapping("server/status")
    public ApiStatusVo status(HttpServletRequest request) {
        ApiStatusVo vo = new ApiStatusVo();
        vo.setFileUriPrefix(sysProperty.getFileUriPrefix());
        vo.setVersion(sysProperty.getVersion());
        vo.setTime(System.currentTimeMillis());
        return vo;
    }
    
    @ApiOperation(value = "心跳接口")
    @PostMapping("server/heartbeat")
    public ActiveTimeVo heartbeat() {
        Platform platform = ServletUtil.getRequestPlatform();
        ExpireTimeBean expireTime = AuthUtil.getExpireTime(platform);
        TBSession ts=(TBSession)ServletUtil.getRequestSession();
        redisUtil.expire(ts.getId(), expireTime.getExpireSeconds(),TimeUnit.SECONDS);
        ActiveTimeVo vo = new ActiveTimeVo();
        vo.setActiveTime(System.currentTimeMillis());
        return vo;
    }
}
