package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.SchoolDto;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.mapper.BasicSchoolMapper;
import com.qmth.teachcloud.common.service.BasicSchoolService;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.Base64Util;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.Base64;
import java.util.List;
import java.util.Queue;

/**
 * <p>
 * 学校表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicSchoolServiceImpl extends ServiceImpl<BasicSchoolMapper, BasicSchool> implements BasicSchoolService {

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    FileStoreUtil fileStoreUtil;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    SysUserService sysUserService;

    @Override
    public List<SchoolDto> listSchool() {
        List<SchoolDto> schoolDtos = this.baseMapper.listSchool();
        for (SchoolDto schoolDto : schoolDtos) {
            if (StringUtils.isNotBlank(schoolDto.getInitPassword())) {
                schoolDto.setInitPassword(new String(Base64Util.decode(schoolDto.getInitPassword())));
            }
        }
        return schoolDtos;
    }

    @Override
    public boolean updateSchool(Long id, String code, String name, String initPassword, MultipartFile logo, String logoMd5) {
        BasicSchool basicSchool = this.getById(id);
        basicSchool.setCode(code);
        basicSchool.setName(name);

        if (logo != null) {
            try {
                String toBase64 = "data:image/png;base64," + new String(Base64.getEncoder().encode(logo.getBytes()), SystemConstant.CHARSET);
                basicSchool.setLogo(toBase64);
            } catch (Exception e) {
                log.error(SystemConstant.LOG_ERROR, e);
            }
        }

        // 修改密码时，同步更新上一次初始密码的用户密码
        String oldInitPassword = Base64Util.encode(SystemConstant.OLD_INIT_PASSWORD.getBytes());
        String currentInitPassword = basicSchool.getInitPassword();
        String newInitPassword = Base64Util.encode(initPassword.getBytes());
        if (StringUtils.isBlank(currentInitPassword) || !currentInitPassword.equals(initPassword)) {
            UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(SysUser::getPassword, newInitPassword)
                    .eq(SysUser::getSchoolId, id)
                    .and(q -> q.eq(SysUser::getPassword, oldInitPassword).or().eq(SysUser::getPassword, currentInitPassword));
            sysUserService.update(updateWrapper);
        }

        // 清除学校缓存
        commonCacheService.removeSchoolCache(basicSchool.getCode());
        commonCacheService.removeSchoolCache(basicSchool.getId());

        basicSchool.setInitPassword(newInitPassword);
        return this.updateById(basicSchool);
    }
}
