package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.params.BasicSemesterParams;
import com.qmth.teachcloud.common.bean.result.BasicSemesterResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSemester;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.mapper.BasicSemesterMapper;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.BasicSemesterService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 学期字典表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-10-28
 */
@Service
public class BasicSemesterServiceImpl extends ServiceImpl<BasicSemesterMapper, BasicSemester> implements BasicSemesterService {

    @Resource
    BasicSemesterService basicSemesterService;

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Override
    public IPage<BasicSemesterResult> basicSemesterPage(Long schoolId, int pageNumber, int pageSize) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.findBasicSemesterPage(new Page<>(pageNumber, pageSize), schoolId, dpr);
    }

    @Transactional
    @Override
    public Long saveBasicSemester(BasicSemesterParams basicSemesterParams) throws IllegalAccessException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return basicSemesterService.saveBasicSemesterCommon(basicSemesterParams, sysUser, sysUser.getSchoolId(), sysUser.getOrgId(), false);
    }


    @Override
    @Transactional
    public Long saveBasicSemesterCommon(BasicSemesterParams basicSemesterParams, SysUser requestUser, Long schoolId, Long orgId, boolean openApi) throws IllegalAccessException {
        SystemConstant.verifyDBFields(basicSemesterParams, basicSemesterParams.getClass());
        Long userId = null;
        if (Objects.nonNull(requestUser)) {
            userId = requestUser.getId();
        }

        Long id = basicSemesterParams.getId();
        String semesterName = basicSemesterParams.getSemesterName();

        // 校验学期唯一性
        BasicSemester checkName = this.getOne(new QueryWrapper<BasicSemester>().lambda()
                .eq(BasicSemester::getEnable, true)
                .eq(BasicSemester::getSchoolId, schoolId)
                .eq(BasicSemester::getName, semesterName));
        if (Objects.nonNull(checkName) && !checkName.getId().equals(id)) {
            if (openApi) {
                // 如果是开放接口（分布式印刷推学期给教研分析），就算没有教研分析学期id，只要名称一致就视为编辑
                id = checkName.getId();
            } else {
                throw ExceptionResultEnum.ERROR.exception("学期名称[" + semesterName + "]已使用");
            }
        }
        if (SystemConstant.longNotNull(id)) {
            // 编辑
            UpdateWrapper<BasicSemester> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(BasicSemester::getName, semesterName)
                    .eq(BasicSemester::getId, id);
            if (SystemConstant.longNotNull(userId)) {
                updateWrapper.lambda().set(BasicSemester::getUpdateId, userId);
            }
            this.update(updateWrapper);
        } else {
            // 新增
            id = SystemConstant.getDbUuid();
            BasicSemester basicSemester = new BasicSemester();
            basicSemester.setId(id);
            basicSemester.setEnable(true);
            basicSemester.setName(semesterName);
            basicSemester.setSchoolId(schoolId);
            basicSemester.setCode(null);
            basicSemester.setOrgId(orgId);
            if (SystemConstant.longNotNull(userId)) {
                basicSemester.setCreateId(userId);
            }
            this.save(basicSemester);
        }
        return id;
    }

    @Transactional
    @Override
    public boolean deleteBasicSemesterById(Long id) {
        if (this.baseMapper.countBasicExamBySemesterId(id) > 0) {
            throw ExceptionResultEnum.ERROR.exception("学期下已创建考试，不能删除");
        }
        return this.removeById(id);
    }

    @Override
    public List<BasicSemester> list(Long schoolId, Boolean enable) {
        QueryWrapper<BasicSemester> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicSemester::getSchoolId, schoolId).eq(BasicSemester::getEnable, true);
        if (Objects.nonNull(enable)) {
            queryWrapper.lambda().eq(BasicSemester::getEnable, true);
        }
        return this.list(queryWrapper);
    }

    @Override
    public BasicSemester selectByExamId(Long examId) {
        return this.baseMapper.selectByExamId(examId);
    }

    @Override
    public boolean enable(Long id, Boolean enable) {
        UpdateWrapper<BasicSemester> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicSemester::getEnable, enable).eq(BasicSemester::getId, id);
        return this.update(updateWrapper);
    }
}
