package com.qmth.teachcloud.mark.entity;

import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.enums.ScanStatus;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.mark.dto.mark.ScoreItem;
import com.qmth.teachcloud.mark.utils.BigDecimalUtils;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.List;

/**
 * <p>
 * 考试考生库
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@TableName("mark_student")
@ApiModel(value = "MarkStudent对象", description = "考试考生库")
public class MarkStudent implements Serializable {

    private static final long serialVersionUID = 1L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "主键(和exam_student表id一样)")
    @TableId(value = "id")
    private Long id;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "考试ID")
    private Long examId;

    @ApiModelProperty(value = "科目代码")
    private String courseCode;

    @ApiModelProperty(value = "科目名称")
    private String courseName;
    
    @ApiModelProperty(value = "试卷编号（交互）")
    private String coursePaperId;

    @ApiModelProperty(value = "试卷类型")
    private String paperNumber;

    @ApiModelProperty(value = "密号")
    private String secretNumber;

    @ApiModelProperty(value = "学号")
    private String studentCode;

    @ApiModelProperty(value = "姓名")
    private String studentName;

    @ApiModelProperty(value = "试卷袋编号")
    private String packageCode;

    @ApiModelProperty(value = "校区")
    private String examPlace;

    @ApiModelProperty(value = "考场")
    private String examRoom;

    @ApiModelProperty(value = "备注")
    private String remark;

    @ApiModelProperty(value = "扫描批次号")
    private String batchCode;

    @ApiModelProperty(value = "原图数量")
    private Integer sheetCount;

    @ApiModelProperty(value = "客观题识别结果")
    private String answers;

    @ApiModelProperty(value = "是否已上传")
    private Integer isUpload;

    @ApiModelProperty(value = "是否缺考")
    private Integer isAbsent;

    @ApiModelProperty(value = "是否人工指定缺考")
    private Integer isManualAbsent;

    @ApiModelProperty(value = "是否违纪")
    private Integer isBreach;

    @ApiModelProperty(value = "上传时间")
    private Long uploadTime;

    @ApiModelProperty(value = "检查时间")
    private Long checkTime;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "检查人ID")
    private Long checkUserId;

    @ApiModelProperty(value = "客观总分")
    private Double objectiveScore;

    @ApiModelProperty(value = "客观得分明细")
    private String objectiveScoreList;

    @ApiModelProperty(value = "主观题状态")
    private SubjectiveStatus subjectiveStatus;

    @ApiModelProperty(value = "主观总分")
    private Double subjectiveScore;

    @ApiModelProperty(value = "主观得分明细")
    private String subjectiveScoreList;

    @ApiModelProperty(value = "学院")
    private String college;

    @ApiModelProperty(value = "班级")
    private String className;

    @ApiModelProperty(value = "任课老师")
    private String teacher;

    @ApiModelProperty(value = "扫描状态")
    private ScanStatus scanStatus;
    //识别缺考
    private Boolean omrAbsent;
    //识别缺考检查
    private Boolean omrAbsentChecked;
    //是否有填涂
    private Boolean questionFilled;

    //是否人工绑定
    private Boolean assigned;

    // 是否标记异常
    private Boolean absentSuspect;

    // 是否有缺页
    private Boolean incomplete;

    //违纪码
    private String breachCode;

    private Integer cardNumber;

    public MarkStudent() {
    }

    public MarkStudent(Long id, Long examId, String courseCode, String courseName, String paperNumber, String studentCode, String studentName, String packageCode, String examPlace, String examRoom, String className) {
        this.id = id;
        this.examId = examId;
        this.courseCode = courseCode;
        this.courseName = courseName;
        this.paperNumber = paperNumber;
        this.studentCode = studentCode;
        this.studentName = studentName;
        this.packageCode = packageCode;
        this.examPlace = examPlace;
        this.examRoom = examRoom;
        this.className = className;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getSecretNumber() {
        return secretNumber;
    }

    public void setSecretNumber(String secretNumber) {
        this.secretNumber = secretNumber;
    }

    public String getStudentCode() {
        return studentCode;
    }

    public void setStudentCode(String studentCode) {
        this.studentCode = studentCode;
    }

    public String getStudentName() {
        return studentName;
    }

    public void setStudentName(String studentName) {
        this.studentName = studentName;
    }

    public String getPackageCode() {
        return packageCode;
    }

    public void setPackageCode(String packageCode) {
        this.packageCode = packageCode;
    }

    public String getExamPlace() {
        return examPlace;
    }

    public void setExamPlace(String examPlace) {
        this.examPlace = examPlace;
    }

    public String getExamRoom() {
        return examRoom;
    }

    public void setExamRoom(String examRoom) {
        this.examRoom = examRoom;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public String getBatchCode() {
        return batchCode;
    }

    public void setBatchCode(String batchCode) {
        this.batchCode = batchCode;
    }

    public Integer getSheetCount() {
        return sheetCount;
    }

    public void setSheetCount(Integer sheetCount) {
        this.sheetCount = sheetCount;
    }

    public String getAnswers() {
        return answers;
    }

    public void setAnswers(String answers) {
        this.answers = answers;
    }

    public Integer getIsUpload() {
        return isUpload;
    }

    public void setIsUpload(Integer isUpload) {
        this.isUpload = isUpload;
    }

    public Integer getIsAbsent() {
        return isAbsent;
    }

    public void setIsAbsent(Integer isAbsent) {
        this.isAbsent = isAbsent;
    }

    public Integer getIsManualAbsent() {
        return isManualAbsent;
    }

    public void setIsManualAbsent(Integer isManualAbsent) {
        this.isManualAbsent = isManualAbsent;
    }

    public Integer getIsBreach() {
        return isBreach;
    }

    public void setIsBreach(Integer isBreach) {
        this.isBreach = isBreach;
    }

    public Long getUploadTime() {
        return uploadTime;
    }

    public void setUploadTime(Long uploadTime) {
        this.uploadTime = uploadTime;
    }

    public Long getCheckTime() {
        return checkTime;
    }

    public void setCheckTime(Long checkTime) {
        this.checkTime = checkTime;
    }

    public Long getCheckUserId() {
        return checkUserId;
    }

    public void setCheckUserId(Long checkUserId) {
        this.checkUserId = checkUserId;
    }

    public Double getObjectiveScore() {
        return objectiveScore;
    }

    public void setObjectiveScore(Double objectiveScore) {
        this.objectiveScore = objectiveScore;
    }

    public String getObjectiveScoreList() {
        return objectiveScoreList;
    }

    public void setObjectiveScoreList(String objectiveScoreList) {
        this.objectiveScoreList = objectiveScoreList;
    }

    public SubjectiveStatus getSubjectiveStatus() {
        return subjectiveStatus;
    }

    public void setSubjectiveStatus(SubjectiveStatus subjectiveStatus) {
        this.subjectiveStatus = subjectiveStatus;
    }

    public Double getSubjectiveScore() {
        return subjectiveScore;
    }

    public void setSubjectiveScore(Double subjectiveScore) {
        this.subjectiveScore = subjectiveScore;
    }

    public String getSubjectiveScoreList() {
        return subjectiveScoreList;
    }

    public void setSubjectiveScoreList(String subjectiveScoreList) {
        this.subjectiveScoreList = subjectiveScoreList;
    }

    public String getCollege() {
        return college;
    }

    public void setCollege(String college) {
        this.college = college;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public String getTeacher() {
        return teacher;
    }

    public void setTeacher(String teacher) {
        this.teacher = teacher;
    }

    public ScanStatus getScanStatus() {
        return scanStatus;
    }

    public void setScanStatus(ScanStatus scanStatus) {
        this.scanStatus = scanStatus;
    }

    public Boolean getQuestionFilled() {
        return questionFilled;
    }

    public void setQuestionFilled(Boolean questionFilled) {
        this.questionFilled = questionFilled;
    }

    public Boolean getAssigned() {
        return assigned;
    }

    public void setAssigned(Boolean assigned) {
        this.assigned = assigned;
    }

    public Boolean getAbsentSuspect() {
        return absentSuspect;
    }

    public void setAbsentSuspect(Boolean absentSuspect) {
        this.absentSuspect = absentSuspect;
    }

    public Boolean getIncomplete() {
        return incomplete;
    }

    public void setIncomplete(Boolean incomplete) {
        this.incomplete = incomplete;
    }

    public String getBreachCode() {
        return breachCode;
    }

    public void setBreachCode(String breachCode) {
        this.breachCode = breachCode;
    }

    public Boolean getOmrAbsent() {
        return omrAbsent;
    }

    public void setOmrAbsent(Boolean omrAbsent) {
        this.omrAbsent = omrAbsent;
    }

    public Integer getCardNumber() {
        return cardNumber;
    }

    public void setCardNumber(Integer cardNumber) {
        this.cardNumber = cardNumber;
    }

    public static String buildScoreList(List<ScoreItem> scoreList) {
        if (scoreList != null) {
            return StringUtils.join(scoreList, ";");
        } else {
            return null;
        }
    }
    
    

    public Boolean getOmrAbsentChecked() {
		return omrAbsentChecked;
	}

	public void setOmrAbsentChecked(Boolean omrAbsentChecked) {
		this.omrAbsentChecked = omrAbsentChecked;
	}

	public String getCoursePaperId() {
		return coursePaperId;
	}

	public void setCoursePaperId(String coursePaperId) {
		this.coursePaperId = coursePaperId;
	}

    public double getTotalScore() {
        double score = 0;
        if (objectiveScore != null) {
            score = BigDecimalUtils.add(score, objectiveScore);
        }
        if (subjectiveScore != null) {
            score = BigDecimalUtils.add(score, subjectiveScore);
        }
        return score;
    }

	@Override
    public String toString() {
        return "MarkStudent{" +
                "id=" + id +
                ", examId=" + examId +
                ", courseCode=" + courseCode +
                ", courseName=" + courseName +
                ", paperNumber=" + paperNumber +
                ", secretNumber=" + secretNumber +
                ", studentCode=" + studentCode +
                ", studentName=" + studentName +
                ", packageCode=" + packageCode +
                ", examPlace=" + examPlace +
                ", examRoom=" + examRoom +
                ", remark=" + remark +
                ", batchCode=" + batchCode +
                ", sheetCount=" + sheetCount +
                ", answers=" + answers +
                ", isUpload=" + isUpload +
                ", isAbsent=" + isAbsent +
                ", isManualAbsent=" + isManualAbsent +
                ", isBreach=" + isBreach +
                ", uploadTime=" + uploadTime +
                ", checkTime=" + checkTime +
                ", checkUserId=" + checkUserId +
                ", objectiveScore=" + objectiveScore +
                ", objectiveScoreList=" + objectiveScoreList +
                ", subjectiveStatus=" + subjectiveStatus +
                ", subjectiveScore=" + subjectiveScore +
                ", subjectiveScoreList=" + subjectiveScoreList +
                ", college=" + college +
                ", className=" + className +
                ", teacher=" + teacher +
                ", scanStatus=" + scanStatus +
                ", questionFilled=" + questionFilled +
                ", assigned=" + assigned +
                ", absentSuspect=" + absentSuspect +
                ", incomplete=" + incomplete +
                ", breachCode=" + breachCode +
                ", omrAbsent=" + omrAbsent +
                ", cardNumber=" + cardNumber +
                ", coursePaperId=" + coursePaperId +
                "}";
    }
}
