package com.qmth.teachcloud.mark.params;

import com.qmth.teachcloud.mark.dto.mark.manage.TrackDTO;
import com.qmth.teachcloud.mark.entity.*;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * 标准评卷结果对象
 * 
 * @author luoshi
 *
 */
public class MarkResult {

    public static final String SPLIT = ",";

    /**
     * 评卷状态
     */
    private String statusValue;

    /**
     * 考生编号
     */
    private Integer studentId;

    /**
     * 评卷任务编号
     */
    private Integer libraryId;

    /**
     * 总分
     */
    private double markerScore;

    /**
     * 分值列表
     */
    private Double[] scoreList;

    /**
     * 阅卷轨迹列表
     */
    private TrackDTO[] trackList;

    /**
     * 特殊标记列表
     */
//    private SpecialTagDTO[] specialTagList;

    /**
     * 所花时间
     */
    private int spent;

    /**
     * 是否问题卷
     */
    private boolean isProblem;

    /**
     * 问题类型
     */
    private Integer problemTypeId;

    private boolean unselective;

    public String getStatusValue() {
        return statusValue;
    }

    public void setStatusValue(String statusValue) {
        this.statusValue = statusValue;
    }

    public Integer getLibraryId() {
        return libraryId;
    }

    public void setLibraryId(Integer libraryId) {
        this.libraryId = libraryId;
    }

    public double getMarkerScore() {
        return markerScore;
    }

    public void setMarkerScore(double markerScore) {
        this.markerScore = markerScore;
    }

    public String getScoreList() {
        List<Object> list = new ArrayList<Object>();
        for (int i = 0; i < scoreList.length; i++) {
            Double d = scoreList[i];
            if (d % 1 == 0) {
                list.add(d.intValue());
            } else {
                list.add(d);
            }
        }
        return StringUtils.join(list, SPLIT);
    }

    public void setScoreList(Double[] scoreList) {
        this.scoreList = scoreList;
    }

    public TrackDTO[] getTrackList() {
        return trackList;
    }

    public void setTrackList(TrackDTO[] trackList) {
        this.trackList = trackList;
    }

    public int getSpent() {
        return spent;
    }

    public void setSpent(int spent) {
        this.spent = spent;
    }

    public Integer getStudentId() {
        return studentId;
    }

    public void setStudentId(Integer studentId) {
        this.studentId = studentId;
    }

    public List<MarkHeaderTrack> getTrackList(MarkArbitrateHistory history) {
        List<MarkHeaderTrack> list = new LinkedList<>();
        if (trackList != null) {
            for (TrackDTO dto : trackList) {
                list.add(dto.transform(history));
            }
        }
        return list;
    }

    public List<MarkTrack> getTrackList(MarkTask markTask, MarkUserGroup markUserGroup) {
        List<MarkTrack> list = new LinkedList<>();
        if (trackList != null) {
            for (TrackDTO dto : trackList) {
                list.add(dto.transform(markTask, markUserGroup));
            }
        }
        return list;
    }

//    public List<MarkHeaderTrack> getHeaderTagList(MarkArbitrateHistory library) {
//        List<MarkHeaderTrack> list = new LinkedList<>();
//        if (specialTagList != null) {
//            for (SpecialTagDTO dto : specialTagList) {
//                list.add(dto.transform(library));
//            }
//        }
//        return list;
//    }

    public boolean isProblem() {
        return isProblem;
    }

    public void setProblem(boolean isProblem) {
        this.isProblem = isProblem;
    }

    public Integer getProblemTypeId() {
        return problemTypeId;
    }

    public void setProblemTypeId(Integer problemTypeId) {
        this.problemTypeId = problemTypeId;
    }

    public boolean isUnselective() {
        return unselective;
    }

    public void setUnselective(boolean unselective) {
        this.unselective = unselective;
    }

}
