package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkArbitrateDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkArbitrateMarkerDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkArbitrateSettingDto;
import com.qmth.teachcloud.mark.dto.mark.manage.Task;
import com.qmth.teachcloud.mark.entity.MarkArbitrateHistory;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.entity.MarkUserGroup;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.mapper.MarkArbitrateHistoryMapper;
import com.qmth.teachcloud.mark.params.MarkResult;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 仲裁记录表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkArbitrateHistoryServiceImpl extends ServiceImpl<MarkArbitrateHistoryMapper, MarkArbitrateHistory> implements MarkArbitrateHistoryService {

    // 并发处理互斥锁
    private Map<Long, Integer> currentTaskMap = new HashMap<>();

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkUserGroupService markUserGroupService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private TaskService taskService;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private MarkService markService;
    @Resource
    private LockService lockService;

    @Override
    public IPage<MarkArbitrateDto> pageArbitrate(Long examId, String paperNumber, Integer groupNumber, Integer pageNumber, Integer pageSize) {
        Page<MarkArbitrateDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkArbitrateDto> markArbitrateDtoIPage = this.baseMapper.pageArbitrate(page, examId, paperNumber, groupNumber);
        for (MarkArbitrateDto arbitrateDto : markArbitrateDtoIPage.getRecords()) {
            if (arbitrateDto.getStudentId() != null) {
                MarkStudent markStudent = markStudentService.getById(arbitrateDto.getStudentId());
                if (markStudent != null) {
                    arbitrateDto.setStudentCode(markStudent.getStudentCode());
                    arbitrateDto.setStudentName(markStudent.getStudentName());
                }
            }
            arbitrateDto.setStatusName(arbitrateDto.getStatus().getName());
            arbitrateDto.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, arbitrateDto.getGroupNumber()));
        }
        return markArbitrateDtoIPage;
    }

    @Override
    public void clearArbitrate(Long arbitrateId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (arbitrateId != null) {
            releaseTask(arbitrateId);
        } else {
            releaseByUser(sysUser.getId());
        }
    }

    @Override
    public List<MarkArbitrateMarkerDto> listMarkerArbitrateRecord(Long arbitrateId) {
        MarkArbitrateHistory markArbitrateHistory = this.getById(arbitrateId);
        List<MarkTask> markTaskList = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndStudentId(markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber(), markArbitrateHistory.getStudentId());
        List<MarkArbitrateMarkerDto> markArbitrateMarkerDtoList = new ArrayList<>();
        for (MarkTask markTask : markTaskList) {
            MarkArbitrateMarkerDto markArbitrateMarkerDto = new MarkArbitrateMarkerDto();
            MarkUserGroup markUserGroup = markUserGroupService.getByExamIdAndPaperNumberAndGroupNumberAndUserId(markTask.getExamId(), markTask.getPaperNumber(), markTask.getGroupNumber(), markTask.getUserId());
            if (markUserGroup != null) {
                markArbitrateMarkerDto.setUserId(markTask.getUserId());
                SysUser sysUser = sysUserService.getById(markTask.getUserId());
                if (sysUser != null) {
                    markArbitrateMarkerDto.setUserLoginName(sysUser.getLoginName());
                    markArbitrateMarkerDto.setUserName(sysUser.getRealName());
                }
                markArbitrateMarkerDto.setMarkTime(markTask.getMarkerTime());
                markArbitrateMarkerDto.setScoreList(markTask.getMarkerScoreList());
                markArbitrateMarkerDto.setTotalScore(markTask.getMarkerScore());
            }
            markArbitrateMarkerDtoList.add(markArbitrateMarkerDto);
        }
        return markArbitrateMarkerDtoList;
    }

    @Override
    public MarkArbitrateSettingDto getArbitrateSetting(Long arbitrateId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        MarkArbitrateHistory markArbitrateHistory = this.getById(arbitrateId);
        if (markArbitrateHistory == null) {
            throw ExceptionResultEnum.ERROR.exception("仲裁数据异常");
        }
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber());
        MarkArbitrateSettingDto markArbitrateSettingDto = new MarkArbitrateSettingDto();
        if (markPaper != null) {
            markArbitrateSettingDto.setMarkMode(markPaper.getMarkMode().name());
            markArbitrateSettingDto.setCourseCode(markPaper.getCourseCode());
            markArbitrateSettingDto.setCourseName(markPaper.getCourseName());
            markArbitrateSettingDto.setPaperNumber(markPaper.getPaperNumber());
            markArbitrateSettingDto.setUserLoginName(sysUser.getLoginName());
            markArbitrateSettingDto.setUserName(sysUser.getRealName());
            if (StringUtils.isNotBlank(markPaper.getPaperFilePath())) {
                markArbitrateSettingDto.setPaperUrl(teachcloudCommonService.filePreview(markPaper.getPaperFilePath()));
            }
            if (StringUtils.isNotBlank(markPaper.getAnswerFilePath())) {
                markArbitrateSettingDto.setAnswerUrl(teachcloudCommonService.filePreview(markPaper.getAnswerFilePath()));
            }
        }
        return markArbitrateSettingDto;
    }

    @Override
    public Task getArbitrateTask(Long arbitrateId) {
        MarkArbitrateHistory markArbitrateHistory = this.getById(arbitrateId);
        MarkGroup markGroup = markGroupService.getByExamIdAndPaperNumberAndGroupNumber(markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber());
        return taskService.build(markArbitrateHistory, markGroup);
    }

    @Override
    public void saveArbitrateTask(MarkResult markResult) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        MarkArbitrateHistory markArbitrateHistory = this.getById(markResult.getLibraryId());
        if (markArbitrateHistory != null) {
            try {
                lockService.watch(LockType.EXAM_SUBJECT, markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber());
                lockService.watch(LockType.GROUP, markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber());
                markService.processArbitrate(markResult, sysUser.getId());
                releaseTask(markArbitrateHistory.getId());
            } catch (Exception e) {
                log.error("ArbitrateController-处理仲裁卷出错", e);
            } finally {
                lockService.unwatch(LockType.GROUP, markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber());
                lockService.unwatch(LockType.EXAM_SUBJECT, markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber());
            }
        } else {
            throw ExceptionResultEnum.ERROR.exception("任务不存在");
        }
    }

    private void releaseTask(Long taskId) {
        synchronized (currentTaskMap) {
            currentTaskMap.remove(taskId);
        }
    }

    private void releaseByUser(Long userId) {
        Set<Long> taskIds = new HashSet<>();
        taskIds.addAll(currentTaskMap.keySet());
        synchronized (currentTaskMap) {
            for (Long taskId : taskIds) {
                Integer value = currentTaskMap.get(taskId);
                if (value != null && value.equals(userId)) {
                    currentTaskMap.remove(taskId);
                }
            }
        }
    }
}
