package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.common.bean.dto.mark.PictureConfig;
import com.qmth.teachcloud.common.bean.params.mark.setting.GroupPictureConfigParams;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupClassProgressDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupProgressDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupSummaryProgressDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupTotalProgressDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupSingleDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupTaskDto;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.entity.MarkUserGroup;
import com.qmth.teachcloud.mark.mapper.MarkGroupMapper;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 评卷分组表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkGroupServiceImpl extends ServiceImpl<MarkGroupMapper, MarkGroup> implements MarkGroupService {

    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkUserGroupService markUserGroupService;
    @Resource
    private MarkUserClassService markUserClassService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkService markService;

    @Override
    public MarkGroupTaskDto listGroupTaskByExamIdAndPaperNumber(Long examId, String paperNumber) {
        MarkGroupTaskDto markGroupTaskDto = new MarkGroupTaskDto();
        markGroupTaskDto.setExamId(examId);
        markGroupTaskDto.setPaperNumber(paperNumber);

        // 分组信息
        List<MarkGroup> groupList = listGroupByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isNotEmpty(groupList)) {
            List<MarkGroupDto> markGroupDtoList = new ArrayList<>();
            for (MarkGroup markGroup : groupList) {
                MarkGroupDto markGroupDto = new MarkGroupDto();
                markGroupDto.setGroupNumber(markGroup.getNumber());
                markGroupDto.setDoubleRate(markGroup.getDoubleRate());
                markGroupDto.setArbitrateThreshold(markGroup.getArbitrateThreshold());
                markGroupDto.setScorePolicy(markGroup.getScorePolicy());
                if (StringUtils.isNotBlank(markGroup.getPicList())) {
                    markGroupDto.setPictureConfigs(JSON.parseArray(markGroup.getPicList(), PictureConfig.class));
                }
                // 评卷员
                markGroupDto.setMarkers(markUserGroupService.listGroupUserByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroup.getNumber()));
                // 题目
                markGroupDto.setQuestions(markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroup.getNumber(), false));
                markGroupDto.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, markGroup.getNumber()));
                markGroupDtoList.add(markGroupDto);
            }
            markGroupTaskDto.setGroups(markGroupDtoList);
        }
        // 分班阅参数
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        markGroupTaskDto.setOpenMarkClass(markPaper != null && markPaper.getOpenMarkClass());
        return markGroupTaskDto;
    }

    @Override
    public List<MarkGroup> listGroupByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkGroup::getExamId, examId)
                .eq(MarkGroup::getPaperNumber, paperNumber)
                .orderByAsc(MarkGroup::getNumber);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public void deleteGroupByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        UpdateWrapper<MarkGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkGroup::getExamId, examId)
                .eq(MarkGroup::getPaperNumber, paperNumber)
                .eq(MarkGroup::getNumber, groupNumber);
        this.remove(updateWrapper);
    }

    @Override
    public void updatePicListByExamIdAndPaperNumberAndGroupNumber(GroupPictureConfigParams groupPictureConfigParams) {
        List<PictureConfig> pictureConfigs = groupPictureConfigParams.getPictureConfigs();
        if (CollectionUtils.isEmpty(pictureConfigs)) {
            throw ExceptionResultEnum.ERROR.exception("没有评卷区数据");
        }
        UpdateWrapper<MarkGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkGroup::getPicList, JSON.toJSONString(pictureConfigs))
                .eq(MarkGroup::getExamId, groupPictureConfigParams.getExamId())
                .eq(MarkGroup::getPaperNumber, groupPictureConfigParams.getPaperNumber())
                .eq(MarkGroup::getNumber, groupPictureConfigParams.getGroupNumber());
        this.update(updateWrapper);
    }

    @Transactional
    @Override
    public void updateOpenMarkClassByExamIdAndPaperNumber(Long examId, String paperNumber, Boolean openMarkClass) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getOpenMarkClass, openMarkClass)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        markPaperService.update(updateWrapper);

        // 禁用，删除分班数据
        if (openMarkClass != null && !openMarkClass) {
            markUserClassService.deleteByExamIdAndPaperNumber(examId, paperNumber);
        }
    }

    @Override
    public MarkGroup getByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        QueryWrapper<MarkGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkGroup::getExamId, examId)
                .eq(MarkGroup::getPaperNumber, paperNumber)
                .eq(MarkGroup::getNumber, groupNumber);
        return this.getOne(queryWrapper);
    }

    @Override
    public MarkGroupSummaryProgressDto summaryGroupProgress(Long examId, String paperNumber) {
        MarkGroupSummaryProgressDto markGroupSummaryProgressDto = new MarkGroupSummaryProgressDto();

        // totalInfo
        MarkGroupTotalProgressDto markGroupTotalProgressDto = new MarkGroupTotalProgressDto();
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        markGroupTotalProgressDto.setStudentCount(markPaper.getStudentCount());
        markGroupTotalProgressDto.setUploadCount(markPaper.getUploadCount());
        markGroupTotalProgressDto.setAbsentCount(markPaper.getAbsentCount());

        List<MarkTask> markTaskList = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndUserId(examId, paperNumber, null, null);
        int taskCount = markTaskList.size();
        int totalMarkedCount = markTaskService.markedCount(markTaskList);
        markGroupTotalProgressDto.setPercent(markTaskService.calcPercent(totalMarkedCount, taskCount));
        markGroupSummaryProgressDto.setTotalInfo(markGroupTotalProgressDto);

        // groupInfo
        List<MarkGroupProgressDto> markGroupProgressDtoList = new ArrayList<>();
        List<MarkGroup> markGroupList = this.listGroupByExamIdAndPaperNumber(examId, paperNumber);
        for (MarkGroup markGroup : markGroupList) {
            MarkGroupProgressDto markGroupProgressDto = new MarkGroupProgressDto();
            markGroupProgressDto.setGroupNumber(markGroup.getNumber());
            markGroupProgressDto.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, markGroup.getNumber()));
            List<MarkUser> markUserList = markUserGroupService.listGroupUserByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroup.getNumber());
            markGroupProgressDto.setMarkerCount(markUserList.size());
            markGroup.setQuestionList(markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroup.getNumber(), false));
            markGroupProgressDto.setScoreList(markGroup.getScoreList());
            List<MarkTask> groupNumberMarkTask = markTaskList.stream().filter(m -> m.getGroupNumber().equals(markGroup.getNumber())).collect(Collectors.toList());
            markGroupProgressDto.setTaskCount(groupNumberMarkTask.size());
            markGroupProgressDto.setMarkedCount(markTaskService.markedCount(groupNumberMarkTask));
            markGroupProgressDto.setLeftCount(markGroupProgressDto.getTaskCount() - markGroupProgressDto.getMarkedCount());
            markGroupProgressDto.setCurrentCount(markService.applyCurrentCount(markGroup));
            markGroupProgressDto.setPercent(markTaskService.calcPercent(markGroupProgressDto.getMarkedCount(), markGroupProgressDto.getTaskCount()));
            markGroupProgressDto.setArbitrateCount(markTaskService.waitArbitrateCount(groupNumberMarkTask));
            markGroupProgressDtoList.add(markGroupProgressDto);
        }
        markGroupSummaryProgressDto.setGroupInfo(markGroupProgressDtoList);
        return markGroupSummaryProgressDto;
    }

    @Override
    public IPage<MarkGroupClassProgressDto> summaryGroupClassProgress(Long examId, String paperNumber, Integer pageNumber, Integer pageSize) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper.getOpenMarkClass()) {
            Page<MarkGroupClassProgressDto> page = new Page<>(pageNumber, pageSize);
            IPage<MarkGroupClassProgressDto> markGroupClassProgressDtoIPage = markUserClassService.pageClassByExamIdAndPaperNumber(page, examId, paperNumber);
            for (MarkGroupClassProgressDto markGroupClassProgressDto : markGroupClassProgressDtoIPage.getRecords()) {
                List<MarkTask> totalMarkTaskList = new ArrayList<>();
                List<MarkUser> totalMarkUserList = new ArrayList<>();
                int count = 0;
                for (String s : markGroupClassProgressDto.getGroupNumbers().split(",")) {
                    Integer groupNumber = Integer.parseInt(s);
                    totalMarkTaskList.addAll(markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndUserId(examId, paperNumber, groupNumber, null));
                    totalMarkUserList.addAll(markUserGroupService.listGroupUserByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber));

                    MarkGroup markGroup = markGroupService.getByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber);
                    List<MarkUser> markUserList = markUserClassService.listClassMarkerByExamIdAndPaperNumberAndGroupNumberAndClassName(examId, paperNumber, groupNumber, markGroupClassProgressDto.getClassName());
                    for (MarkUser markUser : markUserList) {
                        MarkUserGroup markUserGroup = markUserGroupService.getByExamIdAndPaperNumberAndGroupNumberAndUserId(examId, paperNumber, groupNumber, markUser.getUserId());
                        if (markUserGroup != null) {
                            int markerCount = markService.applyCurrentCount(markGroup);
                            count += markerCount;
                        }
                    }
                }
                markGroupClassProgressDto.setMarkerCount(totalMarkUserList.size());
                markGroupClassProgressDto.setTaskCount(totalMarkTaskList.size());
                markGroupClassProgressDto.setMarkedCount(markTaskService.markedCount(totalMarkTaskList));
                markGroupClassProgressDto.setLeftCount(markGroupClassProgressDto.getTaskCount() - markGroupClassProgressDto.getMarkedCount());
                markGroupClassProgressDto.setCurrentCount(count);
                markGroupClassProgressDto.setPercent(markTaskService.calcPercent(markGroupClassProgressDto.getMarkedCount(), markGroupClassProgressDto.getTaskCount()));
                markGroupClassProgressDto.setArbitrateCount(markTaskService.waitArbitrateCount(totalMarkTaskList));
            }
            return markGroupClassProgressDtoIPage;
        }
        return null;
    }

    @Override
    public void updateMarkedCount(Long examId, String paperNumber, Integer groupNumber, int count) {
        UpdateWrapper<MarkGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkGroup::getMarkedCount, count)
                .eq(MarkGroup::getExamId, examId)
                .eq(MarkGroup::getPaperNumber, paperNumber)
                .eq(MarkGroup::getNumber, groupNumber);
        this.update(updateWrapper);
    }

    @Transactional
    @Override
    public void saveGroupBatch(MarkGroupTaskDto markGroupTaskDto) {

    }

    @Transactional
    @Override
    public void updateGroup(MarkGroupSingleDto markGroupSingleDto) {
        Long examId = markGroupSingleDto.getExamId();
        String paperNumber = markGroupSingleDto.getPaperNumber();
        MarkGroupDto markGroupDto = markGroupSingleDto.getGroupInfo();
        MarkGroup markGroup = this.getByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroupDto.getGroupNumber());
        if (markGroup == null) {
            throw ExceptionResultEnum.ERROR.exception("数据不存在");
        }

        markGroup.setDoubleRate(markGroupDto.getDoubleRate());
        markGroup.setScorePolicy(markGroupDto.getScorePolicy());
        markGroup.setArbitrateThreshold(markGroupDto.getArbitrateThreshold());
        markGroup.setPicList(JSON.toJSONString(markGroupDto.getPictureConfigs()));
        this.updateById(markGroup);

        List<MarkUserGroup> markUserGroupList = markUserGroupService.listByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroupDto.getGroupNumber());
        if (CollectionUtils.isNotEmpty(markUserGroupList)) {
            MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
            List<Long> userIds = markUserGroupList.stream().map(MarkUserGroup::getUserId).collect(Collectors.toList());
            List<MarkUserGroup> markUserGroups = new ArrayList<>();
            for (MarkUser marker : markGroupDto.getMarkers()) {
                if (userIds.contains(marker.getUserId())) {
                    continue;
                }
                markUserGroups.add(new MarkUserGroup(examId, paperNumber, markGroupDto.getGroupNumber(), marker.getUserId(), markPaper.getMarkMode()));
            }
            if (CollectionUtils.isNotEmpty(markUserGroups)) {
                markUserGroupService.saveBatch(markUserGroups);
            }
        }
    }
}
