package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.mark.MarkSettingDto;
import com.qmth.teachcloud.mark.dto.mark.score.CheckScoreListDto;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.mark.mapper.MarkPaperMapper;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 考试科目表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkPaperServiceImpl extends ServiceImpl<MarkPaperMapper, MarkPaper> implements MarkPaperService {

    @Override
    public IPage<MarkSettingDto> listPaperSetting(Long examId, String courseCode, String paperNumber, Boolean groupStatus, Integer pageNumber, Integer pageSize) {
        Page<MarkSettingDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkSettingDto> markSettingDtoIPage = this.baseMapper.listPaperSetting(page, examId, courseCode, paperNumber, groupStatus);
        for (MarkSettingDto record : markSettingDtoIPage.getRecords()) {
            if (Objects.nonNull(record.getMarkMode())) {
                record.setMarkModeDisplay(record.getMarkMode().getName());
            }
        }
        return markSettingDtoIPage;
    }

    @Override
    public MarkPaper getByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkPaper> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        return this.getOne(queryWrapper);
    }

    @Override
    public void savePaperSetting(MarkPaper markPaper) {
        MarkPaper markPaperOld = this.getByExamIdAndPaperNumber(markPaper.getExamId(), markPaper.getPaperNumber());
        if (markPaperOld == null) {
            throw ExceptionResultEnum.ERROR.exception("评卷设置数据不存在");
        }
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getMarkMode, markPaper.getMarkMode())
                .set(MarkPaper::getMarkStartTime, markPaper.getMarkStartTime())
                .set(MarkPaper::getMarkEndTime, markPaper.getMarkEndTime())
                .set(MarkPaper::getSheetView, markPaper.getSheetView())
                .set(MarkPaper::getShowObjectScore, markPaper.getObjectiveScore())
                .set(MarkPaper::getAutoScroll, markPaper.getAutoScroll())
                .set(MarkPaper::getPassScore, markPaper.getPassScore())
                .set(MarkPaper::getExcellentScore, markPaper.getExcellentScore())
                .set(MarkPaper::getSheetConfig, markPaper.getSheetConfig())
                .eq(MarkPaper::getExamId, markPaper.getExamId())
                .eq(MarkPaper::getPaperNumber, markPaper.getPaperNumber());
        this.update(updateWrapper);
    }

    @Override
    public Boolean finishPaper(Long examId, List<String> paperNumbers, MarkPaperStatus status) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getStatus, status)
                .eq(MarkPaper::getExamId, examId)
                .in(MarkPaper::getPaperNumber, paperNumbers);
        return this.update(updateWrapper);
    }

    @Override
    public List<MarkPaper> listQualityMarkPaperByStatus(MarkPaperStatus status, int uploadCount) {
        return this.baseMapper.listQualityMarkPaperByStatus(status.name(), uploadCount);
    }

    @Override
    public int getCountByExam(Long examId) {
        QueryWrapper<MarkPaper> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkPaper::getExamId, examId);
        return baseMapper.selectCount(queryWrapper);
    }

    @Override
    public IPage<CheckScoreListDto> listStudentScoreList(Long examId, String courseCode, String paperNumber, Integer pageNumber, Integer pageSize) {
        Page<CheckScoreListDto> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.listStudentScoreList(page, examId, courseCode, paperNumber);
    }

}
