package com.qmth.teachcloud.mark.service.impl;

import com.qmth.teachcloud.common.enums.mark.MarkArbitrateStatus;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.enums.mark.MarkProblemStatus;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.mark.dto.mark.ScoreItem;
import com.qmth.teachcloud.mark.entity.*;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.enums.ScorePolicy;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.params.MarkResult;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.BigDecimalUtils;
import com.qmth.teachcloud.mark.utils.TaskLock;
import com.qmth.teachcloud.mark.utils.TaskLockUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class MarkServiceImpl implements MarkService {

    public static final int UN_SELECTIVE_SCORE = -1;

    private Map<Long, Long> markerLastUpdateTime = new ConcurrentHashMap<>();

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private MarkUserGroupService markUserGroupService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkTrackService markTrackService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkGroupStudentService markGroupStudentService;
    @Resource
    private MarkProblemHistoryService markProblemHistoryService;
    @Resource
    private MarkArbitrateHistoryService markArbitrateHistoryService;
    @Resource
    private MarkSubjectiveScoreService markSubjectiveScoreService;
    @Resource
    private MarkHeaderTrackService markHeaderTrackService;
    @Resource
    LockService lockService;

    /**
     * 某个评卷分组已申请的评卷任务数量
     *
     * @return
     */
    @Override
    public int applyCurrentCount(MarkGroup markGroup) {
        // todo 正在评卷数量查询 (评卷员和评卷分组分开2个方法)
        TaskLock taskLock = getTaskLock(markGroup);
        int count = 0;
        if (taskLock != null) {
            count = taskLock.count();
        }
        return count;
    }

    private TaskLock getTaskLock(MarkGroup markGroup) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markGroup.getExamId(), markGroup.getPaperNumber());
        if (markPaper.getStatus() == MarkPaperStatus.FORMAL) {
            return TaskLockUtil.getFormalTask(getGroupKey(markGroup));
        }
        return null;
    }

    /**
     * 释放某个评卷员的所有锁定任务
     *
     * @param markUserGroup
     */
    @Override
    public void releaseByMarkUserGroup(MarkUserGroup markUserGroup) {
        TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(markUserGroup));
        taskLock.clear(markUserGroup.getId());
    }

    @Override
    public int applyCurrentCount(MarkGroup markGroup, Long markUserGroupId) {
        int count = 0;
        TaskLock taskLock = getTaskLock(markGroup);
        if (taskLock != null) {
            count = taskLock.count(markUserGroupId);
        }
        return count;
    }

    @Override
    public void resetMarker(MarkUserGroup markUserGroup) {
        Long examId = markUserGroup.getExamId();
        String paperNumber = markUserGroup.getPaperNumber();
        Integer groupNumber = markUserGroup.getGroupNumber();
        Long userId = markUserGroup.getUserId();
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper == null || markUserGroupService.getById(markUserGroup.getId()) == null) {
            return;
        }
        if (markPaper.getStatus() == MarkPaperStatus.FORMAL) {
            // 遍历相关评卷任务的模式
            List<MarkTaskStatus> statusList = Arrays.asList(MarkTaskStatus.ARBITRATED, MarkTaskStatus.WAIT_ARBITRATE, MarkTaskStatus.PROBLEM);
            List<MarkTask> markTaskList = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndUserIdAndStatusNotIn(examId, paperNumber, groupNumber, userId, statusList);
            for (MarkTask markTask : markTaskList) {
                Long studentId = markTask.getStudentId();
                markTrackService.deleteByTaskId(markTask.getId());
//                specialTagDao.deleteByLibraryId(library.getId());
                markTaskService.resetById(markTask.getId(), null, null, null, MarkTaskStatus.WAITING);
                lockService.waitlock(LockType.STUDENT, markTask.getStudentId());
                updateStudentGroupStatus(studentId, examId, paperNumber, groupNumber, SubjectiveStatus.UNMARK);
                markStudentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0D, null);
//                inspectedService.cancelByStudent(library.getStudentId());
                lockService.unlock(LockType.STUDENT, markTask.getStudentId());
            }
            markUserGroupService.resetById(markUserGroup.getId());
        }
        this.updateMarkedCount(examId, paperNumber, groupNumber);
        releaseByMarkUserGroup(markUserGroup);
//        inspectHistoryService.deleteByExamIdAndSubjectCode(marker.getExamId(), marker.getSubjectCode());
    }

    @Override
    public void updateMarkedCount(Long examId, String paperNumber, Integer groupNumber) {
        List<MarkTaskStatus> markTaskStatuses = Arrays.asList(MarkTaskStatus.MARKED, MarkTaskStatus.ARBITRATED);
        int count = markTaskService.countByExamIdAndPaperNumberAndGroupNumberAndStatusIn(examId, paperNumber, groupNumber, markTaskStatuses);
        markGroupService.updateMarkedCount(examId, paperNumber, groupNumber, count);
    }

    @Override
    public boolean rejectMarkTask(MarkTask markTask, Long userId) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markTask.getExamId(), markTask.getPaperNumber());
        if (markPaper.getStatus() == MarkPaperStatus.FINISH) {
            return false;
        }
        Long now = System.currentTimeMillis();
        if (markTaskService.resetById(markTask.getId(), null, userId, now, MarkTaskStatus.WAITING)) {
            markTrackService.deleteByTaskId(markTask.getId());
            resetStudentGroup(markTask.getStudentId(), markTask.getExamId(), markTask.getPaperNumber(), markTask.getGroupNumber());
            markProblemHistoryService.resetByMarkTaskId(markTask.getId(), MarkProblemStatus.WAITING, userId, MarkProblemStatus.BACK, now);
            updateMarkedCount(markTask.getExamId(), markTask.getPaperNumber(), markTask.getGroupNumber());
            return true;
        } else {
            return false;
        }
    }

    private void resetStudentGroup(Long studentId, Long examId, String paperNumber, Integer groupNumber) {
        updateStudentGroupStatus(studentId, examId, paperNumber, groupNumber, SubjectiveStatus.UNMARK);
        markStudentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0D, null);
    }

    private void updateStudentGroupStatus(Long studentId, Long examId, String paperNumber, Integer groupNumber, SubjectiveStatus status) {
        MarkGroupStudent gs = new MarkGroupStudent();
        gs.setStudentId(studentId);
        gs.setExamId(examId);
        gs.setPaperNumber(paperNumber);
        gs.setGroupNumber(groupNumber);
        gs.setStatus(status);
        markGroupStudentService.save(gs);
    }

    @Override
    public String getGroupKey(MarkGroup markGroup) {
        return markGroup.getExamId() + "_" + markGroup.getPaperNumber() + "_" + markGroup.getNumber();
    }

    @Override
    public void updateQuality(MarkUserGroup markUserGroup) {
        List<MarkTask> list = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndUserId(markUserGroup.getExamId(), markUserGroup.getPaperNumber(), markUserGroup.getGroupNumber(), markUserGroup.getUserId());
        int finishCount = 0;
        int validCount = 0;
        double sumScore = 0;
        double sumScore2 = 0;
        double avgScore = 0;
        double stdevScore = 0;
        double sumSpent = 0;
        double avgSpent = 0;
        for (MarkTask markTask : list) {
            finishCount++;
            if (markTask.getStatus() == MarkTaskStatus.MARKED) {
                validCount++;
            }
            double score = markTask.getMarkerScore() != null && markTask.getMarkerScore() != UN_SELECTIVE_SCORE ? markTask.getMarkerScore() : 0;
            int spent = markTask.getMarkerSpent() != null ? markTask.getMarkerSpent() : 0;

            sumScore += score;
            sumScore2 += Math.pow(score, 2);
            sumSpent += spent;
            if (finishCount > 0) {
                avgScore = sumScore / finishCount;
                avgSpent = sumSpent / finishCount;
                // 递归法计算标准差
                stdevScore = Math.sqrt(sumScore2 / finishCount - Math.pow(sumScore / finishCount, 2));
            }
        }
        markUserGroupService.updateQualityById(markUserGroup.getId(), finishCount, validCount, avgSpent / 1000, avgScore, stdevScore);
        markerLastUpdateTime.put(markUserGroup.getId(), System.currentTimeMillis());
    }

    @Override
    public boolean needUpdateQuality(MarkUserGroup markUserGroup, int expireMinutes) {
        if (markUserGroup == null) {
            return false;
        }
        Long lastUpdateTime = markerLastUpdateTime.get(markUserGroup.getId());
        MarkTask markTask = markTaskService.getLastOneByUserIdAndStatus(markUserGroup.getExamId(), markUserGroup.getPaperNumber(), markUserGroup.getGroupNumber(), markUserGroup.getUserId(), MarkTaskStatus.MARKED);
        if (markTask != null && markTask.getMarkerTime() != null) {
            long lastMarkTime = markTask.getMarkerTime();
            if (lastUpdateTime != null && lastUpdateTime > lastMarkTime) {
                return false;
            } else {
                return (System.currentTimeMillis() - lastMarkTime) < (expireMinutes * 60 * 1000);
            }
        }
        return false;
    }

    /**
     * 管理员/组长处理仲裁卷
     *
     * @param markResult
     * @param userId
     */
    @Override
    public void processArbitrate(MarkResult markResult, Long userId) {
        MarkArbitrateHistory markArbitrateHistory = markArbitrateHistoryService.getById(markResult.getLibraryId());
        markArbitrateHistory.setUpdateUserId(userId);
        markArbitrateHistory.setTotalScore(markResult.isUnselective() ? UN_SELECTIVE_SCORE : markResult.getMarkerScore());
        markArbitrateHistory.setScoreList(markResult.isUnselective() ? null : markResult.getScoreList());
        markArbitrateHistory.setStatus(MarkArbitrateStatus.MARKED);
        markArbitrateHistory.setUpdateTime(System.currentTimeMillis());
        // 保存阅卷轨迹
        int unansweredCount = 0;
        if (markResult.getTrackList() != null && !markResult.isUnselective()) {
            markHeaderTrackService.deleteByExamIdAndPaperNumberAndGroupNumberAndStudentId(markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber(), markArbitrateHistory.getStudentId());
            List<MarkHeaderTrack> tracks = markResult.getTrackList(markArbitrateHistory);
            for (MarkHeaderTrack t : tracks) {
                markHeaderTrackService.saveOrUpdate(t);
            }
        }
        // 保存特殊标记
//        if (result.getSpecialTagList() != null && !result.isUnselective()) {
//            headerTagDao.deleteByStudentIdAndGroupNumber(history.getStudentId(), history.getGroupNumber());
//            headerTagDao.save(result.getHeaderTagList(history));
//        }
        markArbitrateHistoryService.saveOrUpdate(markArbitrateHistory);
        markTaskService.updateHeaderResult(markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber(), markArbitrateHistory.getStudentId(), markArbitrateHistory.getUpdateUserId(), markArbitrateHistory.getTotalScore(), markArbitrateHistory.getScoreList(), markArbitrateHistory.getUpdateTime(), MarkTaskStatus.ARBITRATED);
        updateMarkedCount(markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber());
        checkStudentGroup(markArbitrateHistory.getStudentId(), markGroupService.getByExamIdAndPaperNumberAndGroupNumber(markArbitrateHistory.getExamId(), markArbitrateHistory.getPaperNumber(), markArbitrateHistory.getGroupNumber()));
    }

    private String getGroupKey(MarkUserGroup markUserGroup) {
        return markUserGroup.getExamId() + "_" + markUserGroup.getPaperNumber() + "_" + markUserGroup.getGroupNumber();
    }

    /**
     * 考生分组判断是否评卷完成，以及后续的统一处理动作
     *
     * @param studentId
     * @param group
     */
    private void checkStudentGroup(Long studentId, MarkGroup group) {
        if (calculateGroup(group, studentId)) {
            updateStudentGroupStatus(studentId, group.getExamId(), group.getPaperNumber(), group.getNumber(), SubjectiveStatus.MARKED);
            updateStudentGroupScore(studentId, group.getExamId(), group.getPaperNumber(), group, group.getMarkScore(), group.getMarkScoreDetail());
            // 未分组的题目
            long unGroupQuestionCount = markQuestionService.countQuestionByExamIdAndPaperNumberAndGroupNumberIsNull(group.getExamId(), group.getPaperNumber(), false);
            long groupCount = CollectionUtils.size(markGroupService.listGroupByExamIdAndPaperNumber(group.getExamId(), group.getPaperNumber()));
            checkStudentSubjective(studentId, groupCount, unGroupQuestionCount);
        } else {
            updateStudentGroupStatus(studentId, group.getExamId(), group.getPaperNumber(), group.getNumber(), SubjectiveStatus.UNMARK);
            markStudentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0D, null);
        }
    }

    @Override
    public void checkStudentSubjective(Long studentId, long groupCount, long unGroupQuestionCount) {
        if (markGroupStudentService.countByStudentIdAndStatus(studentId, SubjectiveStatus.MARKED) == groupCount && unGroupQuestionCount == 0) {
            scoreCalculate(studentId);
        } else {
            markStudentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0D, null);
        }
    }

    private boolean calculateGroup(MarkGroup group, Long studentId) {
        double score = 0;
        List<ScoreItem> detail = new ArrayList<>();
        int count = 0;
        // 未设置算分策略的情况下，默认取平均分
        ScorePolicy policy = group.getScorePolicy() != null ? group.getScorePolicy() : ScorePolicy.AVG;
        List<MarkTask> list = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndStudentId(group.getExamId(), group.getPaperNumber(), group.getNumber(), studentId);
        if (list.isEmpty()) {
            return false;
        }
        boolean selectiveAll = false;
        for (MarkTask markTask : list) {
            if (markTask.getStatus() != MarkTaskStatus.MARKED && markTask.getStatus() != MarkTaskStatus.ARBITRATED) {
                // 有非完成状态的评卷任务，直接返回
                return false;
            }
            double markerScore = markTask.getStatus() == MarkTaskStatus.ARBITRATED ? markTask.getHeaderScore() : markTask.getMarkerScore();
            if (markerScore == UN_SELECTIVE_SCORE) {
                selectiveAll = true;
            }
        }
        if (selectiveAll) {
            group.setMarkScore(UN_SELECTIVE_SCORE);
            return true;
        }
        for (MarkTask markTask : list) {
            count++;
            Double current = markTask.getHeaderScore() != null ? markTask.getHeaderScore() : markTask.getMarkerScore();
            List<ScoreItem> scores = markTask.getScoreList();
            if (count == 1) {
                // 首份评卷任务，直接取总分与明细
                score = current;
                detail = scores;
            } else {
                switch (policy) {
                    case AVG:
                        // 直接累加
                        score = BigDecimalUtils.add(score, current);
                        for (int i = 0; i < detail.size(); i++) {
                            try {
                                ScoreItem item = detail.get(i);
                                ScoreItem other = scores.get(i);
                                item.setScore(BigDecimalUtils.add(item.getScore(), other.getScore()));
                            } catch (Exception e) {
                                continue;
                            }
                        }
                        break;
                    case MAX:
                        // 高分优先
                        if (current > score) {
                            score = current;
                            detail = scores;
                        }
                        break;
                    case MIN:
                        // 低分优先
                        if (current < score) {
                            score = current;
                            detail = scores;
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        // 取平均分策略下，累计分数需要重新计算一次
        if (policy == ScorePolicy.AVG && count > 1) {
            score = BigDecimalUtils.div(score, count);
            for (ScoreItem item : detail) {
                item.setScore(BigDecimalUtils.div(item.getScore(), count));
            }
        }
        group.setMarkScore(score);
        group.setMarkScoreDetail(detail);
        return true;
    }

    private void updateStudentGroupScore(Long studentId, Long examId, String paperNumber, MarkGroup group, double score, List<ScoreItem> scoreList) {
        // scoreDao.deleteByStudentIdAndGroupNumber(studentId, groupNumber);
        List<MarkQuestion> questions = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, group.getNumber(), false);
        List<MarkSubjectiveScore> subjectiveScores = markSubjectiveScoreService.listByStudentIdAndGroupNumber(studentId, group.getNumber());
        for (int i = 0; i < questions.size(); i++) {
            MarkQuestion question = questions.get(i);
            MarkSubjectiveScore ss;
            if (!subjectiveScores.isEmpty() && subjectiveScores.size() == questions.size()) {
                ss = subjectiveScores.get(i);
            } else {
                ss = new MarkSubjectiveScore();
            }
            ss.setStudentId(studentId);
            ss.setExamId(examId);
            ss.setPaperNumber(paperNumber);
            ss.setGroupNumber(group.getNumber());
            ss.setGroupScore(score);
            ss.setMainNumber(question.getMainNumber());
            ss.setSubNumber(String.valueOf(question.getSubNumber()));
            if (score != UN_SELECTIVE_SCORE) {
                ScoreItem item = scoreList.get(i);
                ss.setScore(item.getScore());
                ss.setMainScore(0.0);
                ss.setUncalculate(false);
            } else {
                ss.setScore(score);
                ss.setMainScore(score);
                ss.setUncalculate(true);
            }
//            if (group.getArbitrateThreshold() != null && group.getArbitrateThreshold() > 0) {
//                ss.setUnansweredCount(markHeaderTrackService.countByStudentIdAndQuestionNumberAndUnanswered(studentId, question.getQuestionNumber(), true));
//            } else {
//                ss.setUnansweredCount(markTaskService.countByStudentIdAndQuestionNumberAndUnanswered(studentId, question.getQuestionNumber(), true));
//            }
            markSubjectiveScoreService.saveOrUpdate(ss);
        }
    }

    private void scoreCalculate(Long studentId) {
        List<ScoreItem> scoreList = new ArrayList<>();
        Map<Integer, List<MarkSubjectiveScore>> mainScoreMap = new HashMap<>();
        Map<Integer, Double> scoreMap = new HashMap<>();

        // 循环所有主观得分明细
        List<MarkSubjectiveScore> list = markSubjectiveScoreService.listByStudentIdAndGroupNumber(studentId, null);
        list.sort(null);
        for (MarkSubjectiveScore ss : list) {
            List<MarkSubjectiveScore> mainScoreList = mainScoreMap.get(ss.getMainNumber());
            if (mainScoreList == null) {
                mainScoreList = new ArrayList<MarkSubjectiveScore>();
            }
            mainScoreList.add(ss);
            mainScoreMap.put(ss.getMainNumber(), mainScoreList);
            scoreList.add(new ScoreItem(ss));
        }
        // 计算大题分
        for (Integer mainNumber : mainScoreMap.keySet()) {
            List<MarkSubjectiveScore> mainScoreList = mainScoreMap.get(mainNumber);
            BigDecimal mainScore = BigDecimal.ZERO;
            for (MarkSubjectiveScore subjectiveScore : mainScoreList) {
                if (subjectiveScore.getScore() != UN_SELECTIVE_SCORE) {
                    mainScore = mainScore.add(BigDecimal.valueOf(subjectiveScore.getScore()));
                    subjectiveScore.setUncalculate(false);
                } else {
                    mainScore = new BigDecimal(UN_SELECTIVE_SCORE);
                    subjectiveScore.setUncalculate(true);
                    break;
                }
            }
            for (MarkSubjectiveScore subjectiveScore : mainScoreList) {
                subjectiveScore.setMainScore(mainScore.doubleValue());
                markSubjectiveScoreService.saveOrUpdate(subjectiveScore);
            }
            scoreMap.put(mainNumber, mainScore.doubleValue());
        }
        // 计算选做题分数
        BigDecimal totalScore = BigDecimal.ZERO;
        // 计算非选做题总分
        for (Integer mainNumber : scoreMap.keySet()) {
            totalScore = totalScore.add(BigDecimal.valueOf(scoreMap.get(mainNumber)));
        }
        // 全部评完，更新考生主观题得分
        markStudentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.MARKED, totalScore.doubleValue(), MarkStudent.buildScoreList(scoreList));
    }
}
