package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupClassProgressDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupDto;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkUserClassDto;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkClassUserParams;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkUserClass;
import com.qmth.teachcloud.mark.mapper.MarkUserClassMapper;
import com.qmth.teachcloud.mark.service.MarkGroupService;
import com.qmth.teachcloud.mark.service.MarkUserClassService;
import com.qmth.teachcloud.mark.service.MarkUserGroupService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 评卷员班级 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkUserClassServiceImpl extends ServiceImpl<MarkUserClassMapper, MarkUserClass> implements MarkUserClassService {
    @Resource
    MarkUserGroupService markUserGroupService;

    @Resource
    MarkStudentService markStudentService;

    @Resource
    MarkGroupService markGroupService;

    @Resource
    MarkQuestionService markQuestionService;

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkUserClass> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public List<MarkUserClassDto> listMarkUserClassByExamIdAndPaperNumber(Long examId, String paperNumber) {
        // 课程下所有班级数据
        List<String> paperNumberClassList = markStudentService.listClassByExamIdAndCourseCode(examId, paperNumber);
        // 课程下所有分组
        List<MarkGroup> markGroupList = markGroupService.listGroupByExamIdAndPaperNumber(examId, paperNumber);
        List<MarkGroupDto> markGroupDtoList = markGroupList.stream().map(m -> {
            MarkGroupDto markGroupDto = new MarkGroupDto();
            markGroupDto.setGroupNumber(m.getNumber());
            markGroupDto.setMarkers(markUserGroupService.listGroupUserByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, m.getNumber()));
            markGroupDto.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, m.getNumber()));
            return markGroupDto;
        }).collect(Collectors.toList());


        List<MarkUserClassDto> markUserClassDtoList = new ArrayList<>();
        for (String className : paperNumberClassList) {
            for (MarkGroupDto markGroupDto : markGroupDtoList) {
                MarkUserClassDto markUserClassDto = new MarkUserClassDto();
                markUserClassDto.setClassName(className);
                markUserClassDto.setGroupNumber(markGroupDto.getGroupNumber());
                markUserClassDto.setGroupQuestions(markGroupDto.getGroupQuestions());
                markUserClassDto.setMarkerList(markGroupDto.getMarkers());
                // 班级+分组绑定的评卷员集合
                markUserClassDto.setClassMarkerList(this.listClassMarkerByExamIdAndPaperNumberAndGroupNumberAndClassName(examId, paperNumber, markGroupDto.getGroupNumber(), className));
                markUserClassDtoList.add(markUserClassDto);
            }
        }
        return markUserClassDtoList;
    }

    @Override
    public List<MarkUser> listClassMarkerByExamIdAndPaperNumberAndGroupNumberAndClassName(Long examId, String paperNumber, Integer groupNumber, String className) {
        return this.baseMapper.listClassMarkerByExamIdAndPaperNumberAndGroupNumberAndClassName(examId, paperNumber, groupNumber, className);
    }

    @Transactional
    @Override
    public boolean saveMarkClassUser(MarkClassUserParams markClassUserParams) {
        Long examId = markClassUserParams.getExamId();
        String paperNumber = markClassUserParams.getPaperNumber();
        String className = markClassUserParams.getClassName();
        Integer groupNumber = markClassUserParams.getGroupNumber();
        List<MarkUserClass> markUserClasses = new ArrayList<>();
        for (MarkUser markUser : markClassUserParams.getClassMarkerList()) {
            MarkUserClass markUserClass = new MarkUserClass();
            markUserClass.setExamId(examId);
            markUserClass.setPaperNumber(paperNumber);
            markUserClass.setGroupNumber(groupNumber);
            markUserClass.setClassName(className);
            markUserClass.setUserId(markUser.getUserId());
            markUserClasses.add(markUserClass);
        }
        this.deleteByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber);
        this.saveBatch(markUserClasses);
        return true;
    }

    @Override
    public boolean deleteByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        UpdateWrapper<MarkUserClass> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber)
                .eq(MarkUserClass::getGroupNumber, groupNumber);
        return this.remove(updateWrapper);
    }

    @Override
    public List<MarkUserClass> listByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkUserClass> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<MarkGroupClassProgressDto> pageClassByExamIdAndPaperNumber(Page<MarkGroupClassProgressDto> page, Long examId, String paperNumber) {
        return this.baseMapper.pageClassByExamIdAndPaperNumber(page, examId, paperNumber);
    }
}
