package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.dto.mark.entrance.MarkEntranceDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkQualityChartDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkQualityDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkUserGroupProgressDto;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.entity.MarkUserGroup;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.mapper.MarkUserGroupMapper;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 评卷员表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkUserGroupServiceImpl extends ServiceImpl<MarkUserGroupMapper, MarkUserGroup> implements MarkUserGroupService {

    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkSyncService markSyncService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private MarkService markService;
    @Resource
    private LockService lockService;

    @Override
    public IPage<MarkEntranceDto> listEntranceGroup(Long examId, String courseCode, String paperNumber, Integer pageNumber, Integer pageSize) {
        if (Objects.isNull(examId)) {
            throw ExceptionResultEnum.EXAM_ID_IS_NULL.exception();
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<MarkUserGroup> page = new Page<>(pageNumber, pageSize);
        IPage<MarkEntranceDto> markEntranceDtoIPage = this.baseMapper.listEntranceGroup(page, examId, courseCode, paperNumber, sysUser.getId());
        for (MarkEntranceDto record : markEntranceDtoIPage.getRecords()) {
            // 评卷进度
            if (record.getLibraryCount().intValue() == 0) {
                record.setPercent(0D);
            } else {
                BigDecimal bigDecimal = new BigDecimal(record.getMarkedCount()).divide(new BigDecimal(record.getLibraryCount()), 2, RoundingMode.HALF_UP);
                record.setPercent(bigDecimal.doubleValue());
            }

            // 分组题目
            record.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(record.getExamId(), record.getPaperNumber(), record.getGroupNumber()));
        }
        return markEntranceDtoIPage;
    }

    @Override
    public List<MarkUser> listGroupUserByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        return this.baseMapper.listGroupUserByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber);
    }

    @Override
    public IPage<MarkUserGroupProgressDto> pageGroupUserByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber, String loginName, Integer pageNumber, Integer pageSize) {
        Page<MarkUserGroupProgressDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkUserGroupProgressDto> markUserGroupProgressDtoIPage = this.baseMapper.pageGroupUserByExamIdAndPaperNumberAndGroupNumber(page, examId, paperNumber, groupNumber, loginName);
        for (MarkUserGroupProgressDto record : markUserGroupProgressDtoIPage.getRecords()) {
            record.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, record.getGroupNumber()));
            List<MarkTask> markTaskList = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndUserId(examId, paperNumber, record.getGroupNumber(), record.getUserId());
            record.setTaskCount(markTaskList.size());
            record.setMarkedCount(markTaskService.markedCount(markTaskList));
            MarkGroup markGroup = markGroupService.getByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, record.getGroupNumber());
            record.setCurrentCount(markService.applyCurrentCount(markGroup, record.getMarkUserGroupId()));
        }
        return markUserGroupProgressDtoIPage;
    }

    @Override
    public void addMarkUserGroup(Long examId, String paperNumber, Integer groupNumber, List<Long> userIds) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isNotEmpty(userIds)) {
            List<MarkUserGroup> markUserGroups = new ArrayList<>();
            for (Long userId : userIds) {
                MarkUserGroup markUserGroup = this.getByExamIdAndPaperNumberAndGroupNumberAndUserId(examId, paperNumber, groupNumber, userId);
                if (markUserGroup != null) {
                    markUserGroups.add(new MarkUserGroup(examId, paperNumber, groupNumber, userId, markPaper.getMarkMode()));
                }
            }
            this.saveBatch(markUserGroups);
        }
    }

    @Override
    public void setTaskCount(List<Long> markUserGroupIds, Integer taskCount) {
        for (Long markUserGroupId : markUserGroupIds) {
            MarkUserGroup markUserGroup = this.getById(markUserGroupId);
            if (markUserGroup != null) {
                markUserGroup.setTopCount(taskCount);
                this.updateById(markUserGroup);
            }
        }
    }

    @Override
    public MarkUserGroup getByExamIdAndPaperNumberAndGroupNumberAndUserId(Long examId, String paperNumber, Integer groupNumber, Long userId) {
        QueryWrapper<MarkUserGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserGroup::getExamId, examId)
                .eq(MarkUserGroup::getPaperNumber, paperNumber)
                .eq(MarkUserGroup::getGroupNumber, groupNumber)
                .eq(MarkUserGroup::getUserId, userId);
        return this.getOne(queryWrapper);
    }

    @Override
    public void deleteMarker(Long markUserGroupId) {
        MarkUserGroup markUserGroup = this.getById(markUserGroupId);
        if (markUserGroup != null) {
            throw ExceptionResultEnum.ERROR.exception("评卷员不存在");
        }
        int count = markTaskService.countByExamIdAndPaperNumberAndGroupNumberAndUserId(markUserGroup.getExamId(), markUserGroup.getPaperNumber(), markUserGroup.getGroupNumber(), markUserGroup.getUserId());
        if (count > 0) {
            throw ExceptionResultEnum.ERROR.exception("删除评卷员失败，该评卷员已开始评卷");
        } else {
            this.removeById(markUserGroupId);
        }
    }

    @Override
    public void releaseMarker(List<Long> markUserGroupIds) {
        for (Long markUserGroupId : markUserGroupIds) {
            MarkUserGroup markUserGroup = this.getById(markUserGroupId);
            if (markUserGroup != null) {
                try {
                    lockService.waitlock(LockType.MARK_USER_GROUP, markUserGroupId);
                    markService.releaseByMarkUserGroup(markUserGroup);
                } catch (Exception e) {
                    log.error("release marker error", e);
                } finally {
                    lockService.unlock(LockType.MARK_USER_GROUP, markUserGroupId);
                }
            }
        }
    }

    @Override
    public boolean resetMarker(Long markUserGroupId) {
        MarkUserGroup markUserGroup = this.getById(markUserGroupId);
        if (markUserGroup != null) {
            if (lockService.trylock(LockType.MARKER_RESET, markUserGroupId)) {
                markSyncService.markerResetSync(markUserGroup);
                return true;
            } else {
                return false;
            }
        }
        return true;
    }

    @Override
    public void resetById(Long id) {
        UpdateWrapper<MarkUserGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkUserGroup::getFinishCount, null)
                .set(MarkUserGroup::getAvgScore, null)
                .set(MarkUserGroup::getAvgSpeed, null)
                .set(MarkUserGroup::getStdevScore, null)
                .eq(MarkUserGroup::getId, id);
        this.update(updateWrapper);
    }

    @Override
    public IPage<MarkQualityDto> pageQuality(Long examId, String paperNumber, Integer groupNumber, Long userId, Integer pageNumber, Integer pageSize) {
        Page<MarkQualityDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkQualityDto> markQualityDtoIPage = this.baseMapper.pageQuality(page, examId, paperNumber, groupNumber, userId);
        for (MarkQualityDto qualityDto : markQualityDtoIPage.getRecords()) {
            if (qualityDto.getUserId() != null) {
                SysUser sysUser = sysUserService.getByUserId(qualityDto.getUserId());
                qualityDto.setLoginName(sysUser.getLoginName());
                qualityDto.setName(sysUser.getRealName());
                qualityDto.setOrgName(sysUser.getOrgName());
                String percent = qualityDto.getFinishCount() > 0 ? new DecimalFormat("####.###").format((qualityDto.getFinishCount() - qualityDto.getHeaderFinishCount()) * 100.0 / qualityDto.getFinishCount()) : "0";
                qualityDto.setAdoptionRate(Double.parseDouble(percent));
            }
        }
        return markQualityDtoIPage;
    }

    @Override
    public void updateQuality(Long examId, String paperNumber, Integer groupNumber) {
        if (groupNumber == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择评阅题目");
        }
        MarkGroup markGroup = markGroupService.getByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber);
        if (markGroup != null) {
            String lockKey = markService.getGroupKey(markGroup);
            if (lockService.trylock(LockType.BATCH_QUALITY, lockKey)) {
                markSyncService.updateQuality(this.listByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber), lockKey);
            }
        }
    }

    @Override
    public List<MarkUserGroup> listByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        QueryWrapper<MarkUserGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserGroup::getExamId, examId)
                .eq(MarkUserGroup::getPaperNumber, paperNumber)
                .eq(MarkUserGroup::getGroupNumber, groupNumber)
                .orderByAsc(MarkUserGroup::getId);
        return this.list(queryWrapper);
    }

    @Override
    public void updateQualityById(Long markUserGroupId, int finishCount, int validCount, double avgSpeed, double avgScore, double stdevScore) {
        UpdateWrapper<MarkUserGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkUserGroup::getFinishCount, finishCount)
                .set(MarkUserGroup::getValidCount, validCount)
                .set(MarkUserGroup::getAvgSpeed, avgSpeed)
                .set(MarkUserGroup::getAvgScore, avgScore)
                .set(MarkUserGroup::getStdevScore, stdevScore)
                .eq(MarkUserGroup::getId, markUserGroupId);
        this.update(updateWrapper);
    }

    @Override
    public List<MarkQualityChartDto> listQualityChart(Long examId, String paperNumber, Integer groupNumber) {
        if (groupNumber == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择对应的科目及分组，再查看给分曲线");
        }
        List<MarkQualityChartDto> markQualityChartDtoList = new ArrayList<>();
        List<MarkUserGroup> markUserGroupList = this.listByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, groupNumber);
        Double totalAvgScore = markUserGroupList.stream().collect(Collectors.averagingDouble(MarkUserGroup::getAvgScore));
        for (MarkUserGroup markUserGroup : markUserGroupList) {
            MarkQualityChartDto markQualityChartDto = new MarkQualityChartDto();
            markQualityChartDto.setUserId(markUserGroup.getUserId());
            SysUser sysUser = sysUserService.getById(markUserGroup.getUserId());
            markQualityChartDto.setLoginName(sysUser.getLoginName());
            markQualityChartDto.setName(sysUser.getRealName());
            markQualityChartDto.setTotalAvgScore(Double.parseDouble(new DecimalFormat("####.###").format(totalAvgScore)));
            markQualityChartDto.setAvgScore(markUserGroup.getAvgScore());
            markQualityChartDto.setMaxScore(markUserGroup.getMaxScore());
            markQualityChartDto.setMinScore(markUserGroup.getMinScore());
            markQualityChartDtoList.add(markQualityChartDto);
        }
        return markQualityChartDtoList;
    }
}
