package com.qmth.teachcloud.mark.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.constraints.NotNull;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.scan.ConditionType;
import com.qmth.teachcloud.common.enums.scan.OmrField;
import com.qmth.teachcloud.mark.bean.OmrTaskItem;
import com.qmth.teachcloud.mark.bean.OmrTaskPage;
import com.qmth.teachcloud.mark.dto.OmrTaskDto;
import com.qmth.teachcloud.mark.dto.OmrTaskPageDto;
import com.qmth.teachcloud.mark.dto.ScanOmrTaskDto;
import com.qmth.teachcloud.mark.dto.ScanOmrTaskPageDto;
import com.qmth.teachcloud.mark.dto.ScanOmrTaskResultDto;
import com.qmth.teachcloud.mark.dto.ScanOmrTaskResultPageDto;
import com.qmth.teachcloud.mark.dto.ScanOmrTaskSaveDto;
import com.qmth.teachcloud.mark.dto.ScanOmrTaskStatusDto;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.entity.ScanOmrTask;
import com.qmth.teachcloud.mark.entity.ScanPaper;
import com.qmth.teachcloud.mark.entity.ScanPaperPage;
import com.qmth.teachcloud.mark.entity.ScanStudentPaper;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.enums.OmrTaskStatus;
import com.qmth.teachcloud.mark.mapper.ScanOmrTaskMapper;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.mark.service.ScanAnswerCardService;
import com.qmth.teachcloud.mark.service.ScanOmrTaskService;
import com.qmth.teachcloud.mark.service.ScanPaperPageService;
import com.qmth.teachcloud.mark.service.ScanPaperService;
import com.qmth.teachcloud.mark.service.ScanStudentPaperService;
import com.qmth.teachcloud.mark.utils.TaskLock;
import com.qmth.teachcloud.mark.utils.TaskLockUtil;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class ScanOmrTaskServiceImpl extends ServiceImpl<ScanOmrTaskMapper, ScanOmrTask> implements ScanOmrTaskService {
	@Autowired
	private ScanStudentPaperService studentPaperService;
	@Autowired
	private MarkStudentService studentService;
	@Autowired
	private ScanPaperService paperService;
	@Autowired
	private ScanPaperPageService pageService;
	@Autowired
	private ConcurrentService concurrentService;
	@Autowired
	private ScanAnswerCardService answerCardService;

	private static final String OMR_SUSPECT = "?";

	private static final String OMR_BLANK = "#";

	@Override
	@Transactional
	public ScanOmrTask buildTask(ConditionType c, Long studentId) {
		List<ScanStudentPaper> paperIds = clearAndToDispose(c, studentId);
		if (CollectionUtils.isEmpty(paperIds)) {
			return null;
		}
		MarkStudent student = studentService.getById(studentId);
		// 所有客观题空白
		boolean allObjectiveBlank = isAllBlank(paperIds);
		List<OmrTaskDto> retList = new ArrayList<>();
		boolean multi_blank = false;
		boolean single_blank = false;
		int multi_blank_count = 0;
		int single_blank_count = 0;
		for (ScanStudentPaper spe : paperIds) {
			ScanPaper paper = paperService.getById(spe.getPaperId());
			if (paper == null) {
				throw new ParameterException("paper不存在");
			}
			Long paperId = spe.getPaperId();
			ScanOmrTask task = new ScanOmrTask();
			task.setCardNumber(paper.getCardNumber());
			task.setExamId(paper.getExamId());
			task.setConditions(c.toString());
			task.setPaperId(paperId);
			task.setPaperIndex(spe.getPaperIndex());
			task.setStatus(OmrTaskStatus.WAITING);
			task.setStudentId(spe.getStudentId());
			task.setPages(new ArrayList<>());
			List<ScanPaperPage> pageList = pageService.listByPaperId(paperId);
			OmrTaskDto taskDto = new OmrTaskDto(task);
			retList.add(taskDto);
			// 循环试卷
			for (ScanPaperPage pageEntity : pageList) {
				OmrTaskPageDto pageDto = new OmrTaskPageDto(pageEntity.getPageIndex());
				taskDto.getPage().add(pageDto);
				List<OmrTaskItem> items = pageDto.getItems();
				// 分组类型
				// 识别嫌疑
				if (ConditionType.FILL_SUSPECT.equals(c)) {
					for (int i = 0; pageEntity.getQuestion() != null && pageEntity.getQuestion().getResult() != null
							&& i < pageEntity.getQuestion().getResult().size(); i++) {
						String result = pageEntity.getQuestion().getResult().get(i);
						if (result != null && result.startsWith(OMR_SUSPECT)) {
							OmrTaskItem item = new OmrTaskItem();
							item.setIndex(i + 1);
							item.setField(OmrField.QUESTION);
							item.setOmrResult(result);
							items.add(item);
						}
					}
				}
				// 卷型空选
//                if (ConditionType.PAPER_TYPE_BLANK.equals(c)) {
//                    if (pageEntity.getPaperType() != null
//                            && pageEntity.getPaperType().getResult().contains(OMR_BLANK)) {
//                        OmrTaskItem item = new OmrTaskItem();
//                        item.setIndex(1);
//                        item.setField(OmrField.PAPER_TYPE);
//                        item.setOmrResult(pageEntity.getPaperType().getResult());
//                        items.add(item);
//                    }
//                }
				// 卷型多选
//                if (ConditionType.PAPER_TYPE_EXCEED.equals(c)) {
//                    if ((!pageEntity.getPaperType().getResult().contains(OMR_SUSPECT)
//                            && pageEntity.getPaperType().getResult().length() > 2)
//                            || (pageEntity.getPaperType().getResult().contains(OMR_SUSPECT)
//                                    && pageEntity.getPaperType().getResult().length() > 1)) {
//                        OmrTaskItem item = new OmrTaskItem();
//                        item.setField(OmrField.PAPER_TYPE);
//                        item.setOmrResult(pageEntity.getPaperType().getResult());
//                        items.add(item);
//                    }
//                }
//                if (ConditionType.QUESTION_MULTI_BLANK.equals(c)) {
//                    for (int i = 0; pageEntity.getQuestion() != null && pageEntity.getQuestion().getResult() != null
//                            && i < pageEntity.getQuestion().getResult().size(); i++) {
//                        String result = pageEntity.getQuestion().getResult().get(i);
//                        PaperStructure question = structureService.getPaperStructure(paper.getExamId(),
//                                paper.getCardNumber(), spe.getPaperNumber(), pageEntity.getPageIndex(), i + 1,
//                                OmrField.QUESTION);
//                        if (!question.getSingle() && result != null && result.contains(OMR_BLANK)) {
//                            OmrTaskItem item = new OmrTaskItem();
//                            item.setIndex(i + 1);
//                            item.setField(OmrField.QUESTION);
//                            item.setOmrResult(result);
//                            pageDto.getPageMultiBlankItems().add(item);
//                            multi_blank_count++;
//                            if (multi_blank_count > condition.getValue()) {
//                                multi_blank = true;
//                            }
//                        }
//                    }
//                }
//                if (ConditionType.QUESTION_SINGLE_BLANK.equals(c)
//                        && ScanStatus.SCANNED.equals(student.getStatus()) && !allObjectiveBlank) {
//                    for (int i = 0; pageEntity.getQuestion() != null && pageEntity.getQuestion().getResult() != null
//                            && i < pageEntity.getQuestion().getResult().size(); i++) {
//                        String result = pageEntity.getQuestion().getResult().get(i);
//                        PaperStructure question = structureService.getPaperStructure(paper.getExamId(),
//                                paper.getCardNumber(), spe.getPaperNumber(), pageEntity.getPageIndex(), i + 1,
//                                OmrField.QUESTION);
//                        if (question.getSingle() && result != null && result.contains(OMR_BLANK)) {
//                            OmrTaskItem item = new OmrTaskItem();
//                            item.setIndex(i + 1);
//                            item.setField(OmrField.QUESTION);
//                            item.setOmrResult(result);
//                            pageDto.getPageSingleBlankItems().add(item);
//                            single_blank_count++;
//                            multi_blank_count++;
//                            if (single_blank_count > condition.getValue()) {
//                                single_blank = true;
//                            }
//                        }
//                    }
//                }
//                if (ConditionType.QUESTION_SINGLE_EXCEED.equals(c)
//                        && ScanStatus.SCANNED.equals(student.getStatus()) && !allObjectiveBlank) {
//                    for (int i = 0; pageEntity.getQuestion() != null && pageEntity.getQuestion().getResult() != null
//                            && i < pageEntity.getQuestion().getResult().size(); i++) {
//                        String result = pageEntity.getQuestion().getResult().get(i);
//                        PaperStructure question = structureService.getPaperStructure(paper.getExamId(),
//                                paper.getCardNumber(), spe.getPaperNumber(), pageEntity.getPageIndex(), i + 1,
//                                OmrField.QUESTION);
//                        if (question.getSingle() && result != null && result.length() > 1) {
//                            OmrTaskItem item = new OmrTaskItem();
//                            item.setIndex(i + 1);
//                            item.setField(OmrField.QUESTION);
//                            item.setOmrResult(result);
//                            items.add(item);
//                        }
//                    }
//                }
			}
		}
		if (retList.size() == 0) {
			return null;
		}
		for (OmrTaskDto dto : retList) {
			for (OmrTaskPageDto pageDto : dto.getPage()) {
				if (multi_blank) {
					pageDto.getItems().addAll(pageDto.getPageMultiBlankItems());
				}
				if (single_blank) {
					pageDto.getItems().addAll(pageDto.getPageSingleBlankItems());
				}
				if (pageDto.getItems().size() > 0) {
					OmrTaskPage page = new OmrTaskPage();
					page.setItems(pageDto.getItems());
					page.setIndex(pageDto.getPageIndex());
					dto.getTask().getPages().add(page);
				}
			}
			if (dto.getTask().getPages().size() > 0) {
				return dto.getTask();
			}
		}
		return null;
	}

	private List<ScanStudentPaper> clearAndToDispose(ConditionType c, Long studentId) {
		List<ScanStudentPaper> spes = studentPaperService.findByStudentId(studentId);
		if (CollectionUtils.isEmpty(spes)) {
			return spes;
		}
		List<ScanOmrTask> tasks = getByStudent(c, studentId);
		if (CollectionUtils.isEmpty(tasks)) {
			return spes;
		}
		Map<Integer, Long> speMap = new HashMap<>();
		for (ScanStudentPaper spe : spes) {
			speMap.put(spe.getPaperIndex(), spe.getPaperId());
		}
		Map<Integer, Long> taskMap = new HashMap<>();
		for (ScanOmrTask t : tasks) {
			taskMap.put(t.getPaperIndex(), t.getPaperId());
		}
		if (ConditionType.FILL_SUSPECT.equals(c)) {
			List<ScanStudentPaper> ret = new ArrayList<>();
			for (ScanStudentPaper spe : spes) {
				Long paperId = taskMap.get(spe.getPaperIndex());
				if (paperId == null) {// 没有task的直接创建
					ret.add(spe);
				} else if (paperId.longValue() != spe.getPaperId().longValue()) {// 和task不一致的删除并创建
					delete(c, studentId, paperId);
					ret.add(spe);
				}
			}
			for (ScanOmrTask t : tasks) {
				if (speMap.get(t.getPaperIndex()) == null) {// 不在绑定关系的task删除
					delete(c, studentId, t.getPaperId());
				}
			}
			return ret;
		} else {
			if (spes.size() != tasks.size()) {// 数量不一致的删除重建
				delete(c, studentId);
				return spes;
			}
			for (ScanOmrTask t : tasks) {
				if (!t.getPaperId().equals(speMap.get(t.getPaperIndex()))) {// 有一个不一致的删除重建
					delete(c, studentId);
					return spes;
				}
			}
			return null;// 完全一致的不处理
		}
	}

	private List<ScanOmrTask> getByStudent(ConditionType c, Long studentId) {
		QueryWrapper<ScanOmrTask> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<ScanOmrTask> lw = wrapper.lambda();
		lw.eq(ScanOmrTask::getConditions, c);
		lw.eq(ScanOmrTask::getStudentId, studentId);
		return baseMapper.selectList(wrapper);
	}

	private void delete(ConditionType c, Long studentId, Long paperId) {
		UpdateWrapper<ScanOmrTask> wrapper = new UpdateWrapper<>();
		LambdaUpdateWrapper<ScanOmrTask> lw = wrapper.lambda();
		lw.eq(ScanOmrTask::getConditions, c);
		lw.eq(ScanOmrTask::getStudentId, studentId);
		lw.eq(ScanOmrTask::getPaperId, paperId);
		this.baseMapper.delete(wrapper);
	}

	private void delete(ConditionType c, Long studentId) {
		UpdateWrapper<ScanOmrTask> wrapper = new UpdateWrapper<>();
		LambdaUpdateWrapper<ScanOmrTask> lw = wrapper.lambda();
		lw.eq(ScanOmrTask::getConditions, c);
		lw.eq(ScanOmrTask::getStudentId, studentId);
		this.baseMapper.delete(wrapper);
	}

	private boolean isAllBlank(List<ScanStudentPaper> paperIds) {
		for (ScanStudentPaper spe : paperIds) {
			ScanPaper paper = paperService.getById(spe.getPaperId());
			if (paper == null) {
				throw new ParameterException("paper不存在");
			}
			List<ScanPaperPage> pageList = pageService.listByPaperId(spe.getPaperId());
			for (ScanPaperPage pageEntity : pageList) {
				for (int i = 0; pageEntity.getQuestion() != null && pageEntity.getQuestion().getResult() != null
						&& i < pageEntity.getQuestion().getResult().size(); i++) {
					String result = pageEntity.getQuestion().getResult().get(i);
					if (!"#".equals(result)) {
						return false;
					}
				}
			}
		}
		return true;
	}

	@Override
	public ScanOmrTaskDto getTask(Long examId, Long userId) {
		int retry = 0;
		ScanOmrTaskDto task = null;
		while (task == null) {
			List<ScanOmrTask> list = this.findUnMarked(examId, retry * 20, 20, OmrTaskStatus.WAITING);
			if (list.isEmpty()) {
				break;
			}
			for (ScanOmrTask t : list) {
				if (this.apply(t, userId.toString())) {
					task = toTaskVo(t, false);
					break;
				}
			}
			if (task == null) {
				retry++;
			}
		}
		if (task == null) {
			throw ExceptionResultEnum.ERROR.exception("没有识别对照任务");
		}
		return task;
	}

	private ScanOmrTaskDto toTaskVo(ScanOmrTask task, boolean history) {
		ScanOmrTaskDto vo = new ScanOmrTaskDto();
		MarkStudent student = studentService.getById(task.getStudentId());
		vo.setCardNumber(task.getCardNumber());
		vo.setStudentCode(student.getStudentCode());
		vo.setId(task.getId());
		vo.setName(student.getStudentName());
		vo.setPaperId(task.getPaperId());
		vo.setPaperIndex(task.getPaperIndex());
		vo.setSubjectCode(student.getCourseCode());
		vo.setSubjectName(student.getCourseName());
		List<ScanOmrTaskPageDto> pages = new ArrayList<>();
		for (OmrTaskPage taskPage : task.getPages()) {
			ScanOmrTaskPageDto page = new ScanOmrTaskPageDto();
			Map<Integer, List<String>> question = new HashMap<>();
			Map<Integer, List<String>> selective = new HashMap<>();
			for (OmrTaskItem item : taskPage.getItems()) {
				if (OmrField.ABSENT.equals(item.getField())) {
					page.setAbsent(getBooleanItem(item, history));
				}
				if (OmrField.BREACH.equals(item.getField())) {
					page.setBreach(getBooleanItem(item, history));
				}
				if (OmrField.PAPER_TYPE.equals(item.getField())) {
					page.setPaperType(getStringItem(item, history));
				}
				if (OmrField.QUESTION.equals(item.getField())) {
					List<String> content = getStringItem(item, history);
					if (content != null) {
						question.put(item.getIndex(), content);
					}
				}
				if (OmrField.SELECTIVE.equals(item.getField())) {
					List<String> content = getStringItem(item, history);
					if (content != null) {
						selective.put(item.getIndex(), content);
					}
				}
			}
			if (question.size() > 0) {
				page.setQuestion(question);
			}
			if (selective.size() > 0) {
				page.setSelective(selective);
			}
			// 有需要仲裁的数据才返回结构
			if (page.getAbsent() != null || page.getBreach() != null || page.getPaperType() != null
					|| page.getQuestion() != null || page.getSelective() != null) {
				page.setIndex(taskPage.getIndex());
				ScanPaperPage p = pageService.findPaperIdAndIndex(task.getPaperId(), taskPage.getIndex());
				page.setRecogData(p.getRecogData());
				page.setUri(p.getSheetPath());
				pages.add(page);
			}
		}
		vo.setPages(pages);
		return vo;
	}

	private List<String> getStringItem(OmrTaskItem item, boolean history) {
		List<String> value = new ArrayList<>();
		value.add(item.getOmrResult());
		if (history && item.getFirstResult() != null && item.getSecondResult() == null) {
			value.add(item.getFirstResult());
		} else if (history && item.getSecondResult() != null) {
			value.add(item.getSecondResult());
		}
		return value;
	}

	private List<Boolean> getBooleanItem(OmrTaskItem item, boolean history) {
		List<Boolean> value = new ArrayList<>();
		value.add(Boolean.valueOf(item.getOmrResult()));
		if (history && item.getFirstResult() != null && item.getSecondResult() == null) {
			value.add(Boolean.valueOf(item.getFirstResult()));
		} else if (history && item.getSecondResult() != null) {
			value.add(Boolean.valueOf(item.getSecondResult()));
		}
		return value;
	}

	@Override
	public List<ScanOmrTask> findUnMarked(Long examId, int pageNumber, int pageSize, OmrTaskStatus status) {
		return this.baseMapper.findUnMarked(examId, pageNumber, pageSize, status);
	}

	@Override
	public boolean apply(ScanOmrTask t, String userId) {
		TaskLock taskLock = TaskLockUtil.getOmrTask(t.getExamId().toString());
		boolean lock = taskLock.add(t.getId(), 1, userId);
		// 上锁失败直接返回
		if (!lock) {
			return false;
		}
		// 重复校验任务状态
		if (t.getStatus().equals(OmrTaskStatus.WAITING)) {
			return true;
		} else {
			taskLock.remove(t.getId(), 1);
			return false;
		}
	}

	@Override
	public boolean hasApplied(ScanOmrTask t, String userId) {
		TaskLock taskLock = TaskLockUtil.getOmrTask(t.getExamId().toString());
		return taskLock.exist(t.getId(), 1, userId);
	}

	@Override
	@Transactional
	public ScanOmrTaskSaveDto submitTask(@NotNull ScanOmrTaskResultDto result, @NotNull SysUser user) {
		ScanOmrTask task = this.getById(result.getId());
		if (task == null) {
			throw new ParameterException("任务不存在");
		}
		// if (TaskStatus.PROCESSED.equals(task.getStatus())) {
		// throw new ParameterException("任务已处理");
		// }
		if (!task.getUserId().equals(user.getId())) {
			throw new ParameterException("任务非本人领取");
		}
		if (OmrTaskStatus.WAITING.equals(task.getStatus()) && !hasApplied(task, user.getId())) {
			throw new ParameterException("任务非本人领取");
		}
		concurrentService.getReadWriteLock(LockType.STUDENT + "-" + task.getStudentId()).writeLock().lock();

		for (OmrTaskPage page : task.getPages()) {
			ScanOmrTaskResultPageDto pageVo = result.findPage(page.getIndex());
			if (pageVo == null) {
				throw new ParameterException("page[" + page.getIndex() + "]不存在");
			}
			for (OmrTaskItem item : page.getItems()) {
				if (OmrField.ABSENT.equals(item.getField())) {
					if (pageVo.getAbsent() == null) {
						throw new ParameterException("page[" + page.getIndex() + "].absent不存在");
					}
					item.setFirstResult(pageVo.getAbsent().toString());
				}
				if (OmrField.BREACH.equals(item.getField())) {
					if (pageVo.getBreach() == null) {
						throw new ParameterException("page[" + page.getIndex() + "].breach不存在");
					}
					item.setFirstResult(pageVo.getBreach().toString());
				}
				if (OmrField.QUESTION.equals(item.getField())) {
					String content = pageVo.getQuestion() != null ? pageVo.getQuestion().get(item.getIndex()) : null;
					if (content == null) {
						throw new ParameterException(
								"page[" + page.getIndex() + "].question[" + item.getIndex() + "]不存在");
					}
					item.setFirstResult(content);
				}
				if (OmrField.SELECTIVE.equals(item.getField())) {
					String content = pageVo.getSelective() != null ? pageVo.getSelective().get(item.getIndex()) : null;
					if (content == null) {
						throw new ParameterException(
								"page[" + page.getIndex() + "].selective[" + item.getIndex() + "]不存在");
					}
					item.setFirstResult(content);
				}
			}
		}
		task.setStatus(OmrTaskStatus.PROCESSED);

		updatePaperResult(result, task.getPaperId(), user.getId());
		task.setUserId(user.getId());
		task.setUpdateTime(System.currentTimeMillis());
		this.saveOrUpdate(task);
		this.releaseByTask(task);
		concurrentService.getReadWriteLock(LockType.STUDENT + "-" + task.getStudentId()).writeLock().unlock();
		ScanOmrTaskSaveDto vo = new ScanOmrTaskSaveDto();
		vo.setId(result.getId());
		vo.setUpdateTime(System.currentTimeMillis());
		vo.setStatus(this.getStatus(task.getExamId(), user.getId()));
		return vo;
	}

	private void updatePaperResult(ScanOmrTaskResultDto result, Long paperId, Long userId) {
		ScanPaper paper = paperService.getById(paperId);
		ScanStudentPaper sp = studentPaperService.findByPaperId(paperId);
		if (paper == null) {
			throw new ParameterException("未找到paper信息");
		}
		if (sp == null) {
			throw new ParameterException("paper未绑定考生");
		}
		List<ScanPaperPage> pages = pageService.listByPaperId(paperId);
		if (CollectionUtils.isEmpty(pages)) {
			throw new ParameterException("未找到page信息");
		}
		for (ScanPaperPage page : pages) {
			ScanOmrTaskResultPageDto vo = result.findPage(page.getPageIndex());
			if (vo != null) {
				vo.update(page);
			}
		}
		paperService.savePaperAndPages(paper, pages);
		studentService.updateStudentByPaper(userId, sp.getStudentId(), false);
	}

	@Override
	public boolean hasApplied(ScanOmrTask t, Long userId) {
		TaskLock taskLock = TaskLockUtil.getOmrTask(t.getExamId().toString());
		return taskLock.exist(t.getId(), 1, userId.toString());
	}

	@Override
	public void releaseByTask(ScanOmrTask t) {
		TaskLock taskLock = TaskLockUtil.getOmrTask(t.getExamId().toString());
		taskLock.remove(t.getId(), 1);
	}

	@Override
	public ScanOmrTaskStatusDto getStatus(Long examId, Long userId) {
		ScanOmrTaskStatusDto status = new ScanOmrTaskStatusDto();
		status.setFinishCount(this.getCountByExamAndStatusAndUserId(examId, userId, OmrTaskStatus.PROCESSED));
		status.setTodoCount(this.getCountByExamAndStatus(examId, OmrTaskStatus.WAITING));
		return status;
	}

	@Override
	public int getCountByExamAndStatusAndUserId(Long examId, Long userId, OmrTaskStatus... status) {
		QueryWrapper<ScanOmrTask> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<ScanOmrTask> lw = wrapper.lambda();
		lw.and(param -> {
			for (OmrTaskStatus taskStatus : status) {
				param.or().eq(ScanOmrTask::getStatus, taskStatus);
			}
		});
		lw.eq(ScanOmrTask::getUserId, userId);
		lw.eq(ScanOmrTask::getExamId, examId);
		return this.count(wrapper);
	}

	@Override
	public int getCountByExamAndStatus(Long examId, OmrTaskStatus... status) {
		QueryWrapper<ScanOmrTask> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<ScanOmrTask> lw = wrapper.lambda();
		lw.and(param -> {
			for (OmrTaskStatus taskStatus : status) {
				param.or().eq(ScanOmrTask::getStatus, taskStatus);
			}
		});
		lw.eq(ScanOmrTask::getExamId, examId);
		return this.count(wrapper);
	}

	@Override
	public void releaseByUser(Long examId, Long userId) {
		TaskLock taskLock = TaskLockUtil.getOmrTask(examId.toString());
		taskLock.clear(userId.toString());
	}

	@Override
	public ScanOmrTaskDto history(Long examId, Long taskId, Long userId, Boolean next) {
		ScanOmrTask cur = null;
		if (taskId != null) {
			cur = this.getById(taskId);
		}
		QueryWrapper<ScanOmrTask> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<ScanOmrTask> lw = wrapper.lambda();
		lw.eq(ScanOmrTask::getExamId, examId);
		lw.eq(ScanOmrTask::getUserId, userId);
		lw.eq(ScanOmrTask::getStatus, OmrTaskStatus.PROCESSED);
		if (next != null && next) {
			if (taskId != null) {
				lw.gt(ScanOmrTask::getId, cur.getId());
			}
			lw.orderByAsc(ScanOmrTask::getId);
		} else {
			if (taskId != null) {
				lw.lt(ScanOmrTask::getId, cur.getId());
			}
			lw.orderByDesc(ScanOmrTask::getId);
		}
		wrapper.last("LIMIT 1");
		ScanOmrTask task = this.baseMapper.selectOne(wrapper);
		if (task == null) {
			throw ExceptionResultEnum.ERROR.exception("没有识别对照任务");
		}
		return toTaskVo(task, true);
	}

	@Transactional
	@Override
	public void deleteByStudentId(Long examId, Long studentId) {
		QueryWrapper<ScanOmrTask> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<ScanOmrTask> lw = wrapper.lambda();
		lw.eq(ScanOmrTask::getStudentId, studentId);
		List<ScanOmrTask> tasks = baseMapper.selectList(wrapper);
		if (CollectionUtils.isEmpty(tasks)) {
			return;
		}
		concurrentService.getReadWriteLock(LockType.STUDENT + "-" + studentId).writeLock().lock();
		this.removeByIds(tasks);
		concurrentService.getReadWriteLock(LockType.STUDENT + "-" + studentId).writeLock().unlock();
	}
	
	@Override
	public int getCount(Long examId,OmrTaskStatus status) {
		QueryWrapper<ScanOmrTask> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ScanOmrTask> lw = wrapper.lambda();
        lw.eq(ScanOmrTask::getExamId, examId);
        if(status!=null) {
        	lw.eq(ScanOmrTask::getStatus, status);
        }
        return baseMapper.selectCount(wrapper);
	}
}
