package com.qmth.teachcloud.mark.service.impl;

import java.util.List;

import javax.validation.constraints.NotNull;

import com.qmth.teachcloud.mark.dto.mark.score.StudentPaperDetailDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.entity.ScanAnswerCard;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.ScanStatus;
import com.qmth.teachcloud.mark.service.ScanAnswerCardService;
import com.qmth.teachcloud.mark.entity.ScanPaper;
import com.qmth.teachcloud.mark.entity.ScanPaperPage;
import com.qmth.teachcloud.mark.mapper.ScanPaperMapper;
import com.qmth.teachcloud.mark.service.ScanPaperPageService;
import com.qmth.teachcloud.mark.service.ScanPaperService;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class ScanPaperServiceImpl extends ServiceImpl<ScanPaperMapper, ScanPaper> implements ScanPaperService {
    @Autowired
    private ScanAnswerCardService answerCardService;
    @Autowired
    private ScanPaperPageService paperPageService;
	/**
     * 全量更新paper和所属的page集合，并刷新paper相关属性
     *
     * @param paper
     * @param pages
     */
    @Override
    @Transactional
    public void savePaperAndPages(@NotNull ScanPaper paper,@NotNull List<ScanPaperPage> pages ) {
        // 与卡格式校验
        ScanAnswerCard answerCard = answerCardService.findByExamAndNumber(paper.getExamId(), paper.getCardNumber());
        if (answerCard == null) {
        	throw ExceptionResultEnum.ERROR.exception("卡格式不存在, cardNumber=" + paper.getCardNumber());
        }
        if (pages.size() != (answerCard.getSinglePage() ? 1 : 2)) {
        	throw ExceptionResultEnum.ERROR.exception("paper.page数量与卡格式不符, cardNumber=" + paper.getCardNumber());
        }
        // 更新paper属性
        paper.setPageCount(pages.size());
        paper.setQuestionFilled(false);
        // 判断是否客观题已填涂
        for (ScanPaperPage page : pages) {
            paper.setQuestionFilled(paper.getQuestionFilled() || page.isQuestionFilled());
        }
        // 保存paper数据
        saveOrUpdate(paper);
        for (ScanPaperPage page : pages) {
        	//FIXME 
            // 验证原图与裁切图已上传
//            if (!fileStore.exist(page.getSheetPath())) {
//            	throw ExceptionResultEnum.ERROR.exception("原图不存在:" + page.getSheetPath());
//            }
            page.setPaperId(paper.getId());
        }
        // 保存page数据
        paperPageService.saveOrUpdateBatch(pages);
    }

    @Override
    public int getAssignedCount(Long examId,Boolean checked) {
		QueryWrapper<ScanPaper> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ScanPaper> lw = wrapper.lambda();
        lw.eq(ScanPaper::getExamId, examId);
        lw.eq(ScanPaper::getAssigned, true);
        if(checked!=null) {
        	lw.eq(ScanPaper::getChecked, checked);
        }
        return baseMapper.selectCount(wrapper);
	}

    @Override
    public List<StudentPaperDetailDto> listStudentPaperDetail(Long studentId) {
        return this.baseMapper.listStudentPaperDetail(studentId);
    }

}
