package com.qmth.teachcloud.task.job.service.impl;

import com.qmth.boot.redis.uid.RedisMachineService;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.bean.dto.MqDto;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkUserGroup;
import com.qmth.teachcloud.mark.service.MarkGroupService;
import com.qmth.teachcloud.mark.service.MarkService;
import com.qmth.teachcloud.mark.service.MarkUserGroupService;
import com.qmth.teachcloud.task.job.service.JobService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @Description: job service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/31
 */
@Service
public class JobServiceImpl implements JobService {
    private final static Logger log = LoggerFactory.getLogger(JobServiceImpl.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    SmsSendService smsSendService;

    @Resource
    BasicMessageService basicMessageService;

    @Resource
    ExamPaperStructureService examPaperStructureService;

    @Resource
    MarkPaperService markPaperService;

    @Resource
    MarkGroupService markGroupService;
    @Resource
    MarkUserGroupService markUserGroupService;
    @Resource
    MarkService markService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    RedisMachineService redisMachineService;

    /**
     * 重新生成pdf失败任务
     *
     * @throws IOException
     */
    @Override
    public void resetCreatePdfTask() throws IOException {
        printCommonService.resetCreatePdfTask();
    }

    @Override
    public void sendSmsExpireTask() {
        smsSendService.sendSmsExpireTask();
    }

    @Override
    public void sendSmsOverdueTask() {
        smsSendService.sendSmsOverdueTask();
    }

    @Override
    public void resendSmsTask() {
        basicMessageService.resendSmsTask();
    }

    @Override
    public void subjectCalculate() {
        examPaperStructureService.subjectCalculate();
    }

    @Override
    public void machineHeart() {
        redisUtil.set(SystemConstant.TASK_MACHINE_ID + redisMachineService.getMachineId(), redisMachineService.getMachineId(), 30, TimeUnit.SECONDS);
    }
    @Override
    public void updateMarkerQuality() {
        log.info("start auto-update marker");
        try {
            List<MarkPaper> markPaperList = markPaperService.listQualityMarkPaperByStatus(MarkPaperStatus.FORMAL, 0);
            for (MarkPaper markPaper : markPaperList) {
                List<MarkGroup> markGroups = markGroupService.listGroupByExamIdAndPaperNumber(markPaper.getExamId(), markPaper.getPaperNumber());
                for (MarkGroup markGroup : markGroups) {
                    List<MarkUserGroup> markUserGroups = markUserGroupService.listByExamIdAndPaperNumberAndGroupNumber(markPaper.getExamId(), markPaper.getPaperNumber(), markGroup.getNumber());
                    for (MarkUserGroup marker : markUserGroups) {
                        try {
                            if (markService.needUpdateQuality(marker, 30)) {
                                log.info("start auto-update markerId: " + marker.getId());
                                markService.updateQuality(marker);
                            }
                        } catch (Exception e) {
                            log.error("update marker quality error for markerId=" + marker.getId(), e);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("auto-update marker error", e);
        } finally {
            log.info("finish auto-update marker");
        }
    }

    /**
     * 组装job
     *
     * @param redisKey
     */
    @Override
    public void assembleJob(String redisKey) {
        Long size = redisUtil.getHashSize(redisKey);
        if (Objects.nonNull(size) && size.longValue() > 0) {
            log.info("redisKey:{}缓冲区的消息数为:{}", redisKey, size);
            Map map = redisUtil.getHashEntries(redisKey);
            map.forEach((k, v) -> {
                MqDto mqDto = (MqDto) v;
                redisUtil.sendMessage(mqDto.getTopic(), mqDto);
            });
        }
    }
}
