USE teachcloud_db;

-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
DROP TABLE IF EXISTS `basic_attachment`;
CREATE TABLE `basic_attachment` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(100) NOT NULL COMMENT '文件名',
  `type` varchar(10) NOT NULL COMMENT '文件类型',
  `size` int NOT NULL COMMENT '单位（KB）',
  `md5` varchar(32) NOT NULL COMMENT '文件md5值',
  `path` varchar(2000) NOT NULL COMMENT '存储路径',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `obj_id` bigint DEFAULT NULL COMMENT '关联业务id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='所有附件记录表';

-- ----------------------------
-- Table structure for basic_campus
-- ----------------------------
DROP TABLE IF EXISTS `basic_campus`;
CREATE TABLE `basic_campus` (
  `id` bigint NOT NULL COMMENT '校区id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `campus_name` varchar(64) DEFAULT NULL COMMENT '校区名称',
  `campus_code` varchar(64) DEFAULT NULL COMMENT '校区编码',
  `print_house_id` bigint DEFAULT NULL COMMENT '印刷室id',
  `enable` tinyint DEFAULT '1' COMMENT '是否可用',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `campus_schoolId_code_idx` (`school_id`,`campus_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础校区表';

-- ----------------------------
-- Table structure for basic_card_rule
-- ----------------------------
DROP TABLE IF EXISTS `basic_card_rule`;
CREATE TABLE `basic_card_rule` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(100) NOT NULL COMMENT '题卡规则名称',
  `exam_number_style` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，PASTE-粘贴条码，FILL-卡号填涂',
  `exam_number_digit` int DEFAULT NULL COMMENT '考号位数',
  `paper_type` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，FILL-卡号填涂',
  `exam_absent` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `write_sign` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `discipline` tinyint(1) DEFAULT NULL COMMENT '违纪填涂（1：是，0：否）',
  `required_fields` text NOT NULL COMMENT '必选字段',
  `extend_fields` text NOT NULL COMMENT '扩展字段',
  `title_rule` varchar(200) NOT NULL COMMENT '题卡标题规则',
  `attention` text NOT NULL COMMENT '注意事项',
  `objective_attention` text COMMENT '客观题注意事项',
  `subjective_attention` text COMMENT '主观题注意事项',
  `enable` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `fill_number` tinyint DEFAULT NULL COMMENT '卡号填涂位数',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='题卡规则';

-- ----------------------------
-- Table structure for basic_clazz
-- ----------------------------
DROP TABLE IF EXISTS `basic_clazz`;
CREATE TABLE `basic_clazz` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `major_id` bigint NOT NULL COMMENT '专业id',
  `clazz_code` varchar(50) DEFAULT NULL COMMENT '班级编号',
  `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
  `clazz_name` varchar(50) DEFAULT NULL COMMENT '班级名称',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否可用',
  `org_id` bigint DEFAULT NULL COMMENT '权限机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `clazz_schoolId_name_idx` (`school_id`,`major_id`,`clazz_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础班级表';

-- ----------------------------
-- Table structure for basic_college
-- ----------------------------
DROP TABLE IF EXISTS `basic_college`;
CREATE TABLE `basic_college` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL,
  `college_name` varchar(50) NOT NULL COMMENT '学院名称',
  `org_id` bigint NOT NULL COMMENT '数据权限机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `college_schoolId_name_idx` (`school_id`,`college_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学生学院表';

-- ----------------------------
-- Table structure for basic_course
-- ----------------------------
DROP TABLE IF EXISTS `basic_course`;
CREATE TABLE `basic_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `code` varchar(100) NOT NULL COMMENT '课程编码',
  `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
  `name` varchar(50) NOT NULL COMMENT '课程名称',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `teaching_room_id` bigint DEFAULT NULL COMMENT '教研室id',
  `enable` tinyint DEFAULT NULL,
  `historic_name` varchar(100) DEFAULT NULL COMMENT '历史课程名称',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `course_schoolId_code_idx` (`school_id`,`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='课程表';

-- ----------------------------
-- Table structure for basic_exam
-- ----------------------------
DROP TABLE IF EXISTS `basic_exam`;
CREATE TABLE `basic_exam` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL,
  `org_id` bigint NOT NULL,
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
  `name` varchar(100) DEFAULT NULL COMMENT '考试名称',
  `auto_sync_student` tinyint(1) DEFAULT '0' COMMENT '是否自动同步考务数据（默认否）',
  `category` varchar(45) DEFAULT NULL COMMENT '考试类型（ONLINE：线上，OFFLINE：线下）',
  `exam_model` varchar(45) DEFAULT NULL COMMENT '业务模式（MODEL1：电子交卷环节需要提交考务数据，MODEL2：电子交卷环节不需要提交考务数据，只输入印刷份数即可，MODEL3：电子交卷环节不需要提交考务数据）',
  `review` tinyint DEFAULT NULL COMMENT '是否启用入库审核：0-禁用，1-启用',
  `print_method` varchar(6) DEFAULT NULL COMMENT 'AUTO-自动，MANUAL-手动',
  `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试表';

-- ----------------------------
-- Table structure for basic_exam_rule
-- ----------------------------
DROP TABLE IF EXISTS `basic_exam_rule`;
CREATE TABLE `basic_exam_rule` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `required_fields` text COMMENT '必选字段',
  `extend_fields` text NOT NULL COMMENT '扩展字段',
  `exam_number_digit` int DEFAULT NULL COMMENT '考号位数',
  `sign_scope` text COMMENT '签到表字段可选范围',
  `package_scope` text COMMENT '卷袋贴字段可选范围',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务规则';

-- ----------------------------
-- Table structure for basic_major
-- ----------------------------
DROP TABLE IF EXISTS `basic_major`;
CREATE TABLE `basic_major` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `belong_org_id` bigint NOT NULL COMMENT '所属机构id',
  `name` varchar(100) NOT NULL COMMENT '专业名称',
  `code` varchar(100) DEFAULT NULL COMMENT '专业编码',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `major_schoolId_name_idx` (`school_id`,`name`,`belong_org_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='专业字典表';

-- ----------------------------
-- Table structure for basic_message
-- ----------------------------
DROP TABLE IF EXISTS `basic_message`;
CREATE TABLE `basic_message` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint NOT NULL COMMENT '消息接收人用户',
  `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
  `mobile_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
  `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `message_type` varchar(45) DEFAULT NULL COMMENT '消息类型',
  `business_operate` varchar(45) NOT NULL COMMENT '业务操作',
  `business_id` bigint DEFAULT NULL COMMENT '业务id',
  `template_code` varchar(45) NOT NULL COMMENT '消息模板代码',
  `variable_params` text COMMENT '变量参数内容',
  `template_content` mediumtext COMMENT '模板内容',
  `send_status` varchar(45) DEFAULT NULL COMMENT '消息发送状态',
  `send_result` varchar(200) DEFAULT NULL COMMENT '消息发送结果',
  `remark` text COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `resend_count` int DEFAULT NULL COMMENT '重试次数',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for basic_operation_log
-- ----------------------------
DROP TABLE IF EXISTS `basic_operation_log`;
CREATE TABLE `basic_operation_log` (
  `id` bigint NOT NULL COMMENT 'id',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '数据权限id',
  `detail` mediumtext NOT NULL COMMENT '日志详情',
  `level` varchar(32) DEFAULT NULL COMMENT '日志等级（PRIMARY("一级日志：敏感信息"),SECONDARY("二级日志：重要信息"),TERTIARY("三级日志：次要信息"),QUATERNARY("四级日志：普通信息")）',
  `customized_operation_type` varchar(32) NOT NULL COMMENT '自定义记录日志操作类型（LOGIN("用户登录"),LOGOUT("用户登出"),ADD("新增"),DELETE("删除"),UPDATE("修改"),IMPORT("导入"),EXPORT("导出"),UN_KNOW("未知")）',
  `operation_type` varchar(32) DEFAULT NULL COMMENT '操作类型（UN_KNOW("未知"),SELECT("查询"),EDIT("新增或更新"),DELETE("删除")）',
  `operation_unit` varchar(128) DEFAULT NULL COMMENT '操作对象（略）',
  `system_type` varchar(32) DEFAULT NULL COMMENT '日志系统类型',
  `url` varchar(64) DEFAULT NULL COMMENT '请求地址url',
  `privilege_id` bigint DEFAULT NULL COMMENT '权限菜单id',
  `clazz_name` varchar(64) DEFAULT NULL COMMENT '类名称',
  `method` varchar(64) DEFAULT NULL COMMENT '方法名称',
  `args` mediumtext COMMENT '参数',
  `run_time` bigint DEFAULT NULL COMMENT '方法运行时间',
  `run_status` varchar(32) DEFAULT NULL COMMENT '方法运行状态',
  `return_value` mediumtext COMMENT '方法返回值',
  `user_id` bigint NOT NULL COMMENT '用户对象id',
  `user_name` varchar(32) DEFAULT NULL COMMENT '用户名称',
  `ip` varchar(32) NOT NULL COMMENT 'ip地址',
  `create_time` bigint DEFAULT NULL COMMENT '日志记录时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `operation_log_idx` (`school_id`,`customized_operation_type`,`org_id`,`user_id`,`create_time`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础日志记录表';

-- ----------------------------
-- Table structure for basic_print_config
-- ----------------------------
DROP TABLE IF EXISTS `basic_print_config`;
CREATE TABLE `basic_print_config` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校ID',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
  `card_rule_id` bigint DEFAULT NULL COMMENT '题卡规则（-1代表通卡）',
  `print_content` varchar(255) DEFAULT NULL COMMENT '试卷、题卡',
  `backup_method` varchar(10) DEFAULT NULL COMMENT '试卷题卡备份方式（ROOM按考场）',
  `backup_count` double DEFAULT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
  `draw_rule` varchar(10) DEFAULT NULL COMMENT '抽卷规则（ONE只抽一次，CIRCLE循环抽取）',
  `variable_content` mediumtext COMMENT '变量印品参数json',
  `ordinary_content` mediumtext COMMENT '普通印品参数json',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  `review` tinyint DEFAULT NULL COMMENT '是否启用入库审核：0-禁用，1-启用',
  `print_method` varchar(6) DEFAULT NULL COMMENT 'AUTO-自动，MANUAL-手动',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='印品配置表';

-- ----------------------------
-- Table structure for basic_role_data_permission
-- ----------------------------
DROP TABLE IF EXISTS `basic_role_data_permission`;
CREATE TABLE `basic_role_data_permission` (
  `id` bigint NOT NULL COMMENT 'id',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `privilege_id` bigint NOT NULL COMMENT '权限id',
  `data_permission_type` varchar(45) NOT NULL COMMENT '数据权限类型',
  `org_scope` varchar(45) DEFAULT NULL COMMENT '数据权限机构权限组',
  `org_id` bigint DEFAULT NULL COMMENT '创建人机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `role_data_permission_idx` (`role_id`,`privilege_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色数据权限表';

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
DROP TABLE IF EXISTS `basic_school`;
CREATE TABLE `basic_school` (
  `id` bigint NOT NULL COMMENT '主键',
  `code` varchar(20) NOT NULL COMMENT '学校编码',
  `domain_name` varchar(20) DEFAULT NULL COMMENT '域名',
  `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
  `access_key` varchar(50) DEFAULT NULL COMMENT '访问key',
  `access_secret` varchar(50) DEFAULT NULL COMMENT '访问秘钥',
  `remark` text COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `logo` mediumtext COMMENT '学校logo',
  `init_password` varchar(100) DEFAULT NULL COMMENT '该学校用户的初始密码，默认为12345678',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `code` (`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校表';

-- ----------------------------
-- Table structure for basic_semester
-- ----------------------------
DROP TABLE IF EXISTS `basic_semester`;
CREATE TABLE `basic_semester` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `name` varchar(100) NOT NULL COMMENT '学期名称',
  `code` varchar(100) DEFAULT NULL COMMENT '学期编码',
  `start_time` bigint DEFAULT NULL COMMENT '开始日期',
  `end_time` bigint DEFAULT NULL COMMENT '结束日期',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `semester_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学期字典表';

-- ----------------------------
-- Table structure for basic_student
-- ----------------------------
DROP TABLE IF EXISTS `basic_student`;
CREATE TABLE `basic_student` (
  `id` bigint NOT NULL COMMENT '学生id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `student_name` varchar(100) NOT NULL COMMENT '学生姓名',
  `student_code` varchar(100) NOT NULL COMMENT '学生编号',
  `phone_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
  `belong_org_id` bigint DEFAULT NULL COMMENT '所属院系',
  `major_id` bigint DEFAULT NULL COMMENT '专业id',
  `clazz_id` bigint DEFAULT NULL COMMENT '班级主键',
  `enable` tinyint DEFAULT '1' COMMENT '可用状态（1：可用，0：不可用）',
  `org_id` bigint DEFAULT NULL COMMENT '可操作人机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `student_schoolId_code_orgId_idx` (`school_id`,`student_code`,`belong_org_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校id';

-- ----------------------------
-- Table structure for basic_teach_course
-- ----------------------------
DROP TABLE IF EXISTS `basic_teach_course`;
CREATE TABLE `basic_teach_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint DEFAULT NULL COMMENT '用户ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用？1-是 ；0-否',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='老师和课程关联表';

-- ----------------------------
-- Table structure for basic_template
-- ----------------------------
DROP TABLE IF EXISTS `basic_template`;
CREATE TABLE `basic_template` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(30) NOT NULL COMMENT '模板名称',
  `classify` varchar(10) DEFAULT NULL COMMENT 'template_type值为VARIABLE、ORDINARY时，不为空。\n  SIGN-签到表，PACKAGE-卷袋贴，CHECK_IN-登记表',
  `display_range` mediumtext COMMENT '卷袋贴、签到表数据范围',
  `preview_path` varchar(500) DEFAULT NULL COMMENT '预览地址',
  `attachment_id` bigint DEFAULT NULL COMMENT '模板附件ID（保存到附件表ID）',
  `enable` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `diallel` tinyint(1) DEFAULT '1' COMMENT '考生信息是否双列显示',
  `text_desc` varchar(500) DEFAULT NULL COMMENT '文字说明',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='通用模板';

-- ----------------------------
-- Table structure for basic_template_org
-- ----------------------------
DROP TABLE IF EXISTS `basic_template_org`;
CREATE TABLE `basic_template_org` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `type` varchar(20) NOT NULL COMMENT '模板类型：GENERIC-通卡模板，VARIABLE-变量印品模板，ORDINARY-普通印品模板，CARD_RULE-题卡规则，PRINT_CONFIG-印品配置',
  `rule_id` bigint NOT NULL COMMENT '通卡模板ID',
  `org_id` bigint NOT NULL COMMENT '组织机构ID',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='适用学院（题卡规则、模板适用学院）';

-- ----------------------------
-- Table structure for basic_user_course
-- ----------------------------
DROP TABLE IF EXISTS `basic_user_course`;
CREATE TABLE `basic_user_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint DEFAULT NULL COMMENT '用户ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID',
  `clazz_id` bigint DEFAULT NULL COMMENT '授课班级',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用？1-是 ；0-否',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和课程关联表';

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
DROP TABLE IF EXISTS `basic_verify_code`;
CREATE TABLE `basic_verify_code` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint NOT NULL COMMENT '用户ID',
  `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
  `expire_time` bigint NOT NULL COMMENT '到期时间',
  `valid_period` int NOT NULL COMMENT '单位（分）',
  `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信验证码记录表';

-- ----------------------------
-- Table structure for client_print_data
-- ----------------------------
DROP TABLE IF EXISTS `client_print_data`;
CREATE TABLE `client_print_data` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
  `machine_code` varchar(45) DEFAULT NULL COMMENT '机器唯一码',
  `pages_a4` int DEFAULT NULL COMMENT 'A4页数量',
  `paper_pages` int DEFAULT NULL COMMENT '试卷数量',
  `card_pages` int DEFAULT NULL COMMENT '题卡数量',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='各机器打印数量';

-- ----------------------------
-- Table structure for client_status
-- ----------------------------
DROP TABLE IF EXISTS `client_status`;
CREATE TABLE `client_status` (
  `id` bigint NOT NULL AUTO_INCREMENT COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `machine_code` varchar(100) NOT NULL COMMENT '机器唯一码',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划ID',
  `course_code` varchar(100) NOT NULL COMMENT '课程代码',
  `course_name` varchar(200) NOT NULL COMMENT '课程名称',
  `paper_number` varchar(50) NOT NULL COMMENT '试卷编号',
  `is_try` tinyint DEFAULT NULL COMMENT '是否打样：true-是，false-否',
  `try_time` bigint DEFAULT NULL COMMENT '打样时间',
  `is_pass` tinyint DEFAULT NULL COMMENT '是否合格：true-合格，false-不合格',
  `is_download` tinyint DEFAULT NULL COMMENT '是否缓存：true-是，false-否',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `idx_1` (`school_id`,`exam_detail_id`,`course_code`,`paper_number`,`machine_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='机器打样记录';

-- ----------------------------
-- Table structure for cloud_user_push_status
-- ----------------------------
DROP TABLE IF EXISTS `cloud_user_push_status`;
CREATE TABLE `cloud_user_push_status` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `user_id` bigint NOT NULL,
  `marker_status` varchar(100) NOT NULL COMMENT '评卷员推送状态(UserPushResultEnum):BIND_SUCCESS("绑定成功"),BIND_FAILED("绑定失败"),UNBIND_SUCCESS("解绑成功"),UNBIND_FAILED("解绑失败");',
  `marker_leader_status` varchar(100) NOT NULL COMMENT '科组长推送状态(UserPushResultEnum):BIND_SUCCESS("绑定成功"),BIND_FAILED("绑定失败"),UNBIND_SUCCESS("解绑成功"),UNBIND_FAILED("解绑失败");',
  `sync_status` varchar(100) NOT NULL COMMENT '同步状态(SyncStatusEnum)，ALREADY_PUSH("已经完成推送"),NEED_PUSH_AGAIN("需要重新推送");',
  `create_id` bigint DEFAULT NULL COMMENT '创建人\n',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `idx_school_id_user_id` (`school_id`,`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='云阅卷用户推送状态表';

-- ----------------------------
-- Table structure for exam_card
-- ----------------------------
DROP TABLE IF EXISTS `exam_card`;
CREATE TABLE `exam_card` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `course_code` varchar(30) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `title` varchar(200) NOT NULL COMMENT '标题',
  `make_method` varchar(10) DEFAULT NULL COMMENT '题卡制作方式：SELECT-选择已有题卡，SELF-自助创建，CUST-客户制卡',
  `status` varchar(45) DEFAULT NULL COMMENT 'STAGE-暂存，SUBMIT-提交',
  `content` mediumtext COMMENT '题卡工具制作题卡内容',
  `stage_content` mediumtext COMMENT '暂存内容（提交后置空）',
  `html_content` mediumtext COMMENT 'html格式内容',
  `type` varchar(45) NOT NULL COMMENT '题卡类型：GENERIC-通卡，CUSTOM-自定义',
  `create_method` varchar(45) DEFAULT NULL COMMENT '通卡创建方式（UPLOAD-上传，CUSTOMIZE-自定义）',
  `attachment_id` bigint DEFAULT NULL COMMENT 'type=GENERIC时必传，上传附件ID',
  `sync_status` tinyint(1) DEFAULT NULL COMMENT '是否同步成功：true-是， false-否',
  `enable` tinyint(1) DEFAULT '1' COMMENT '1正常，0禁用',
  `used` tinyint(1) DEFAULT '0' COMMENT '是否绑定命题任务，默认0',
  `remark` mediumtext COMMENT '备注',
  `card_rule_id` bigint DEFAULT NULL COMMENT '题卡规则Id',
  `backup_attachment` mediumtext COMMENT '备用题卡json',
  `jpg_attachment` text COMMENT '题卡转换成的jpg文件信息',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='题卡';

-- ----------------------------
-- Table structure for exam_card_detail
-- ----------------------------
DROP TABLE IF EXISTS `exam_card_detail`;
CREATE TABLE `exam_card_detail` (
  `id` bigint NOT NULL COMMENT '主键',
  `card_id` bigint NOT NULL COMMENT '题卡ID',
  `content` mediumtext COMMENT '题卡工具制作题卡内容',
  `html_content` mediumtext COMMENT 'html格式内容',
  `attachment_id` mediumtext COMMENT '备用题卡json',
  `cust_attachment_id` bigint DEFAULT NULL COMMENT '客服制卡上传附件ID',
  `jpg_attachment_info` text COMMENT '题卡转换成的jpg文件信息',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='题卡详情';

-- ----------------------------
-- Table structure for exam_detail
-- ----------------------------
DROP TABLE IF EXISTS `exam_detail`;
CREATE TABLE `exam_detail` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `print_plan_id` bigint NOT NULL COMMENT '印刷计划ID',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `print_plan_name` varchar(100) DEFAULT NULL COMMENT '印刷计划名称',
  `exam_start_time` bigint NOT NULL COMMENT '考试开始日期',
  `exam_end_time` bigint NOT NULL COMMENT '考试结束日期',
  `exam_place` varchar(30) DEFAULT NULL COMMENT '考场',
  `total_subjects` int DEFAULT NULL COMMENT '总科次：考场总科次',
  `exam_room` varchar(30) DEFAULT NULL COMMENT '教室',
  `print_progress` double DEFAULT NULL COMMENT '打印进度',
  `print_user` varchar(30) DEFAULT NULL COMMENT '打印员：一个考场只能被一个打印员认领',
  `package_code` varchar(30) DEFAULT NULL COMMENT '考场代码：卷袋贴、签到表上条码',
  `paper_pages` int DEFAULT NULL COMMENT '试卷总计（张）',
  `card_pages` int DEFAULT NULL COMMENT '题卡总计（张）',
  `pages_a4` int DEFAULT NULL COMMENT 'A4印量小计（页）',
  `status` varchar(30) DEFAULT NULL COMMENT '状态：NEW: "新建",READY: "就绪",WAITING: "待印刷",PRINTING: "印刷中",FINISH: "已完成"',
  `validate` tinyint(1) DEFAULT '0' COMMENT '是否校验：true-是，false-否',
  `print_start_time` bigint DEFAULT NULL COMMENT '打印开始时间',
  `print_end_time` bigint DEFAULT NULL COMMENT '打印完成时间',
  `attachment_id` bigint DEFAULT NULL COMMENT '附件id',
  `card_attachment_id` bigint DEFAULT NULL COMMENT '题卡合并A3文件附件ID',
  `attachment_path` mediumtext COMMENT '印品附件路径',
  `exam_data_source` varchar(20) DEFAULT NULL COMMENT '数据来源（EXAM_TASK：卷库查询生成，FILE_IMPORT：考务文件导入）',
  `normal` tinyint DEFAULT '1' COMMENT '是否正常，true正常，false做废',
  `print_house_id` bigint DEFAULT NULL COMMENT '印刷室ID',
  `backup_count` double DEFAULT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
  `task_id` bigint DEFAULT NULL COMMENT 'pdf生成异步任务ID（t_b_task）',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `idx_print_plan_id` (`print_plan_id`) USING BTREE,
  KEY `exam_detail_school_id_IDX` (`school_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据';

-- ----------------------------
-- Table structure for exam_detail_course
-- ----------------------------
DROP TABLE IF EXISTS `exam_detail_course`;
CREATE TABLE `exam_detail_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_detail_id` bigint NOT NULL COMMENT '考试明细id',
  `course_code` varchar(20) NOT NULL COMMENT '课程编码',
  `course_name` varchar(50) NOT NULL COMMENT '课程名称',
  `paper_number` varchar(30) NOT NULL COMMENT '页数',
  `total_subjects` int DEFAULT NULL COMMENT '总科次：考场总科次',
  `paper_pages_a3` int DEFAULT NULL COMMENT '试卷单科次准印量A3（页）',
  `card_pages_a3` int DEFAULT NULL COMMENT '题卡单科次准印量A3（页）',
  `clazz_id` text DEFAULT NULL COMMENT '班级id',
  `clazz_name` mediumtext COMMENT '班级（取学生表中的班级集合）',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `paper_type` varchar(30) DEFAULT NULL COMMENT '当前试卷类型',
  `attachment_id` mediumtext COMMENT '备用题卡json',
  `common_attachment_id` mediumtext COMMENT '模式2下考生公共题卡json',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `detail_course_schoolId_idx` (`school_id`,`paper_number`,`paper_type`) USING BTREE,
  KEY `idx_exam_detail_id` (`exam_detail_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据（考场关联科目）';

-- ----------------------------
-- Table structure for exam_detail_course_paper_type
-- ----------------------------
DROP TABLE IF EXISTS `exam_detail_course_paper_type`;
CREATE TABLE `exam_detail_course_paper_type` (
  `id` bigint NOT NULL,
  `exam_detail_course_id` bigint NOT NULL,
  `paper_type` varchar(5) NOT NULL COMMENT '卷型',
  PRIMARY KEY (`id`),
  KEY `idx_exam_detail_course_id` (`exam_detail_course_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据（考场关联科目对应的卷型）';

-- ----------------------------
-- Table structure for exam_paper_class_marker
-- ----------------------------
DROP TABLE IF EXISTS `exam_paper_class_marker`;
CREATE TABLE `exam_paper_class_marker` (
  `id` bigint NOT NULL,
  `exam_paper_structure_id` bigint DEFAULT NULL COMMENT '评卷参数id',
  `marker_id` bigint DEFAULT NULL COMMENT '评卷员id',
  `login_name` varchar(100) DEFAULT NULL COMMENT '评卷员登录名',
  `class_name` mediumtext COMMENT '班级',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分班阅（评卷员班级对应关系表）';

-- ----------------------------
-- Table structure for exam_paper_group
-- ----------------------------
DROP TABLE IF EXISTS `exam_paper_group`;
CREATE TABLE `exam_paper_group` (
  `id` bigint NOT NULL COMMENT '分组id',
  `exam_paper_structure_id` bigint NOT NULL COMMENT '试卷结构id',
  `group_number` int NOT NULL COMMENT '分组序号',
  `double_rate` tinyint DEFAULT NULL COMMENT '双评比例（0关闭，1开启）',
  `arbitrate_threshold` int DEFAULT NULL COMMENT '仲裁阈值（开启双评必填）',
  `score_policy` varchar(45) DEFAULT NULL COMMENT '合分策略 开启双评是必填 AVG("平均分", 1), MAX("最高分", 2), MIN("最低分", 3)',
  `trial_count` int DEFAULT NULL COMMENT '试评数量',
  `mark_mode` varchar(45) DEFAULT NULL COMMENT '评卷模式 COMMON("普通"), TRACK("轨迹")',
  `question_info` text COMMENT '题目信息',
  `picture_config` text COMMENT '图片配置',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `exam_paper_group_unique` (`exam_paper_structure_id`,`group_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='试卷分组表';

-- ----------------------------
-- Table structure for exam_paper_group_marker
-- ----------------------------
DROP TABLE IF EXISTS `exam_paper_group_marker`;
CREATE TABLE `exam_paper_group_marker` (
  `id` bigint NOT NULL COMMENT '评卷员-分组关系id',
  `group_id` bigint NOT NULL COMMENT '分组id',
  `marker_id` bigint NOT NULL COMMENT '评卷员id',
  `login_name` varchar(100) DEFAULT NULL COMMENT '评卷员登录名',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='评卷员-分组关系表';

-- ----------------------------
-- Table structure for exam_paper_structure
-- ----------------------------
DROP TABLE IF EXISTS `exam_paper_structure`;
CREATE TABLE `exam_paper_structure` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `semester_name` varchar(100) DEFAULT NULL COMMENT '学期',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `third_relate_id` bigint DEFAULT NULL COMMENT '云阅卷考试ID',
  `third_relate_name` varchar(255) DEFAULT NULL COMMENT '云阅卷考试名称',
  `paper_number` varchar(50) NOT NULL COMMENT '试卷编号',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(255) DEFAULT NULL COMMENT '课程名称',
  `sequence` varchar(5) DEFAULT NULL COMMENT '课程创建的任务序号（和exam_task保持一致）',
  `paper_type` varchar(20) DEFAULT NULL COMMENT '试卷类型',
  `status` varchar(700) DEFAULT NULL COMMENT '{\n    "structure":\n    {\n        "save": false,\n        "sync": false\n    },\n    "group":\n    {\n        "save": false,\n        "sync": false\n    },\n    "class":\n    {\n        "save": false,\n        "sync": false\n    },\n    "objective":\n    {\n        "save": false,\n        "sync": false\n    },\n    "markerLeader":\n    {\n        "save": false,\n        "sync": false\n    },\n    "marker":\n    {\n        "save": false,\n        "sync": false\n    },\n    "answerFile":\n    {\n        "save": false,\n        "sync": false\n    }\n}',
  `is_finish` tinyint(1) DEFAULT '0' COMMENT '是否完成。根据status中组合状态计算。用于当查询条件',
  `paper_answer` text COMMENT '试卷原卷和标答附件ID',
  `objective_structure` mediumtext COMMENT '客观题试卷结构JSON',
  `subjective_structure` mediumtext COMMENT '主观题试卷结构JSON',
  `paper_info_json` mediumtext COMMENT '整个试卷信息json（前端提供的全部信息）',
  `proposition_teacher_id` bigint DEFAULT NULL COMMENT '命题老师ID',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用（0-禁用，1-启用）',
  `mark_leader` text COMMENT '科组长，样例：[{markerId:1, loginName:"mt01"}]',
  `cloud_info_json` text COMMENT '云阅卷试卷结构',
  `structure_change` tinyint(1) DEFAULT NULL COMMENT '云阅卷试卷结构和知学知考结构不一样（默认false）',
  `object_answer_change` tinyint(1) DEFAULT '0' COMMENT '客观题答案有提交，需要重新统分。（1：有变动，需要统分，0：不统分）',
  `open_class_reading` tinyint(1) DEFAULT '0' COMMENT '是否开启分班阅',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `exam_paper_structure_unique` (`school_id`,`exam_id`,`paper_number`,`paper_type`) USING BTREE,
  KEY `exam_paper_structure_status_IDX` (`status`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for exam_print_plan
-- ----------------------------
DROP TABLE IF EXISTS `exam_print_plan`;
CREATE TABLE `exam_print_plan` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `name` varchar(150) DEFAULT NULL COMMENT '名称',
  `exam_start_time` bigint NOT NULL COMMENT '考试开始时间',
  `exam_end_time` bigint NOT NULL COMMENT '考试结束时间',
  `print_content` varchar(30) NOT NULL COMMENT '{“PAPER”, "CARD"}',
  `backup_method` varchar(10) NOT NULL COMMENT 'PLACE-按考点，ROOM-按考场',
  `backup_count` double NOT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
  `draw_rule` varchar(10) DEFAULT NULL COMMENT 'ONE-只抽一次，CIRCLE-可循环抽取',
  `variable_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "SIGN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	},\n  	{\n  		"type": "PACKAG",\n  		"usedId": 3,\n  		"backupMethod": "STUDENT",\n  		"backupCount": 5\n  	}\n  ]',
  `ordinary_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "CHECK_IN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	}\n  ]',
  `status` varchar(20) NOT NULL COMMENT '印刷计划状态',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `third_relate_id` bigint DEFAULT NULL COMMENT '第三方系统关联ID（目前只有云阅卷）',
  `third_relate_name` varchar(100) DEFAULT NULL COMMENT '第三方系统关联考试名称（目前只有云阅卷）',
  `sync_status` varchar(20) DEFAULT NULL COMMENT '是否同步成功',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='印刷计划管理';

-- ----------------------------
-- Table structure for exam_student
-- ----------------------------
DROP TABLE IF EXISTS `exam_student`;
CREATE TABLE `exam_student` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `exam_detail_course_id` bigint NOT NULL COMMENT '考试明细课程id',
  `student_name` varchar(100) NOT NULL COMMENT '考生姓名',
  `student_code` varchar(100) NOT NULL COMMENT '考生代码',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `paper_number` varchar(30) NOT NULL COMMENT '试卷编号',
  `ticket_number` varchar(30) DEFAULT NULL COMMENT '准考证号',
  `site_number` varchar(30) DEFAULT NULL COMMENT '座位号',
  `print_paper` tinyint DEFAULT NULL COMMENT '打印试卷',
  `print_card` tinyint DEFAULT NULL COMMENT '打印题卡',
  `extend_fields` text COMMENT '扩展字段',
  `paper_type` varchar(2) DEFAULT NULL COMMENT '试卷类型',
  `college_name` varchar(100) DEFAULT NULL COMMENT '学生学院名称',
  `major_name` varchar(100) DEFAULT NULL COMMENT '专业名称',
  `clazz_id` varchar(100) DEFAULT NULL COMMENT '班级Id',
  `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
  `teach_clazz_id` bigint DEFAULT NULL COMMENT '教学班id',
  `teach_clazz_name` varchar(100) DEFAULT NULL COMMENT '教学班名称',
  `student_clazz_type` varchar(100) DEFAULT NULL COMMENT '主体班级类型',
  `campus_name` varchar(200) DEFAULT NULL COMMENT '校区',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `attachment_id` bigint DEFAULT NULL COMMENT '附件id',
  `sync_status` tinyint(1) DEFAULT NULL COMMENT '是否同步成功：true-是， false-否',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `unique_ticket_number_idx` (`school_id`,`ticket_number`),
  KEY `exam_student_schoolId_idx` (`school_id`,`student_code`) USING BTREE,
  KEY `idx_exam_detail_course_id` (`exam_detail_course_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考生';

-- ----------------------------
-- Table structure for exam_task
-- ----------------------------
DROP TABLE IF EXISTS `exam_task`;
CREATE TABLE `exam_task` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `course_code` varchar(20) NOT NULL COMMENT '课程代码',
  `course_name` varchar(50) NOT NULL COMMENT '课程名称',
  `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
  `paper_number` varchar(30) NOT NULL COMMENT '试卷编号',
  `sequence` varchar(5) DEFAULT NULL COMMENT '课程创建的任务序号',
  `start_time` bigint DEFAULT NULL COMMENT '命题开始时间',
  `end_time` bigint DEFAULT NULL COMMENT '命题结束时间',
  `card_rule_id` bigint NOT NULL COMMENT '题卡规则',
  `user_id` bigint DEFAULT NULL COMMENT '命题老师',
  `transfer_id` bigint DEFAULT NULL COMMENT '转办人ID',
  `status` varchar(50) DEFAULT NULL COMMENT '任务状态，DRAFT：新建，SUBMIT：已提交，STAGE：待命题',
  `review` tinyint DEFAULT NULL COMMENT '是否审核，0-禁用，1-启用',
  `review_status` varchar(8) DEFAULT NULL COMMENT '审核状态：PASS-审核通过，NOT_PASS-审核不通过，默认为空-未审核',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
  `batch_no` varchar(32) DEFAULT NULL COMMENT '批次id',
  `teaching_room_id` bigint DEFAULT NULL COMMENT '教学单位、教研室',
  `teacher_name` varchar(100) DEFAULT NULL COMMENT '拟卷教师',
  `lecturer_name` varchar(100) DEFAULT NULL COMMENT '主讲教师',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `cancel_remark` mediumtext COMMENT '作废原因',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `flow_id` bigint DEFAULT NULL COMMENT '流程id',
  `approve_form_attachment_id` bigint DEFAULT NULL COMMENT '命题任务审批表附件id',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `task_schoolId_flowId_idx` (`school_id`,`flow_id`,`card_rule_id`) USING BTREE,
  KEY `idx_school_id_paper_number` (`school_id`,`paper_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务';

-- ----------------------------
-- Table structure for exam_task_apply_temp
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_apply_temp`;
CREATE TABLE `exam_task_apply_temp` (
  `id` bigint unsigned NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校Id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `exam_task_content` mediumtext COMMENT '命题任务json',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='卷库查询-命题任务申请临时表';

-- ----------------------------
-- Table structure for exam_task_detail
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_detail`;
CREATE TABLE `exam_task_detail` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
  `paper_type` varchar(20) DEFAULT NULL COMMENT '试卷类型，多个以”/“分隔，如A/B/C/D',
  `draw_count` int DEFAULT NULL COMMENT '单次抽卷数量',
  `paper_attachment_ids` text COMMENT '转换为A3格式的试卷附件\n[\\n  	{\\n  		"type": "A",\\n  		"attachmentId": 12,\\n  		"totalPages": 3\\n  	},\\n  	{\\n  		"type": "B",\\n  		"attachmentId": 13,\\n  		"totalPages": 2\\n  	}\\n  ]',
  `make_method` varchar(45) DEFAULT NULL COMMENT '题卡绑定方式',
  `paper_confirm_attachment_ids` text COMMENT '[\n  	{\n  		"attachmentId": 12\n  	},\n  	{\n  		"attachmentId": 13\n  	}\n  ]',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `exposed_paper_type` varchar(20) DEFAULT NULL COMMENT '曝光卷型：已打印卷型，多个以”/“分隔，如A/B/C/D',
  `relate_paper_type` varchar(5) DEFAULT NULL COMMENT '已曝光卷型',
  `unexposed_paper_type` varchar(20) DEFAULT NULL COMMENT '未曝光卷型',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `enable` tinyint DEFAULT '1' COMMENT '0-禁用，1-启用',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `exam_task_detail_idx` (`exam_task_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务明细表（入库申请）';

-- ----------------------------
-- Table structure for exam_task_paper_log
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_paper_log`;
CREATE TABLE `exam_task_paper_log` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
  `paper_type` varchar(45) DEFAULT NULL COMMENT '试卷类型',
  `paper_attachment_ids` mediumtext COMMENT '修改后试卷',
  `card_id` bigint DEFAULT NULL COMMENT '修改后题卡ID',
  `review` tinyint(1) DEFAULT NULL COMMENT '是否审核：true-审核，false-未审核',
  `review_status` varchar(45) DEFAULT NULL COMMENT '审核状态：PASS-通过，NOT_PASS-不通过',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `exam_task_paper_idx` (`exam_task_id`,`review`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='卷库修改审核临时表';

-- ----------------------------
-- Table structure for exam_task_print
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_print`;
CREATE TABLE `exam_task_print` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL,
  `org_id` bigint DEFAULT NULL,
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划ID',
  `course_code` varchar(45) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(50) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
  `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
  `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
  `package_seq` varchar(30) DEFAULT NULL COMMENT '卷袋序号',
  `exam_place` varchar(100) DEFAULT NULL,
  `exam_room` varchar(100) DEFAULT NULL,
  `invigilator_teacher` varchar(45) DEFAULT NULL COMMENT '监考老师',
  `class_id` varchar(100) DEFAULT NULL COMMENT '班级ID集合',
  `class_name` varchar(500) DEFAULT NULL COMMENT '班级名称集合',
  `student_count` int DEFAULT NULL COMMENT '考生总人数',
  `campus_id` bigint DEFAULT NULL COMMENT '校区ID',
  `print_house_id` bigint DEFAULT NULL COMMENT '印刷室ID',
  `print_house_name` varchar(100) DEFAULT NULL COMMENT '印刷室名称',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
  `extend_fields` text COMMENT '扩展字段',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='以试卷作为起点生成考务数据';

-- ----------------------------
-- Table structure for exam_task_review_log
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_review_log`;
CREATE TABLE `exam_task_review_log` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
  `review_status` varchar(8) NOT NULL COMMENT 'PASS-通过，NOT_PASS-不通过',
  `reason` mediumtext COMMENT '审核意见',
  `operate_id` bigint DEFAULT NULL COMMENT '操作人id',
  `operate_time` bigint DEFAULT NULL COMMENT '操作时间',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务审核日志表';

-- ----------------------------
-- Table structure for exam_task_sync
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_sync`;
CREATE TABLE `exam_task_sync` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_code` varchar(50) NOT NULL COMMENT '课程代码',
  `paper_number` varchar(50) NOT NULL COMMENT '试着编号',
  `paper_type` varchar(50) DEFAULT NULL COMMENT '试卷类型',
  `total_paper_type` varchar(50) DEFAULT NULL COMMENT '所有卷型',
  `org_code` varchar(45) DEFAULT NULL COMMENT '机构代码',
  `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
  `sync_status` varchar(45) DEFAULT NULL COMMENT '同步状态',
  `error_msg` varchar(500) DEFAULT NULL COMMENT '同步失败原因',
  `sync_user_id` bigint DEFAULT NULL COMMENT '同步用户ID',
  `sync_start_time` bigint DEFAULT NULL COMMENT '同步开始时间 ',
  `sync_end_time` bigint DEFAULT NULL COMMENT '同步结束时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for exam_task_temp
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_temp`;
CREATE TABLE `exam_task_temp` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `batch_no` varchar(32) NOT NULL COMMENT '本次导入数据临时ID',
  `course_code` varchar(20) NOT NULL COMMENT '课程代码',
  `course_name` varchar(50) NOT NULL COMMENT '课程名称',
  `paper_number` varchar(30) DEFAULT NULL COMMENT '试卷编号',
  `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
  `user_account` varchar(45) DEFAULT NULL COMMENT '命题老师账号',
  `user_name` varchar(45) DEFAULT NULL COMMENT '命题老师姓名',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务临时表';

-- ----------------------------
-- Table structure for grade_batch
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch`;
CREATE TABLE `grade_batch` (
  `id` bigint NOT NULL COMMENT '分析批次主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `semester_id` bigint NOT NULL COMMENT '学期id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `batch_code` varchar(32) NOT NULL COMMENT '批次编号（在学校下唯一）',
  `batch_name` varchar(32) NOT NULL COMMENT '批次名称（2020~2021学年上学期期末考试）',
  `batch_time` bigint NOT NULL COMMENT '分析批次时间',
  `status` varchar(32) NOT NULL COMMENT '批次状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
  `third_semester_id` bigint DEFAULT NULL COMMENT '教研分析学期id',
  `third_exam_id` bigint DEFAULT NULL COMMENT '第三方考试id',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  `source` varchar(1000) DEFAULT NULL COMMENT '原始数据',
  `progress` decimal(8,2) DEFAULT NULL COMMENT '进度',
  `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
  `notify_time` bigint DEFAULT NULL COMMENT '回调时间',
  `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-批次表';

-- ----------------------------
-- Table structure for grade_batch_paper
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch_paper`;
CREATE TABLE `grade_batch_paper` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `batch_id` bigint NOT NULL COMMENT '分析批次id',
  `grade_paper_id` bigint DEFAULT NULL COMMENT '试卷id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `status` varchar(32) NOT NULL COMMENT '分析试卷状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
  `teach_college_id` bigint NOT NULL COMMENT '当前开课学院id',
  `teach_college_name` varchar(100) NOT NULL COMMENT '当前开课学院名称',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `enable` tinyint DEFAULT NULL COMMENT '是否可用',
  `publish_status` tinyint DEFAULT NULL COMMENT '是否发布',
  `grade_course_code` varchar(100) DEFAULT NULL COMMENT '推送教研分析后回传的教研分析课程编号（考试下唯一）',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-批次试卷关联关系表';

-- ----------------------------
-- Table structure for grade_batch_student
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch_student`;
CREATE TABLE `grade_batch_student` (
  `id` bigint NOT NULL,
  `batch_id` bigint DEFAULT NULL COMMENT '分析-批次表ID',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
  `student_code` varchar(45) DEFAULT NULL COMMENT '学号',
  `student_name` varchar(100) DEFAULT NULL COMMENT '学生姓名',
  `ticket_number` varchar(45) DEFAULT NULL COMMENT '准考证号',
  `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
  `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据（包含任课老师）';

-- ----------------------------
-- Table structure for grade_batch_student_clazz
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch_student_clazz`;
CREATE TABLE `grade_batch_student_clazz` (
  `id` bigint NOT NULL,
  `batch_id` bigint DEFAULT NULL COMMENT '分析-批次表ID',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
  `clazz_id` varchar(45) DEFAULT NULL COMMENT '班级ID',
  `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
  `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
  `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据班级（包含任课老师）';

-- ----------------------------
-- Table structure for grade_module_define
-- ----------------------------
DROP TABLE IF EXISTS `grade_module_define`;
CREATE TABLE `grade_module_define` (
  `id` bigint NOT NULL COMMENT '模块熟练度主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `module_type` varchar(32) NOT NULL COMMENT '维度模块类型',
  `interpret` varchar(255) DEFAULT NULL COMMENT '解释',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  `define` varchar(255) DEFAULT NULL COMMENT '二级熟练度定义',
  `level` varchar(32) DEFAULT NULL COMMENT '等级',
  `min` varchar(32) DEFAULT NULL COMMENT '展示用最小边界',
  `max` varchar(32) DEFAULT NULL COMMENT '展示用最大边界',
  `scope` varchar(32) DEFAULT NULL COMMENT '取值范围（计算用）',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-模块二级考察点维度熟练度配置，及模块简要描述表';

-- ----------------------------
-- Table structure for grade_module_evaluation
-- ----------------------------
DROP TABLE IF EXISTS `grade_module_evaluation`;
CREATE TABLE `grade_module_evaluation` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `module_type` varchar(64) NOT NULL COMMENT '模块类型（知识、维度、素养）',
  `formula` varchar(64) NOT NULL COMMENT '计算公式规则',
  `scope` varchar(64) NOT NULL COMMENT '取值范围（计算用）',
  `level_code` varchar(32) NOT NULL COMMENT '等级',
  `level_name` varchar(32) NOT NULL COMMENT '水平等级层次',
  `result` text COMMENT '诊断结果',
  `advice` text COMMENT '学习建议',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='校验-考察点模块评价建议表';

-- ----------------------------
-- Table structure for grade_paper
-- ----------------------------
DROP TABLE IF EXISTS `grade_paper`;
CREATE TABLE `grade_paper` (
  `id` bigint NOT NULL COMMENT '试卷id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
  `start_time` bigint DEFAULT NULL COMMENT '该试卷考试开始时间',
  `end_time` bigint DEFAULT NULL COMMENT '该试卷考试结束时间',
  `coefficient` decimal(12,4) DEFAULT NULL COMMENT '赋分系数（赋值分=卷面成绩 +（卷面满分-卷面成绩）/ 赋分系数）',
  `total_score` decimal(12,4) NOT NULL COMMENT '试卷满分',
  `pass_score` decimal(12,4) NOT NULL COMMENT '试卷及格分',
  `score_type` varchar(64) NOT NULL COMMENT '赋分类型\\n1.NEED_ASSIGN_SCORE("是","需要且按照公式赋分")\\n2.NO_NEED_ASSIGN_SCORE("否","不需要赋分，标准分即为赋分")\\n3.SPECIAL_ASSIGN_SCORE("特殊赋分","特殊赋分，按照excel表进行赋分")',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='试卷基础信息表（赋分系数、试卷类型、满分、及格分等）';

-- ----------------------------
-- Table structure for grade_paper_dimension
-- ----------------------------
DROP TABLE IF EXISTS `grade_paper_dimension`;
CREATE TABLE `grade_paper_dimension` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
  `dimension_type` varchar(32) NOT NULL COMMENT '维度类型 - 知识，能力，素养',
  `code_primary` varchar(32) NOT NULL COMMENT '一级维度编号',
  `name_primary` varchar(255) NOT NULL COMMENT '一级维度名称',
  `code_second` varchar(32) DEFAULT NULL COMMENT '二级维度编号',
  `name_second` varchar(255) DEFAULT NULL COMMENT '二级维度名称',
  `interpretation` varchar(255) DEFAULT NULL COMMENT '一级维度术语解释',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-考察点维度表';

-- ----------------------------
-- Table structure for grade_paper_struct
-- ----------------------------
DROP TABLE IF EXISTS `grade_paper_struct`;
CREATE TABLE `grade_paper_struct` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
  `question_name` varchar(200) NOT NULL COMMENT '大题名称',
  `number_type` varchar(32) NOT NULL COMMENT '题号类型（客观题、主观题）',
  `big_question_number` varchar(64) NOT NULL COMMENT '大题号',
  `small_question_number` varchar(64) NOT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
  `big_topic_name` varchar(64) NOT NULL COMMENT '大题名称',
  `full_score` decimal(12,4) NOT NULL COMMENT '满分',
  `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
  `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
  `knowledge_dimension` varchar(255) NOT NULL COMMENT '知识维度',
  `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
  `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-试卷结构（蓝图）表';

-- ----------------------------
-- Table structure for qrtz_blob_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_blob_triggers`;
CREATE TABLE `qrtz_blob_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `BLOB_DATA` blob,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `SCHED_NAME` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_calendars
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_calendars`;
CREATE TABLE `qrtz_calendars` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `CALENDAR_NAME` varchar(200) NOT NULL,
  `CALENDAR` blob NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_cron_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_cron_triggers`;
CREATE TABLE `qrtz_cron_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `CRON_EXPRESSION` varchar(120) NOT NULL,
  `TIME_ZONE_ID` varchar(80) DEFAULT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_fired_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_fired_triggers`;
CREATE TABLE `qrtz_fired_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `ENTRY_ID` varchar(95) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `INSTANCE_NAME` varchar(200) NOT NULL,
  `FIRED_TIME` bigint NOT NULL,
  `SCHED_TIME` bigint NOT NULL,
  `PRIORITY` int NOT NULL,
  `STATE` varchar(16) NOT NULL,
  `JOB_NAME` varchar(200) DEFAULT NULL,
  `JOB_GROUP` varchar(200) DEFAULT NULL,
  `IS_NONCONCURRENT` varchar(1) DEFAULT NULL,
  `REQUESTS_RECOVERY` varchar(1) DEFAULT NULL,
  PRIMARY KEY (`SCHED_NAME`,`ENTRY_ID`) USING BTREE,
  KEY `IDX_QRTZ_FT_TRIG_INST_NAME` (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE,
  KEY `IDX_QRTZ_FT_INST_JOB_REQ_RCVRY` (`SCHED_NAME`,`INSTANCE_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
  KEY `IDX_QRTZ_FT_J_G` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_FT_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_FT_T_G` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_FT_TG` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_job_details
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_job_details`;
CREATE TABLE `qrtz_job_details` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `JOB_NAME` varchar(200) NOT NULL,
  `JOB_GROUP` varchar(200) NOT NULL,
  `DESCRIPTION` varchar(250) DEFAULT NULL,
  `JOB_CLASS_NAME` varchar(250) NOT NULL,
  `IS_DURABLE` varchar(1) NOT NULL,
  `IS_NONCONCURRENT` varchar(1) NOT NULL,
  `IS_UPDATE_DATA` varchar(1) NOT NULL,
  `REQUESTS_RECOVERY` varchar(1) NOT NULL,
  `JOB_DATA` blob,
  PRIMARY KEY (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_J_REQ_RECOVERY` (`SCHED_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
  KEY `IDX_QRTZ_J_GRP` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_locks
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_locks`;
CREATE TABLE `qrtz_locks` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `LOCK_NAME` varchar(40) NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`LOCK_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_paused_trigger_grps
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_paused_trigger_grps`;
CREATE TABLE `qrtz_paused_trigger_grps` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_scheduler_state
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_scheduler_state`;
CREATE TABLE `qrtz_scheduler_state` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `INSTANCE_NAME` varchar(200) NOT NULL,
  `LAST_CHECKIN_TIME` bigint NOT NULL,
  `CHECKIN_INTERVAL` bigint NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simple_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simple_triggers`;
CREATE TABLE `qrtz_simple_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `REPEAT_COUNT` bigint NOT NULL,
  `REPEAT_INTERVAL` bigint NOT NULL,
  `TIMES_TRIGGERED` bigint NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simprop_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simprop_triggers`;
CREATE TABLE `qrtz_simprop_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `STR_PROP_1` varchar(512) DEFAULT NULL,
  `STR_PROP_2` varchar(512) DEFAULT NULL,
  `STR_PROP_3` varchar(512) DEFAULT NULL,
  `INT_PROP_1` int DEFAULT NULL,
  `INT_PROP_2` int DEFAULT NULL,
  `LONG_PROP_1` bigint DEFAULT NULL,
  `LONG_PROP_2` bigint DEFAULT NULL,
  `DEC_PROP_1` decimal(13,4) DEFAULT NULL,
  `DEC_PROP_2` decimal(13,4) DEFAULT NULL,
  `BOOL_PROP_1` varchar(1) DEFAULT NULL,
  `BOOL_PROP_2` varchar(1) DEFAULT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_triggers`;
CREATE TABLE `qrtz_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `JOB_NAME` varchar(200) NOT NULL,
  `JOB_GROUP` varchar(200) NOT NULL,
  `DESCRIPTION` varchar(250) DEFAULT NULL,
  `NEXT_FIRE_TIME` bigint DEFAULT NULL,
  `PREV_FIRE_TIME` bigint DEFAULT NULL,
  `PRIORITY` int DEFAULT NULL,
  `TRIGGER_STATE` varchar(16) NOT NULL,
  `TRIGGER_TYPE` varchar(8) NOT NULL,
  `START_TIME` bigint NOT NULL,
  `END_TIME` bigint DEFAULT NULL,
  `CALENDAR_NAME` varchar(200) DEFAULT NULL,
  `MISFIRE_INSTR` smallint DEFAULT NULL,
  `JOB_DATA` blob,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_J` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_C` (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE,
  KEY `IDX_QRTZ_T_G` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_STATE` (`SCHED_NAME`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_N_STATE` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_N_G_STATE` (`SCHED_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_NEXT_FIRE_TIME` (`SCHED_NAME`,`NEXT_FIRE_TIME`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_ST` (`SCHED_NAME`,`TRIGGER_STATE`,`NEXT_FIRE_TIME`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_ST_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_ST_MISFIRE_GRP` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
DROP TABLE IF EXISTS `sys_config`;
CREATE TABLE `sys_config` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `config_key` varchar(100) NOT NULL COMMENT '参数键名',
  `config_name` varchar(100) NOT NULL COMMENT '参数名称',
  `config_value` varchar(100) DEFAULT NULL COMMENT '参数键值',
  `remark` varchar(100) DEFAULT NULL COMMENT '备注',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `sort` int DEFAULT '1' COMMENT '排序',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参数表';

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
DROP TABLE IF EXISTS `sys_org`;
CREATE TABLE `sys_org` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `type` varchar(50) DEFAULT NULL COMMENT '类型，SCHOOL：学校，COLLEGE：学院，FACULTY：院系，TEACHING_ROOM：教研室，PRINTING_HOUSE：印刷厂',
  `code` varchar(20) DEFAULT NULL COMMENT '机构代码',
  `name` varchar(100) NOT NULL COMMENT '机构名称',
  `parent_id` bigint DEFAULT NULL COMMENT '上级机构id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `historic_name` varchar(100) DEFAULT NULL COMMENT '历史机构名称',
  `third_relate_id` int DEFAULT NULL COMMENT '云阅卷机构ID',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `org_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校组织架构';

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_privilege`;
CREATE TABLE `sys_privilege` (
  `id` bigint NOT NULL COMMENT '主键',
  `name` varchar(100) NOT NULL COMMENT '菜单名称',
  `url` varchar(100) NOT NULL COMMENT 'URL地址',
  `type` varchar(50) NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件，TAB：选项卡',
  `parent_id` bigint DEFAULT NULL COMMENT '上级菜单',
  `sequence` int NOT NULL COMMENT '序号',
  `property` varchar(30) DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
  `related` text COMMENT '关联属性，type为URL且property为AUTH才有',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `default_auth` tinyint DEFAULT '1' COMMENT '默认权限，0：否，1：是',
  `front_display` tinyint(1) DEFAULT '1' COMMENT '是否前端展示',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='菜单权限表';

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_role`;
CREATE TABLE `sys_role` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(50) NOT NULL COMMENT '角色名称',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `type` varchar(20) DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，SCHOOL_ADMIN：管理员：EXAM_TEACHER：考务老师，QUESTION_TEACHER：命题老师，CUSTOMER：客服人员，PRINTER：印刷人员，CUSTOM：自定义',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `default_role` tinyint NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
  `source` varchar(100) DEFAULT NULL COMMENT '角色来源，ANALYSIS：教研分析',
  `interpret` varchar(1000) DEFAULT NULL COMMENT '角色说明',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `role_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色表';

-- ----------------------------
-- Table structure for sys_role_group_member
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_group_member`;
CREATE TABLE `sys_role_group_member` (
  `id` bigint NOT NULL COMMENT '主键',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `user_id` bigint NOT NULL COMMENT '用户id',
  `member_id` bigint NOT NULL COMMENT '成员id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `role_group_roleId_memberId_idx` (`role_id`,`member_id`,`enable`) USING BTREE,
  KEY `role_group_member_idx` (`role_id`,`user_id`,`member_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色组成员表';

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_privilege`;
CREATE TABLE `sys_role_privilege` (
  `id` bigint NOT NULL COMMENT '主键',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `privilege_id` bigint NOT NULL COMMENT '菜单id',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `school_id` bigint DEFAULT NULL COMMENT '内置角色，按学校自定义的权限',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色和菜单关联表';

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
DROP TABLE IF EXISTS `sys_user`;
CREATE TABLE `sys_user` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `login_name` varchar(100) NOT NULL COMMENT '用户名',
  `real_name` varchar(100) NOT NULL COMMENT '姓名',
  `code` varchar(100) DEFAULT NULL COMMENT '工号',
  `password` varchar(50) NOT NULL COMMENT '密码',
  `mobile_number` varchar(30) DEFAULT NULL COMMENT '手机号',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `pwd_count` tinyint DEFAULT '0' COMMENT '密码修改次数，默认为0',
  `pwd_update_time` bigint DEFAULT NULL COMMENT '密码修改时间',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `historic_name` varchar(100) DEFAULT NULL COMMENT '历史用户名称',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `user_schoolId_loginName_idx` (`school_id`,`login_name`) USING BTREE,
  UNIQUE KEY `uq_school_id_mobile_number` (`school_id`,`mobile_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户表';

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_user_role`;
CREATE TABLE `sys_user_role` (
  `id` bigint NOT NULL COMMENT '主键',
  `user_id` bigint NOT NULL COMMENT '用户id',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `privilege_id` bigint DEFAULT NULL COMMENT '权限id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和角色关联表';

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
DROP TABLE IF EXISTS `t_b_session`;
CREATE TABLE `t_b_session` (
  `id` varchar(100) NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `identity` varchar(100) NOT NULL COMMENT '用户标识',
  `type` varchar(50) NOT NULL COMMENT '用户类型',
  `source` varchar(50) NOT NULL COMMENT '访问来源',
  `platform` varchar(50) NOT NULL COMMENT '设备分类',
  `device_id` varchar(100) NOT NULL COMMENT '设备标识',
  `address` varchar(100) NOT NULL COMMENT '登录IP地址',
  `access_token` varchar(50) NOT NULL COMMENT '访问令牌',
  `last_access_time` bigint DEFAULT NULL COMMENT '最近访问时间',
  `last_access_ip` varchar(100) DEFAULT NULL COMMENT '最近访问IP地址',
  `update_time` bigint DEFAULT NULL COMMENT '令牌更新时间',
  `expire_time` bigint DEFAULT NULL COMMENT '令牌强制失效时间',
  `app_source` varchar(50) DEFAULT NULL COMMENT '应用来源',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='会话表';

-- ----------------------------
-- Table structure for t_b_sync_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sync_task`;
CREATE TABLE `t_b_sync_task` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `semester_id` bigint DEFAULT NULL COMMENT '学期',
  `exam_id` bigint DEFAULT NULL COMMENT '考试',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `type` varchar(50) NOT NULL COMMENT '推送类型，USER_PUSH：用户推送',
  `third_relate_id` bigint DEFAULT NULL COMMENT '第三方系统关联ID（目前只有云阅卷）',
  `third_relate_name` varchar(300) DEFAULT NULL COMMENT '云阅卷考试名称',
  `exam_time` varchar(20) DEFAULT NULL COMMENT '考试时间',
  `object_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `org_code` varchar(45) DEFAULT NULL COMMENT '学院编码',
  `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
  `summary` text COMMENT '实时摘要信息',
  `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `remark` mediumtext COMMENT '备注',
  `reset_count` int DEFAULT '0' COMMENT '重试次数',
  `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
  `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='云阅卷同步任务表';

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_task`;
CREATE TABLE `t_b_task` (
  `id` bigint NOT NULL COMMENT '主键',
  `semester_id` bigint DEFAULT NULL COMMENT '学期',
  `exam_id` bigint DEFAULT NULL COMMENT '考试',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
  `type` varchar(50) NOT NULL COMMENT '任务类型，USER_IMPORT：用户导入，\nQUESTION_MISSION_BATCH_CREATE：批量新建命题任务，\nSAMPLE_EXPORT：导出审核样本，\nPAPER_DOWNLOAD：卷库下载，\nEXAMINATION_IMPORT：考务数据导入，\nEXAMINATION_EXPORT：考务数据导出，\nPRINT_PDF_DOWNLOAD：批量下载pdf，\nCREATE_PDF：生成pdf，\nCOURSE_IMPORT：课程导入',
  `entity_id` bigint DEFAULT NULL COMMENT '关联业务对象id',
  `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
  `summary` text COMMENT '实时摘要信息',
  `progress` double DEFAULT NULL COMMENT '执行进度',
  `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
  `import_file_name` varchar(100) DEFAULT NULL COMMENT '导入文件名',
  `import_file_path` varchar(500) DEFAULT NULL COMMENT '导入文件路径',
  `result_file_path` varchar(500) DEFAULT NULL COMMENT '导出文件路径',
  `error_file_path` varchar(500) DEFAULT NULL COMMENT '错误数据文件路径',
  `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `remark` mediumtext COMMENT '备注',
  `obj_name` varchar(500) DEFAULT NULL COMMENT '实体名称',
  `reset_count` int DEFAULT '0' COMMENT '重试次数',
  `version` int DEFAULT '0' COMMENT '更新版本号',
  `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='导入导出任务表';

-- ----------------------------
-- Table structure for t_c_statistics
-- ----------------------------
DROP TABLE IF EXISTS `t_c_statistics`;
CREATE TABLE `t_c_statistics` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `exam_task_detail_ids` mediumtext COMMENT '命题任务明细id',
  `exam_detail_course_ids` mediumtext COMMENT '考务数据课程id',
  `college_id` bigint NOT NULL COMMENT '学院id',
  `college_name` varchar(100) NOT NULL COMMENT '学院名称',
  `course_name` varchar(100) NOT NULL COMMENT '课程名称',
  `course_code` varchar(100) NOT NULL COMMENT '课程代码',
  `teaching_room_id` bigint NOT NULL COMMENT '教研室id',
  `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
  `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
  `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
  `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `batch_no` varchar(100) NOT NULL COMMENT '批次号',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `print_sum` varchar(100) DEFAULT NULL COMMENT '总印份数',
  `status` varchar(50) DEFAULT NULL COMMENT '完成状态，FINISH：已完成，UN_FINISH：未完成',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '命题明细id',
  `clazz_content` mediumtext COMMENT '班级详细信息内容',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_c_statistics_temp
-- ----------------------------
DROP TABLE IF EXISTS `t_c_statistics_temp`;
CREATE TABLE `t_c_statistics_temp` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `exam_task_detail_id` bigint DEFAULT NULL COMMENT '命题任务明细id',
  `exam_detail_course_id` bigint DEFAULT NULL COMMENT '考务数据课程id',
  `college_id` bigint NOT NULL COMMENT '学院id',
  `college_name` varchar(100) NOT NULL COMMENT '学院名称',
  `course_name` varchar(100) NOT NULL COMMENT '课程名称',
  `course_code` varchar(100) NOT NULL COMMENT '课程代码',
  `teaching_room_id` bigint NOT NULL COMMENT '教研室id',
  `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
  `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
  `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
  `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `batch_no` varchar(100) NOT NULL COMMENT '批次号',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `data_source` varchar(50) DEFAULT NULL COMMENT '数据来源，IMPORT：导入，JOIN：关联查询的数据',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '命题明细id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_f_custom_flow
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow`;
CREATE TABLE `t_f_custom_flow` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(100) NOT NULL COMMENT '流程名称',
  `model_type` varchar(50) NOT NULL COMMENT '模式类型，USER_FIXED：指定人员,APPROVE_SET：发起人指定',
  `type` varchar(100) NOT NULL COMMENT '流程类型，ELECTRON_FLOW：电子交卷审核，QUESTION_ELECTRON_FLOW：题库试题提交审核，PAPER_ELECTRON_FLOW：题库试卷审核',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `publish` tinyint NOT NULL DEFAULT '1' COMMENT '是否发布，0：否，1：是',
  `object_data` mediumtext NOT NULL COMMENT '自定义流程源数据',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `version` bigint NOT NULL DEFAULT '1' COMMENT '版本号',
  `flow_key` varchar(100) DEFAULT NULL COMMENT '流程key',
  `act_flow_id` varchar(100) DEFAULT NULL COMMENT '流程版本id',
  `flow_process_var` mediumtext COMMENT '流程过程变量数据',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `custom_flow_schoolId_type_idx` (`school_id`,`type`,`enable`,`publish`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程表';

-- ----------------------------
-- Table structure for t_f_custom_flow_entity
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_entity`;
CREATE TABLE `t_f_custom_flow_entity` (
  `id` bigint NOT NULL COMMENT '主键',
  `t_f_custom_flow_id` bigint NOT NULL COMMENT '自定义流程id',
  `flow_id` bigint DEFAULT NULL COMMENT '流程id',
  `flow_process_var` mediumtext NOT NULL COMMENT '流程过程变量',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `flow_entity_flowId_idx` (`flow_id`) USING BTREE,
  KEY `flow_entity_custom_flowId_idx` (`t_f_custom_flow_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程实体数据';

-- ----------------------------
-- Table structure for t_f_flow_approve
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_approve`;
CREATE TABLE `t_f_flow_approve` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `flow_id` bigint NOT NULL COMMENT '流程id',
  `approve_id` bigint DEFAULT NULL COMMENT '审核id',
  `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `setup` tinyint NOT NULL COMMENT '计划步骤',
  `approve_setup` int DEFAULT NULL COMMENT '审批步骤',
  `approve_max_setup` int DEFAULT NULL COMMENT '最大审批步骤',
  `transfer_assignee` mediumtext COMMENT '流程转办',
  `all_assignee` mediumtext COMMENT '会签审批人',
  `reject_link` mediumtext COMMENT '驳回路线',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `flow_approve_idx` (`school_id`,`flow_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程审核表';

-- ----------------------------
-- Table structure for t_f_flow_join
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_join`;
CREATE TABLE `t_f_flow_join` (
  `id` bigint NOT NULL COMMENT '主键',
  `flow_entity_id` bigint NOT NULL COMMENT '自定义流程实体id',
  `object_table` varchar(100) NOT NULL COMMENT '实体表',
  `object_id` bigint NOT NULL COMMENT '实体id',
  `flow_id` bigint NOT NULL COMMENT '流程id',
  `level` int NOT NULL COMMENT '流程等级',
  `type` varchar(50) NOT NULL COMMENT '类型，NEW：新增，RESTART：重新提交',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `task_flowId_idx` (`flow_id`) USING BTREE,
  KEY `task_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='实体和流程关联表';

-- ----------------------------
-- Table structure for t_f_flow_log
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_log`;
CREATE TABLE `t_f_flow_log` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `flow_id` bigint NOT NULL COMMENT '流程id',
  `object_table` varchar(100) DEFAULT NULL COMMENT '实体表',
  `object_id` bigint NOT NULL COMMENT '实体id',
  `approve_id` bigint DEFAULT NULL COMMENT '审核人id',
  `approve_setup` tinyint DEFAULT NULL COMMENT '审核步骤',
  `approve_remark` mediumtext COMMENT '审核意见',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `approve_operation` varchar(50) NOT NULL COMMENT '审批操作，SUBMIT：提交，APPROVE：审批，REJECT：驳回，END：终止',
  `pend_approve_id` mediumtext COMMENT '待审核人id',
  `paper_attachment_id` mediumtext COMMENT '试卷附件',
  `approve_multi_setup` int DEFAULT NULL COMMENT '审批步骤',
  `approve_user_approve_type` varchar(30) DEFAULT NULL COMMENT '审批用户类别，ORDER：依次审批，ALL：会签（所有人必须审批），SOME：或签（一名审批人同意或拒绝即可）',
  `task_id` bigint DEFAULT NULL COMMENT '流程节点id',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `flow_log_idx` (`school_id`,`flow_id`) USING BTREE,
  KEY `flow_log_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程流水表';

-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
DROP TABLE IF EXISTS `t_g_error`;
CREATE TABLE `t_g_error` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `obj_id` bigint DEFAULT NULL COMMENT '实体id',
  `summary` mediumtext COMMENT '实时摘要信息',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='全局异常错误信息表';

-- ----------------------------
-- Table structure for t_m_mq_message
-- ----------------------------
DROP TABLE IF EXISTS `t_m_mq_message`;
CREATE TABLE `t_m_mq_message` (
  `id` varchar(50) NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `topic` varchar(100) NOT NULL COMMENT '消息topic',
  `tag` varchar(100) NOT NULL COMMENT '消息tag',
  `body` mediumtext COMMENT '消息内容',
  `type` varchar(50) NOT NULL COMMENT '消息类型',
  `obj_id` varchar(50) NOT NULL COMMENT '关联业务id',
  `obj_name` varchar(100) DEFAULT NULL COMMENT '关联业务名称',
  `ack` tinyint NOT NULL COMMENT '消息ack',
  `sequence` int DEFAULT NULL COMMENT '消息序号',
  `properties` varchar(1000) DEFAULT NULL COMMENT '扩展类型',
  `create_time` bigint NOT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `timestamp` bigint NOT NULL COMMENT '时间戳',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='mq消息';

-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
DROP TABLE IF EXISTS `t_s_auth`;
CREATE TABLE `t_s_auth` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `access_key` varchar(255) DEFAULT NULL COMMENT 'accessKey',
  `access_secret` varchar(255) DEFAULT NULL COMMENT 'accessSecret',
  `description` text COMMENT '离线授权证书',
  `type` varchar(16) NOT NULL COMMENT '授权类型',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `expire_time` bigint DEFAULT NULL COMMENT '过期时间',
  `file` mediumblob COMMENT '文件数据',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='激活授权配置表';

-- ----------------------------
-- Table structure for t_school_privilege
-- ----------------------------
DROP TABLE IF EXISTS `t_school_privilege`;
CREATE TABLE `t_school_privilege` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `privilege_id` bigint NOT NULL COMMENT '菜单权限id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校菜单权限表';

-- ----------------------------
-- Table structure for t_school_role
-- ----------------------------
DROP TABLE IF EXISTS `t_school_role`;
CREATE TABLE `t_school_role` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `role_id` bigint NOT NULL COMMENT '菜单权限id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校角色权限表';

-- ----------------------------
-- Table structure for t_sync_exam_log
-- ----------------------------
DROP TABLE IF EXISTS `t_sync_exam_log`;
CREATE TABLE `t_sync_exam_log` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `exam_id` bigint DEFAULT NULL COMMENT '云阅卷考试ID',
  `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
  `exam_name` varchar(100) DEFAULT NULL COMMENT '云阅卷考试名称',
  `status` varchar(20) DEFAULT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
  `result` varchar(20) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  `start_time` bigint DEFAULT NULL COMMENT '同步开始时间',
  `end_time` bigint DEFAULT NULL COMMENT '结束同步时间',
  `summary` mediumtext,
  `total_courses` int DEFAULT NULL COMMENT '总课程数',
  `total_exam_tasks` int DEFAULT NULL COMMENT '总命题任务数',
  `total_students` int DEFAULT NULL COMMENT '总考生数',
  `success_students` int DEFAULT NULL COMMENT '成功人数',
  `fail_students` int DEFAULT NULL COMMENT '失败人数',
  `error_file_path` varchar(500) DEFAULT NULL COMMENT '错误日志文件地址',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='同步考试日志表';

-- ----------------------------
-- Table structure for t_sync_exam_student_score
-- ----------------------------
DROP TABLE IF EXISTS `t_sync_exam_student_score`;
CREATE TABLE `t_sync_exam_student_score` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `semester_id` bigint DEFAULT NULL COMMENT '学期id',
  `major_name` varchar(100) DEFAULT NULL COMMENT '专业id',
  `clazz_id` bigint DEFAULT NULL COMMENT '班级id',
  `exam_id` bigint DEFAULT NULL COMMENT '云阅卷考试id',
  `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
  `exam_code` varchar(100) DEFAULT NULL COMMENT '云阅卷考试标识',
  `exam_number` varchar(100) DEFAULT NULL COMMENT '准考证号',
  `student_code` varchar(100) DEFAULT NULL COMMENT '学号',
  `name` varchar(100) NOT NULL COMMENT '姓名',
  `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
  `subject_code` varchar(100) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
  `paper_type` varchar(50) DEFAULT NULL COMMENT '试卷类型',
  `college` varchar(100) DEFAULT NULL COMMENT '学院',
  `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级',
  `teacher` varchar(100) DEFAULT NULL COMMENT '教师',
  `status` tinyint DEFAULT NULL COMMENT '考生状态，1-正常，2-缺考(包含未上传)，3-违纪',
  `sheet_urls` mediumtext COMMENT '原卷url',
  `local_sheet_urls` mediumtext COMMENT '从云阅卷下载下来的原图url',
  `total_score` double DEFAULT NULL COMMENT '总分',
  `objective_score` double DEFAULT NULL COMMENT '客观题分数',
  `subjective_score` double DEFAULT NULL COMMENT '主观题分数',
  `sync_data` mediumtext COMMENT '同步数据json',
  `trajectory_urls` varchar(500) DEFAULT NULL COMMENT '轨迹图地址',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `score_schoolId_idx` (`school_id`,`semester_id`,`org_id`,`major_name`,`clazz_id`,`student_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='同步考生成绩表';

-- ----------------------------
-- Table structure for t_sync_stmms_exam
-- ----------------------------
DROP TABLE IF EXISTS `t_sync_stmms_exam`;
CREATE TABLE `t_sync_stmms_exam` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `org_code` varchar(45) DEFAULT NULL COMMENT '机构代码',
  `exam_id` int DEFAULT NULL COMMENT '云阅卷ID',
  `exam_name` varchar(45) DEFAULT NULL COMMENT '云阅卷考试名称',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='云阅卷考试表';

-- ----------------------------
-- Table structure for teach_clazz
-- ----------------------------
DROP TABLE IF EXISTS `teach_clazz`;
CREATE TABLE `teach_clazz` (
  `id` bigint NOT NULL COMMENT '教学班表主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `teach_clazz_code` varchar(20) DEFAULT NULL COMMENT '教学班编号',
  `teach_clazz_name` varchar(50) NOT NULL COMMENT '教学班名称',
  `user_id` bigint NOT NULL COMMENT '用户id',
  `basic_course_id` bigint NOT NULL COMMENT '基础课程主键',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `teach_clazz_unique` (`school_id`,`basic_course_id`,`teach_clazz_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教学班表';

-- ----------------------------
-- Table structure for teach_course
-- ----------------------------
DROP TABLE IF EXISTS `teach_course`;
CREATE TABLE `teach_course` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `basic_course_id` bigint NOT NULL COMMENT '基础课程id',
  `user_id` bigint NOT NULL COMMENT '创建此教学课程的用户id',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否删除',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `teach_course_unique` (`school_id`,`basic_course_id`,`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教学课程表';

-- ----------------------------
-- Table structure for teach_student
-- ----------------------------
DROP TABLE IF EXISTS `teach_student`;
CREATE TABLE `teach_student` (
  `id` bigint NOT NULL COMMENT '教学学生关系表主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `student_id` bigint NOT NULL COMMENT '学生id',
  `teach_clazz_id` bigint NOT NULL COMMENT '教学班级id',
  `basic_course_id` bigint NOT NULL COMMENT '基础课程id（冗余）',
  `user_id` bigint NOT NULL COMMENT '用户id（冗余）',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `teach_student_unique` (`school_id`,`student_id`,`teach_clazz_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教学学生关系表';

-- ----------------------------
-- Function structure for getRootChildId
-- ----------------------------
DROP FUNCTION IF EXISTS `getRootChildId`;
delimiter ;;
CREATE FUNCTION `getRootChildId`(rootId bigint)
 RETURNS mediumtext CHARSET utf8mb4
  DETERMINISTIC
BEGIN
    DECLARE ptemp MEDIUMTEXT;
    DECLARE ctemp MEDIUMTEXT;
    SET ptemp = '#';
    SET ctemp = cast(rootId as CHAR);
    WHILE ctemp IS NOT NULL DO
      SET ptemp = concat(ptemp, ',', ctemp);
SELECT group_concat(id) INTO ctemp FROM sys_org WHERE FIND_IN_SET(parent_id, ctemp) > 0;
END WHILE;
RETURN ptemp;
END
;;
delimiter ;

INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1, '基础配置', 'base', 'MENU', NULL, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2, '用户管理', 'user', 'MENU', 1, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3, '配置管理', 'rule', 'MENU', 1, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4, '组织架构', 'OrganizationManage', 'MENU', 2, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (5, '用户管理', 'UserManage', 'MENU', 2, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (6, '角色管理', 'RoleManage', 'MENU', 2, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (7, '权限管理', 'MenuManage', 'MENU', 129, 6, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (8, '通用规则', 'CommonRule', 'MENU', 3, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (9, '题卡规则管理', 'CardRuleManage', 'MENU', 3, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (11, '印品管理', 'PrintTemplateManage', 'MENU', 3, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (13, '课程管理', 'CourseManage', 'MENU', 356, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (14, '考试中心', 'exam', 'MENU', NULL, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (38, '印刷管理', 'print', 'MENU', 14, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (39, '命题任务管理', 'ExamTaskManage', 'MENU', 652, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (40, '入库申请', 'TaskApplyManage', 'MENU', 652, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (41, '入库审核', 'TaskReviewManage', 'MENU', 652, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (42, '卷库查询', 'TaskPaperManage', 'MENU', 652, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (43, '印刷计划管理', 'PrintPlanManage', 'MENU', 38, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (44, '考务数据导入', 'BusinessDataExport', 'MENU', 38, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (45, '考务明细查询', 'BusinessDataDetail', 'MENU', 38, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (47, '印刷任务管理', 'PrintTaskManage', 'MENU', 38, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (48, '查询', '/api/admin/exam/print/list', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (49, '按类型查询模板', '/api/admin/exam/print/get_template', 'URL', 43, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (50, '新增/修改', '/api/admin/exam/print/save', 'URL', 43, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (51, '删除', '/api/admin/exam/print/remove', 'URL', 43, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (52, '查询', '/api/admin/exam/print/data_list', 'URL', 44, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (53, '汇总数据查询', '/api/admin/exam/print/data_total', 'URL', 44, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (54, '导入', '/api/admin/exam/print/data_import', 'URL', 44, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (55, '查看明细', '/api/admin/exam/print/data_detail_list', 'URL', 44, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (56, '查询', '/api/admin/exam/print/relate_list', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (57, '关联/更换试卷', '/api/admin/exam/print/relate_update', 'URL', 43, 2, 'AUTH', '58', 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (58, '查询所有试卷编号/关联卷型', '/api/admin/exam/print/relate_get_paper_numbers', 'URL', 43, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (59, '查询', '/api/admin/exam/print/task_list', 'URL', 47, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (60, '提交印刷', '/api/admin/exam/print/task_submit', 'URL', 47, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (61, '撤回提交', '/api/admin/exam/print/task_cancel', 'URL', 47, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (62, '汇总数据查询', '/api/admin/exam/print/task_total_data', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (63, '查询', '/api/admin/exam/task/list', 'URL', 39, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (64, '更改/指派命题老师', '/api/admin/exam/task/assign_user', 'URL', 39, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (65, '启用/禁用', '/api/admin/exam/task/enable', 'URL', 39, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (66, '新建', '/api/admin/exam/task/save', 'URL', 39, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (67, '批量新建-文件导入', '/api/admin/exam/task/import', 'URL', 39, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (68, '批量新建', '/api/admin/exam/task/save_batch', 'URL', 39, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (69, '审核历史', '/api/admin/exam/task/review_list', 'URL', 39, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (70, '查询', '/api/admin/exam/task/apply_list', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (71, '提交/暂存', '/api/admin/exam/task/apply_save', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (72, '撤回/重新申请', '/api/admin/exam/task/apply_status', 'URL', 40, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (73, '查询未审核', '/api/admin/exam/task/review_list_unaudited', 'URL', 41, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (74, '审核', '/api/admin/exam/task/review_save', 'URL', 41, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (75, '批量审核', '/api/admin/exam/task/review_save_batch', 'URL', 41, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (76, '查询', '/api/admin/exam/task/paper_list', 'URL', 42, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (77, '查询', '/api/admin/sys/user/list', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (78, '新增/修改', '/api/admin/sys/user/save', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (79, '启用/禁用', '/api/admin/sys/user/enable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (80, '重置密码', '/api/admin/sys/user/reset_password', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (81, '导入', '/api/admin/sys/user/import', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (82, '绑定角色', '/api/admin/sys/user/bind_roles', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (83, '查询', '/api/admin/sys/role/list', 'URL', 6, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (84, '删除', '/api/admin/sys/role/remove', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (85, '新增/修改', '/api/admin/sys/role/save', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (86, '启用/禁用', '/api/admin/sys/role/enable', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (87, '用户已绑定角色列表', '/api/admin/sys/role/get_user_roles', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (88, '查询', '/api/admin/sys/privilege/list', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (89, '新增/修改', '/api/admin/sys/privilege/save', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (90, '删除', '/api/admin/sys/privilege/remove', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (91, '角色已绑定权限列表', '/api/admin/sys/privilege/get_role_privileges', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (92, '组织机构-查询', '/api/admin/sys/org/list', 'URL', 4, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (93, '组织机构-新增/修改', '/api/admin/sys/org/save', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (94, '组织机构-删除', '/api/admin/sys/org/remove', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (95, '组织机构-启用/禁用', '/api/admin/sys/org/enable', 'URL', 4, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (96, '查询', '/api/admin/basic/course/list', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (97, '新增/修改', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (98, '删除', '/api/admin/basic/course/remove', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (99, '查询', '/api/admin/basic/exam_rule/list', 'URL', 8, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (100, '新增/修改', '/api/admin/basic/exam_rule/save', 'URL', 8, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (101, '查询', '/api/admin/basic/card_rule/list', 'URL', 9, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (102, '查询单个', '/api/admin/basic/card_rule/get_one', 'URL', 9, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (103, '新增/修改', '/api/admin/basic/card_rule/save', 'URL', 9, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (104, '删除', '/api/admin/basic/card_rule/delete', 'URL', 9, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (105, '查询-汇总数据', '/api/admin/exam/print/relate_total', 'URL', 43, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (112, '系统设置', 'data', 'MENU', 1, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (113, '任务管理', 'DataTaskManage', 'MENU', 112, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (114, '查询', '/api/admin/data/task/query', 'URL', 113, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (117, '命题待办接口', '/api/admin/work/exam/task/ready', 'URL', 16, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (118, '命题待办计数接口', '/api/admin/work/exam/task/ready_count', 'URL', 16, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (119, '命题分配待办接口', '/api/admin/work/exam/task/new', 'URL', 16, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (120, '命题分配待办计数接口', '/api/admin/work/exam/task/new_count', 'URL', 16, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (121, '审核待办接口', '/api/admin/work/exam/task/submit', 'URL', 16, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (122, '审核待办计数接口', '/api/admin/work/exam/task/submit_count', 'URL', 16, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (123, '查询', '/api/admin/basic/template/list', 'URL', 11, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (124, '新增/修改', '/api/admin/basic/template/save', 'URL', 11, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (125, '启用/禁用', '/api/admin/basic/template/enable', 'URL', 11, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (129, '公共接口', 'common', 'MENU', NULL, 12, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (130, '用户登录相关', 'login', 'MENU', 129, 1, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (131, '用户登录', '/api/admin/common/login', 'URL', 130, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (132, '查询用户权限', '/api/admin/common/get_menu', 'URL', 130, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (133, '发送验证码', '/api/admin/common/get_verify_code', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (134, '修改密码', '/api/admin/sys/user/update_password', 'URL', 130, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (135, '用户登出', '/api/admin/common/logout', 'URL', 130, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (136, '模糊查询', 'query', 'MENU', 129, 2, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (137, '模糊查询-印刷计划', '/api/admin/exam/print/query', 'URL', 136, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (138, '题卡规则', '/api/admin/basic/card_rule/query', 'URL', 136, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (139, '模糊查询-课程', '/api/admin/basic/course/query', 'URL', 136, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (140, '模糊查询-试卷编号', '/api/admin/exam/task/paper_number_query', 'URL', 136, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (141, '模糊查询-命题老师', '/api/admin/exam/task/user_query', 'URL', 136, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (142, '模糊查询-考点', '/api/admin/exam/print/data_place_query', 'URL', 136, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (143, '模糊查询-考场', '/api/admin/exam/print/data_room_query', 'URL', 136, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (144, '文件相关', 'file', 'MENU', 129, 3, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (145, '文件上传', '/api/admin/common/file/upload', 'URL', 144, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (146, '文件查询', '/api/admin/common/file/get_one', 'URL', 144, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (147, '文件下载', '/api/admin/common/file/download', 'URL', 144, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (148, '文件预览', '/api/admin/common/file/preview', 'URL', 144, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (149, '系统相关', 'sys', 'MENU', 129, 4, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (150, '查询枚举类型', '/api/admin/common/get_enums', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (151, '查询学校列表', '/api/admin/common/school/list', 'URL', 149, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (152, '查询系统参数', '/api/admin/common/sys_config/get_one', 'URL', 149, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (153, '根据学校代码查询学校信息', '/api/admin/common/school/query_by_school_code', 'URL', 149, 4, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (154, '其他', 'other', 'MENU', 129, 5, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (155, '用户导入（异步）', '/api/admin/sys/user/async/import', 'URL', 154, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (156, '用户导出', '/api/admin/sys/user/export', 'URL', 154, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (157, '模板下载', '/api/admin/exam/print/template_download', 'URL', 44, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (158, '结果导出', '/api/admin/exam/print/data_export', 'URL', 44, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (159, '查询', '/api/admin/exam/print/data_detail', 'URL', 45, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (160, '重新提交印刷', '/api/admin/exam/print/task_resubmit', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (161, '查看PDF', '/api/admin/exam/print/task_view_pdf', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (162, '批量下载PDF', '/api/admin/exam/print/task_download_pdf', 'URL', 47, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (163, '导出审核样本', '/api/admin/exam/task/review_export', 'URL', 41, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (164, '下载试卷', '/api/admin/exam/task/paper_download', 'URL', 42, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (165, '试卷启用/禁用', '/api/admin/exam/task/paper_enable', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (166, '查询已审核', '/api/admin/exam/task/review_list_audited', 'URL', 41, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (167, '查询可选择题卡', '/api/admin/exam/card/select_card_list', 'URL', 40, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (168, '查询申请详情', '/api/admin/exam/task/apply_get_one', 'URL', 40, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (169, '根据课程查询命题老师', '/api/admin/sys/user/user_list', 'URL', 39, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (170, '客服制卡', 'customer', 'MENU', NULL, 9, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (171, '客服制卡', 'customerManage', 'MENU', 170, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (172, '题卡审核', 'CustomerCard', 'MENU', 171, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (173, '客服制卡-查询', '/api/admin/exam/card/cust_list', 'URL', 172, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (174, '导入模板下载', '/api/admin/common/download_import_template', 'URL', 149, 15, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (175, '题卡管理', 'card', 'MENU', NULL, 10, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (176, '查询题卡规则', '/api/admin/basic/card_rule/get_one', 'URL', 175, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (177, '查询题卡详情', '/api/admin/exam/card/get_one', 'URL', 175, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (178, '保存/提交', '/api/admin/exam/card/save', 'URL', 175, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (179, '查询考务规则', '/api/admin/basic/exam_rule/list', 'URL', 149, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (180, '查询可设置权限列表', '/api/admin/sys/privilege/list_auth', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (181, '超管中心', 'admin', 'MENU', 129, 7, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (182, '查询客服', '/api/admin/sys/user/list_customer', 'URL', 181, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (183, '新增客服', '/api/admin/sys/user/save_customer', 'URL', 181, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (184, '查询用户可绑定角色', '/api/admin/sys/role/list_to_user', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (185, '客服制卡-批量下载文件', '/api/admin/exam/card/download_files', 'URL', 172, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (186, '查看印品', '/api/admin/exam/print/template_view', 'URL', 47, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (187, '客户端登录', '/api/admin/client/user/login', 'URL', NULL, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (188, '学生明细', '/api/admin/exam/print/get_student_detail', 'URL', 45, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (189, '修改', '/api/admin/exam/task/paper_update', 'URL', 42, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (190, '重新生成pdf', '/api/admin/data/task/reset_create_pdf', 'URL', 113, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (191, '批量下载试卷PDF、题卡', '/api/admin/exam/task/paper_card_download_pdf', 'URL', 42, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (192, '预览', '/api/admin/basic/template/read_content', 'URL', 11, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (195, '获取服务器时间', '/api/admin/common/get_system_time', 'URL', 129, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (198, '数据同步', '/api/admin/exam/print/sync_data_cloud', 'URL', 47, 9, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (199, '客户端', 'client', 'MENU', NULL, 11, NULL, '200,201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (200, '试卷打样-查询列表', '/api/admin/client/paper_try/list', 'URL', 199, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (201, '试卷打样-查看/试印/重印', '/api/admin/client/paper_try/print', 'URL', 199, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (202, '试卷打样-批量试印', '/api/admin/client/paper_try/print_batch', 'URL', 199, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (203, '试卷打样-标记合格状态', '/api/admin/client/paper_try/tag_pass', 'URL', 199, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (204, '查询列表', '/api/admin/client/print/task_list', 'URL', 199, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (205, '导出', '/api/admin/client/print/task_list_export', 'URL', 199, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (206, '汇总数据查询', '/api/admin/client/print/task_total_data', 'URL', 199, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (207, '查看', '/api/admin/client/print/preview', 'URL', 199, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (208, '印刷/缓存数据', '/api/admin/client/print/get_print_data', 'URL', 199, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (209, '批量缓存', '/api/admin/client/print/get_print_data_batch', 'URL', 199, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (210, '缓存后更新状态', '/api/admin/client/print/update_download', 'URL', 199, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (211, '校验', '/api/admin/client/print/validate_data', 'URL', 199, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (212, '更新打印进度', '/api/admin/client/print/update_progress', 'URL', 199, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (213, '重打-查询考生列表', '/api/admin/client/print/list_student', 'URL', 199, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (214, '重打-内容查询', '/api/admin/client/print/get_reprint_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (215, '统计查询-查询列表', '/api/admin/client/print/statistics_list', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (216, '统计查询-汇总数据查询', '/api/admin/client/print/statistics_total_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (217, '查询', 'Select', 'BUTTON', 5, 1, 'AUTH', '77', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (218, '新增', 'Add', 'BUTTON', 5, 2, 'AUTH', '78', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (219, '编辑', 'Edit', 'LINK', 5, 1, 'AUTH', '78', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (220, '启用/禁用', 'Enable', 'LINK', 5, 2, 'AUTH', '79', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (221, '列表', 'List', 'LIST', 5, 1, 'AUTH', '77', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (222, '查询条件', 'Condition', 'CONDITION', 5, 1, 'AUTH', '184,87', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (223, '重置密码', 'Reset', 'LINK', 5, 3, 'AUTH', '80', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (224, '新增', 'Add', 'BUTTON', 4, 1, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (225, '子部门', 'Add', 'LINK', 4, 1, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (226, '编辑', 'Edit', 'LINK', 4, 2, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (227, '删除', 'Delete', 'LINK', 4, 3, 'AUTH', '94', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (228, '树形列表', 'List', 'LIST', 4, 1, 'AUTH', '92', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (229, '查询', 'Select', 'BUTTON', 6, 1, 'AUTH', '83,88', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (230, '新增', 'Add', 'BUTTON', 6, 2, 'AUTH', '85,89', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (231, '编辑', 'Edit', 'LINK', 6, 1, 'AUTH', '83,91', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (232, '删除', 'Delete', 'LINK', 6, 2, 'AUTH', '84,90', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (233, '列表', 'List', 'LIST', 6, 1, 'AUTH', '83', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (234, '查询条件', 'Condition', 'CONDITION', 4, 1, 'AUTH', '92', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (235, '查询条件', 'Condition', 'CONDITION', 6, 1, 'AUTH', '83', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (236, '考务规则配置', 'Add', 'BUTTON', 8, 1, 'AUTH', '99, 100', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (237, '查询', 'Select', 'BUTTON', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (238, '新增', 'Add', 'BUTTON', 9, 2, 'AUTH', '103', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (239, '查询条件', 'Condition', 'CONDITION', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (240, '列表', 'List', 'LIST', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (243, '导入', 'Import', 'BUTTON', 5, 3, 'AUTH', '81', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (249, '查询', 'Select', 'BUTTON', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (250, '新增', 'Add', 'BUTTON', 11, 2, 'AUTH', '124', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (251, '预览', 'Preview', 'LINK', 11, 1, 'AUTH', '192', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (252, '编辑', 'Edit', 'LINK', 11, 2, 'AUTH', '124,763', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (253, '启用/禁用', 'Enable', 'LINK', 11, 3, 'AUTH', '125', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (254, '查询条件', 'Condition', 'CONDITION', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (255, '列表', 'List', 'LIST', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (263, '查询', 'Select', 'BUTTON', 13, 1, 'AUTH', '96,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (268, '编辑', 'Edit', 'LINK', 13, 1, 'AUTH', '97', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (269, '删除', 'Delete', 'LINK', 13, 2, 'AUTH', '368', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (270, '查询条件', 'Condition', 'CONDITION', 13, 2, 'AUTH', '96', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (271, '列表', 'List', 'LIST', 13, 1, 'AUTH', '96', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (272, '查询', 'Select', 'BUTTON', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (273, '批量新增', 'BatchAdd', 'BUTTON', 39, 2, 'AUTH', '67', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (274, '新增', 'Add', 'BUTTON', 39, 3, 'AUTH', '66,169,441,442,445,482', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (275, '预览', 'Preview', 'LINK', 39, 1, 'AUTH', '68', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (277, '启用/禁用', 'Cancel', 'LINK', 39, 3, 'AUTH', '65', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (278, '查询条件', 'Condition', 'CONDITION', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (279, '列表', 'List', 'LIST', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (280, '查询', 'Select', 'BUTTON', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (281, '预览', 'Preview', 'LINK', 40, 1, 'AUTH', '168,477,568,417,420,421,441,442,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (282, '撤销申请', 'End', 'LINK', 40, 2, 'AUTH', '72', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (283, '立即申请', 'Edit', 'LINK', 40, 3, 'AUTH', '71,167,424,477,417,420,421,441,442,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (284, '查询条件', 'Condition', 'CONDITION', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (285, '列表', 'List', 'LIST', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (288, '查询（未审核）', 'NotReviewSelect', 'BUTTON', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (289, '导出审核样本（未审核）', 'NotReviewExport', 'BUTTON', 41, 2, 'AUTH', '163', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (290, '批量不通过（未审核）', 'NotReviewBatchNotPass', 'BUTTON', 41, 3, 'AUTH', '75', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (291, '批量通过（未审核）', 'NotReviewBatchPass', 'BUTTON', 41, 4, 'AUTH', '75', 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (292, '审核（未审核）', 'NotReviewEdit', 'LINK', 41, 1, 'AUTH', '74,417,420,421,441,442,477,417,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (293, '查询条件（未审核）', 'NotReviewCondition', 'CONDITION', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (294, '列表（未审核）', 'NotReviewList', 'LIST', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (295, '查询（已审核）', 'ReviewSelect', 'BUTTON', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (296, '查看详情（已审核）', 'ReviewPreview', 'LINK', 41, 1, 'AUTH', '74,477,417,420,421,441,442,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (297, '查询条件（已审核）', 'ReviewCondition', 'CONDITION', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (298, '列表（已审核）', 'ReviewList', 'LIST', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (299, '查询', 'Select', 'BUTTON', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (300, '批量下载试卷题卡', 'BatchDownload', 'BUTTON', 42, 2, 'AUTH', '191', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (301, '预览', 'Preview', 'LINK', 42, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (302, '启用/禁用', 'Enable', 'LINK', 42, 2, 'AUTH', '165', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (303, '下载', 'Download', 'LINK', 42, 3, 'AUTH', '164', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (304, '查询条件', 'Condition', 'CONDITION', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (305, '列表', 'List', 'LIST', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (308, '编辑', 'Edit', 'LINK', 42, 4, 'AUTH', '189', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (309, '查询', 'Select', 'BUTTON', 43, 1, 'AUTH', '48,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (310, '新增', 'Add', 'BUTTON', 43, 2, 'AUTH', '50,49,621', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (311, '查看', 'Preview', 'LINK', 43, 1, 'AUTH', '56,57,58,105,825,826', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (312, '编辑', 'Edit', 'LINK', 43, 2, 'AUTH', '50', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (313, '删除', 'Delete', 'LINK', 43, 3, 'AUTH', '51', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (314, '查询条件', 'Condition', 'CONDITION', 43, 1, 'AUTH', '48', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (315, '列表', 'List', 'LIST', 43, 1, 'AUTH', '48,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (316, '查询', 'Select', 'BUTTON', 44, 1, 'AUTH', '52,53', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (317, '模版下载', 'TempleteDownload', 'BUTTON', 44, 2, 'AUTH', '157', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (318, '导出', 'Export', 'BUTTON', 44, 3, 'AUTH', '158', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (319, '导入', 'Import', 'BUTTON', 44, 4, 'AUTH', '54', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (320, '预览', 'Preview', 'LINK', 44, 1, 'AUTH', '55', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (321, '查询条件', 'Condition', 'CONDITION', 44, 1, 'AUTH', '52', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (322, '列表', 'List', 'LIST', 44, 1, 'AUTH', '52,53', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (323, '查询', 'Select', 'BUTTON', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (324, '预览', 'Preview', 'LINK', 45, 1, 'AUTH', '188', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (325, '查询条件', 'Condition', 'CONDITION', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (326, '列表', 'List', 'LIST', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (330, '重新生成pdf', 'CreatePdf', 'LINK', 47, 6, 'AUTH', '190', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (331, '查询', 'Select', 'BUTTON', 47, 1, 'AUTH', '59,62', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (332, '查看pdf', 'Download', 'LINK', 47, 1, 'AUTH', '161,162', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (333, '查询条件', 'Condition', 'CONDITION', 47, 1, 'AUTH', '59', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (334, '列表', 'List', 'LIST', 47, 1, 'AUTH', '59', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (335, '撤回提交', 'End', 'LINK', 47, 2, 'AUTH', '61', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (336, '批量下载PDF', 'BatchDownload', 'BUTTON', 47, 2, 'AUTH', '336', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (337, '查询', 'Select', 'BUTTON', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (338, '导出日志', 'Export', 'LINK', 113, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (339, '下载文件', 'Download', 'LINK', 113, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (340, '查询条件', 'Condition', 'CONDITION', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (341, '列表', 'List', 'LIST', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (342, '预览', 'Preview', 'LINK', 9, 1, 'AUTH', '102', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (343, '编辑', 'Edit', 'LINK', 9, 2, 'AUTH', '103', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (345, '删除', 'Delete', 'LINK', 9, 3, 'AUTH', '104', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (346, '题卡审核-未审核查询', 'NotReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (347, '题卡审核-未审核批量下载试卷文件', 'NotReviewBatchDownload', 'BUTTON', 172, 2, 'AUTH', '185', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (348, '题卡审核-未审核查询条件', 'NotReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (349, '题卡审核-未审核列表', 'NotReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (350, '题卡审核-未审核预览', 'NotReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (351, '题卡审核-未审核设计题卡', 'NotReviewDesign', 'LINK', 172, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (352, '题卡审核-已审核查询', 'ReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (353, '题卡审核-已审核查询条件', 'ReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (354, '题卡审核-已审核列表', 'ReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (355, '题卡审核-已审核预览', 'ReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (356, '字典管理', 'dictionary', 'MENU', 1, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (357, '学生学院管理', 'CollegeManage', 'MENU', 356, 3, NULL, NULL, 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (358, '学生管理', 'StudentManage', 'MENU', 356, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (359, '查询', '/api/admin/basic/college/query', 'URL', 357, 1, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (360, '新增/编辑', '/api/admin/basic/college/save', 'URL', 357, 2, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (361, '删除（逻辑）', '/api/admin/basic/college/delete', 'URL', 357, 3, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (362, '查询', '/api/admin/basic/student/query', 'URL', 358, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (363, '增/编辑', '/api/admin/basic/student/save', 'URL', 358, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (364, '批量删除（逻辑）', '/api/admin/basic/student/delete_batch', 'URL', 358, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (365, '批量导入（异步）', '/api/admin/basic/student/data_import', 'URL', 358, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (366, '查询', '/api/admin/basic/course/query', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (367, '新增/编辑', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (368, '批量删除', '/api/admin/basic/course/delete_batch', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (369, '批量导入', '/api/admin/basic/course/data_import', 'URL', 13, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (370, '获取系统编码', '/api/admin/common/get_code', 'URL', 149, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (371, '根据类型查询机构', '/api/admin/sys/org/find_by_type', 'URL', 4, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (372, '查询', 'Select', 'BUTTON', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (373, '列表', 'List', 'LIST', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (374, '新增', 'Add', 'BUTTON', 357, 1, 'AUTH', '360,370', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (375, '编辑', 'Edit', 'LINK', 357, 1, 'AUTH', '360', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (376, '删除', 'Delete', 'LINK', 357, 1, 'AUTH', '361', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (377, '查询', 'Select', 'BUTTON', 358, 1, 'AUTH', '362,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (378, '列表', 'List', 'LIST', 358, 1, 'AUTH', '362', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (379, '新增', 'Add', 'BUTTON', 358, 1, 'AUTH', '359,363,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (380, '编辑', 'Edit', 'LINK', 358, 1, 'AUTH', '359,363', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (381, '删除', 'Delete', 'LINK', 358, 1, 'AUTH', '364', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (382, '导入', 'Import', 'BUTTON', 358, 2, 'AUTH', '365', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (383, '启用/禁用', 'Enable', 'LINK', 9, 4, 'AUTH', '384', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (384, '启用/禁用', '/api/admin/basic/card_rule/enable', 'URL', 9, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (385, '新增', 'Add', 'BUTTON', 13, 1, 'AUTH', '367,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (388, '导入', 'Import', 'BUTTON', 13, 2, 'AUTH', '369', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (389, '机构列表', 'List', 'LIST', 13, 1, 'AUTH', NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (390, '流程管理', 'flow', 'MENU', 1, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (391, '查询条件', 'Condition', 'CONDITION', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (392, '数据推送管理', 'PrintPlanPushManage', 'MENU', 38, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (393, '查询', 'Select', 'BUTTON', 392, 1, 'AUTH', '49,431', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (394, '批量推送', 'BatchSend', 'BUTTON', 392, 2, 'AUTH', '395,435,557,665,792,793', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (395, '校验同步规则', '/api/admin/exam/print_sync/check_sync_mode', 'URL', 392, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (396, '预览', 'Preview', 'LINK', 392, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (397, '推送', 'Send', 'LINK', 392, 2, 'AUTH', '395,435,557,665,792,793', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (398, '查询条件', 'Condition', 'CONDITION', 392, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (399, '列表', 'List', 'LIST', 392, 1, 'AUTH', '431', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (400, '查询条件', 'Condition', 'CONDITION', 358, 1, 'AUTH', '377', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (402, '流程管理', 'FlowManage', 'MENU', 390, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (403, '流程审批记录管理', 'ApproveRecordManage', 'MENU', 390, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (404, '查询', 'Select', 'BUTTON', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (405, '查询条件', 'Condition', 'CONDITION', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (406, '列表', 'List', 'LIST', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (407, '查询', 'Select', 'BUTTON', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (408, '新建流程', 'Register', 'BUTTON', 402, 2, 'AUTH', '414', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (409, '删除', 'Delete', 'LINK', 402, 2, 'AUTH', '415', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (410, '查询条件', 'Condition', 'CONDITION', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (411, '列表', 'List', 'LIST', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (413, '重新生成pdf', 'CreatePdf', 'LINK', 113, 3, 'AUTH', '190', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (414, '保存和发布流程', '/api/admin/custom/flow/save', 'URL', 402, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (415, '流程逻辑删除', '/api/admin/custom/flow/enable', 'URL', 402, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (416, '流程列表', '/api/admin/custom/flow/list', 'URL', 402, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (417, '审批流程', '/api/admin/custom/flow/task/approve', 'URL', 402, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (418, '流程审批记录列表', '/api/admin/custom/flow/approve/list', 'URL', 402, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (419, '流程审批记录逻辑删除', '/api/admin/custom/flow/approve/enable', 'URL', 402, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (420, '流程终止', '/api/admin/custom/flow/end', 'URL', 402, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (421, '获取所有流程节点', '/api/admin/custom/flow/task/all', 'URL', 402, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (422, '删除', 'Delete', 'LINK', 403, 1, 'AUTH', '419', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (427, '发布印刷任务-印刷室列表', '/api/admin/exam/task/list_task_print_house', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (431, '计划查询', '/api/admin/exam/print_sync/list_sync', 'URL', 392, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (435, '推送', '/api/admin/exam/print_sync/sync_data_cloud', 'URL', 392, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (437, '查询模板', 'List', 'LIST', 43, 1, 'AUTH', '49', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (439, '题卡标题查询', '/api/admin/exam/task/find_card_title', 'URL', 40, 7, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (440, '题卡标题查询', 'List', 'LIST', 40, 1, 'AUTH', '439', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (441, '根据流程类型获取流程节点', '/api/admin/custom/flow/get_flow_info_by_type', 'URL', 402, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (442, '流程节点转他人审批', '/api/admin/custom/flow/task/approver/exchange', 'URL', 402, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (446, '做废/恢复', '/api/admin/exam/print/task_normal', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (447, '作废/恢复', 'Normal', 'LINK', 47, 3, 'AUTH', '446', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (448, '指派/更改', 'Assign', 'LINK', 39, 4, 'AUTH', '64', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (450, '班级管理', 'ClazzManage', 'MENU', 356, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (451, '查询', '/api/admin/basic/clazz/query', 'URL', 450, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (452, '新增/编辑', '/api/admin/basic/clazz/save', 'URL', 450, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (453, '批量删除（逻辑）', '/api/admin/basic/clazz/delete_batch', 'URL', 450, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (455, '查询', 'Select', 'BUTTON', 450, 1, 'AUTH', '451', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (456, '列表', 'List', 'LIST', 450, 1, 'AUTH', '451', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (457, '新增', 'Add', 'BUTTON', 450, 1, 'AUTH', '452', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (458, '编辑', 'Edit', 'LINK', 450, 1, 'AUTH', '452', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (459, '删除', 'Delete', 'LINK', 450, 1, 'AUTH', '453', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (461, '查询条件', 'Condition', 'CONDITION', 450, 1, 'AUTH', '451', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (462, '班级数据源查询', '/api/admin/basic/clazz/datasource', 'URL', 450, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (463, '新建任务', 'ExamTaskApplyManage', 'BUTTON', 40, 8, 'AUTH', '49,167,174,371,464,465,427,642,417,420,421,441,442,640,641,642', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (464, '新建命题申请', '/api/admin/exam/task/submit_task_apply', 'URL', 40, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (465, '新建命题申请-班级列表', '/api/admin/exam/task/list_task_apply_class', 'URL', 39, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (466, '命题任务-审批-获取试卷审批表数据', '/api/admin/exam/task/find_approve_form_json', 'URL', 39, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (467, '命题任务-审批-获取试卷审批表pdf', '/api/admin/exam/task/find_approve_form_pdf', 'URL', 39, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (468, '下载审批表', 'Download', 'LINK', 40, 5, 'AUTH', '466,467', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (469, '命题计划统计', 'Statistics', 'MENU', 14, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (470, '命题计划统计表', 'StatisticsManage', 'MENU', 469, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (471, '查询', 'Select', 'BUTTON', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (472, '模板下载', 'Download', 'BUTTON', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (473, '导入', 'Import', 'BUTTON', 470, 3, 'AUTH', '475', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (474, '列表', 'List', 'LIST', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (475, '导入', '/api/admin/statistics/import', 'URL', 470, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (476, '列表', '/api/admin/statistics/list', 'URL', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (477, '命题任务-考试对象查询', '/api/admin/exam/task/find_exam_object', 'URL', 39, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (478, '刷新数据', 'Freshen', 'BUTTON', 470, 4, 'AUTH', '480', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (479, '删除数据', 'Delete', 'BUTTON', 470, 5, 'AUTH', '481', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (480, '刷新数据', '/api/admin/statistics/freshen', 'URL', 470, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (481, '删除数据', '/api/admin/statistics/delete', 'URL', 470, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (483, '阅卷管理', 'stmms', 'MENU', NULL, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (486, '阅卷管理', 'MarkTaskManage', 'MENU', 483, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (487, '成绩查询', 'ScoreArchive', 'MENU', 561, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (488, '评卷参数设置', 'UploadStructure', 'MENU', 486, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (489, '列表', '/api/admin/exam/structure/list', 'URL', 488, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (490, '上传', '/api/admin/exam/structure/upload', 'URL', 488, 2, 'AUTH', NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (491, '阅卷管理', 'MarkerLogin', 'MENU', 486, 1, NULL, '492,493,541', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (492, '评卷员登录', '/api/admin/exam/sso/marker_login', 'URL', 491, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (493, '科组长登录', '/api/admin/exam/sso/marker_leader_login', 'URL', 491, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (494, '列表', 'List', 'LIST', 488, 1, 'AUTH', '489', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (495, '分班阅（开关）', 'OpenClassReading', 'BUTTON', 488, 9, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (498, '列表', 'List', 'LIST', 487, 1, 'AUTH', '542', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (499, '查询条件', 'Condition', 'CONDITION', 487, 1, 'AUTH', '542', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (500, '查询', 'Select', 'BUTTON', 487, 1, 'AUTH', '542', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (501, '数据同步', 'Sync', 'BUTTON', 487, 2, 'AUTH', '546', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (502, '成绩导出', 'Export', 'BUTTON', 487, 3, 'AUTH', '543', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (503, '一键下载', 'BatchDownload', 'BUTTON', 487, 4, 'AUTH', '545', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (504, '查看原卷', 'Preview', 'LINK', 487, 1, 'AUTH', '563', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (505, '下载', 'Download', 'LINK', 487, 2, 'AUTH', '544', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (506, '归档管理', 'ScoreArchiveManage', 'MENU', NULL, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (507, '菜单管理', 'MenuManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (508, '自定义菜单', 'CustomMenu', 'MENU', 507, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (509, '同步', 'SyncUser', 'BUTTON', 5, 1, 'AUTH', '532', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (510, '学期管理', 'SemesterManage', 'MENU', 356, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (511, '新增', 'Add', 'BUTTON', 510, 1, 'AUTH', '534', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (512, '编辑', 'Edit', 'LINK', 510, 1, 'AUTH', '534', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (513, '删除', 'Delete', 'LINK', 510, 2, 'AUTH', '535', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (514, '列表', 'List', 'LIST', 510, 1, 'AUTH', '533', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (515, '专业管理', 'MajorManage', 'MENU', 356, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (516, '新增', 'Add', 'BUTTON', 515, 1, 'AUTH', '537', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (517, '编辑', 'Edit', 'LINK', 515, 1, 'AUTH', '537', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (518, '删除', 'Delete', 'LINK', 515, 2, 'AUTH', '538', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (519, '列表', 'List', 'LIST', 515, 1, 'AUTH', '536', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (520, '查询条件', 'Condition', 'CONDITION', 515, 1, 'AUTH', '536', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (521, '同步管理', 'SyncManage', 'MENU', 112, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (522, '查询', 'Select', 'BUTTON', 515, 1, 'AUTH', '536', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (523, '查询', 'Select', 'BUTTON', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (524, '查询条件', 'Condition', 'CONDITION', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (525, '列表', 'List', 'LIST', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (526, '导出日志', 'Export', 'LINK', 521, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (527, '查询自定义菜单权限', '/api/admin/set/menu/custom/list', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (528, '学校已绑定自定义菜单权限列表', '/api/admin/set/menu/custom/get_school_custom_privileges', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (529, '学校新增/修改自定义菜单权限', '/api/admin/set/menu/custom/save', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (530, '查询', 'Select', 'BUTTON', 508, 1, 'AUTH', '527,528', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (531, '保存', 'Save', 'BUTTON', 508, 2, 'AUTH', '529', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (532, '同步', '/api/admin/sys/user/push', 'URL', 5, 8, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (533, '查询', '/api/admin/basic/semester/query', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (534, '新增/编辑', '/api/admin/basic/semester/save', 'URL', 510, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (535, '删除', '/api/admin/basic/semester/delete', 'URL', 510, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (536, '查询', '/api/admin/basic/major/query', 'URL', 515, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (537, '新增/编辑', '/api/admin/basic/major/save', 'URL', 515, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (538, '删除', '/api/admin/basic/major/delete', 'URL', 515, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (539, '共用接口-查询字典', '/api/admin/common/get_dictionary', 'URL', 149, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (540, '查看试卷结构', '/api/admin/exam/structure/preview_structure', 'URL', 488, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (541, '获取用户阅卷角色', '/api/admin/common/get_open_role', 'URL', 491, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (542, '查询接口', '/api/admin/sync/score/list', 'URL', 500, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (543, '成绩导出接口', '/api/admin/sync/score/export', 'URL', 502, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (544, '成绩动态轨迹图下载接口', '/api/admin/sync/score/download', 'URL', 505, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (545, '成绩动态轨迹图一键下载接口', '/api/admin/sync/score/batch_download', 'URL', 503, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (546, '成绩查询同步接口', '/api/admin/sync/score/sync', 'URL', 501, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (547, '阅卷试卷上传待办计数接口', '/api/admin/work/structure/task/ready_count', 'URL', 16, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (548, '阅卷试卷上传待办接口', '/api/admin/work/structure/task/ready', 'URL', 16, 8, 'SYS', '490', 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (549, '短信日志', 'SmsManage', 'MENU', 112, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (550, '查询', 'Select', 'BUTTON', 549, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (551, '查询', '/api/admin/basic/message/list', 'URL', 550, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (552, '查询短信类型', '/api/admin/basic/message/list_message_types', 'URL', 550, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (553, '短信失败重发', '/api/admin/basic/message/resend', 'URL', 550, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (554, '查询条件', 'Condition', 'CONDITION', 549, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (555, '列表', 'List', 'LIST', 549, 1, 'AUTH', '551,552', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (556, '短信失败重发', 'Send', 'LINK', 549, 1, 'AUTH', '553', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (557, '查询', '/api/admin/data/sync/query', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (558, '模糊查询', '/api/admin/basic/semester/list', 'URL', 510, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (559, '启用/禁用', 'Enable', 'LINK', 510, 1, 'AUTH', '560', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (560, '启用/禁用', '/api/admin/basic/semester/enable', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (561, '归档管理', 'ScoreArchivePrimary', 'MENU', 506, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (562, '提交印刷', 'Submit', 'LINK', 47, 5, 'AUTH', '60', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (563, '查看原卷接口', '/api/admin/sync/score/preview', 'URL', 504, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (564, '考试试卷文件上传接口', '/api/admin/common/file/exam_paper_upload', 'URL', 144, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (566, '审核-印刷员驳回后命题老师修改', '/api/admin/exam/task/apply_resave', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (567, '查看', 'Preview', 'LINK', 403, 1, 'AUTH', '168,568,640,641,477', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (568, '查询任务详情', '/api/admin/exam/task/exam_task_get_one', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (569, '根据条件删除（物理）', '/api/admin/basic/student/delete_by_condition', 'URL', 358, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (570, '批量删除', 'Delete', 'BUTTON', 358, 1, 'AUTH', '569', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (571, '批量删除', 'Delete', 'BUTTON', 450, 1, 'AUTH', '453', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (572, '批量删除', 'Delete', 'BUTTON', 515, 1, 'AUTH', '538', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (573, '根据查询条件批量删除', '/api/admin/basic/course/delete_by_query', 'URL', 13, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (574, '批量删除', 'Delete', 'BUTTON', 13, 5, 'AUTH', '573', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (575, '课程管理', 'basicInfo', 'MENU', 650, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (576, '课程管理', 'CourseSimpleManage', 'MENU', 575, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (598, '教学学生-查询', '/api/admin/teach/student/page', 'URL', 597, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (599, '教学学生-创建教学学生', '/api/admin/teach/student/create', 'URL', 597, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (600, '教学学生-批量删除', '/api/admin/teach/student/delete', 'URL', 597, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (601, '教学学生-批量导入', '/api/admin/teach/student/import', 'URL', 597, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (602, '教学学生-列表', 'List', 'LIST', 597, 1, 'AUTH', '598', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (603, '教学学生-查询', 'Select', 'BUTTON', 597, 2, 'AUTH', '598', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (604, '教学学生-新增', 'Add', 'BUTTON', 597, 3, 'AUTH', '599', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (605, '教学学生-批量导入', 'Import', 'BUTTON', 597, 4, 'AUTH', '601', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (606, '教学学生-删除', 'Delete', 'LINK', 597, 5, 'AUTH', '600', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (607, '教学学生-批量删除', 'DeleteBatch', 'BUTTON', 597, 6, 'AUTH', '600', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (609, '考试管理', 'ExamManage', 'MENU', 356, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (610, '查询', 'Select', 'BUTTON', 609, 1, 'AUTH', '616', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (611, '新增', 'Add', 'BUTTON', 609, 3, 'AUTH', '617', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (612, '启用/禁用', 'Enable', 'LINK', 609, 4, 'AUTH', '618', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (613, '删除', 'Delete', 'LINK', 609, 5, 'AUTH', '619', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (614, '设置', 'Set', 'LINK', 609, 6, 'AUTH', '620,621,622,623,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (615, '查询条件', 'Condition', 'CONDITION', 609, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (616, '查询', '/api/admin/basic/exam/page', 'URL', 609, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (617, '新增/修改', '/api/admin/basic/exam/save', 'URL', 609, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (618, '启用/禁用', '/api/admin/basic/exam/enable', 'URL', 609, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (619, '删除', '/api/admin/basic/exam/delete', 'URL', 609, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (620, '设置', '/api/admin/basic/exam/set_in_userd', 'URL', 609, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (621, '查询', '/api/admin/basic/print_config/page', 'URL', 614, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (622, '新增/修改', '/api/admin/basic/print_config/save', 'URL', 614, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (623, '启用/禁用', '/api/admin/basic/print_config/enable', 'URL', 614, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (624, '题卡管理', 'CardManage', 'MENU', 3, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (625, '查询', 'Select', 'BUTTON', 624, 1, 'AUTH', '627', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (626, '查询条件', 'Condition', 'CONDITION', 624, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (627, '查询', '/api/admin/exam/card/page', 'URL', 624, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (628, '编辑', 'Edit', 'LINK', 609, 7, 'AUTH', '617', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (629, '新增', 'Add', 'BUTTON', 624, 3, 'AUTH', '631', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (630, '编辑题卡/编辑信息', 'Edit', 'LINK', 624, 4, 'AUTH', '631', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (631, '新增/修改', '/api/admin/exam/card/save_generic', 'URL', 624, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (632, '列表', 'List', 'LIST', 609, 7, 'AUTH', '616', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (633, '列表', 'List', 'LIST', 624, 5, 'AUTH', '627', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (634, '发送验证码（绑定手机号）', '/api/admin/common/get_verify_code_for_bind', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (635, '删除', 'Delete', 'LINK', 624, 6, 'AUTH', '636', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (636, '删除', '/api/admin/exam/card/delete_generic', 'URL', 624, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (637, '模糊查询', '/api/admin/basic/exam/query', 'URL', 609, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (638, '编辑', 'Edit', 'LINK', 402, 1, 'AUTH', '639', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (639, '编辑流程', '/api/admin/custom/flow/edit', 'URL', 402, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (640, '获取当前流程节点信息', '/api/admin/custom/flow/task/info', 'URL', 402, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (641, '流程信息', '/api/admin/custom/flow/view', 'URL', 402, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (642, '根据考试ID和机构ID查询印品', '/api/admin/basic/print_config/get_by_exam_id_and_org_id', 'URL', 614, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (643, '重命名', 'Rename', 'LINK', 402, 3, 'AUTH', '643', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (644, '重命名自定义流程名称', '/api/admin/custom/flow/rename', 'URL', 402, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (647, '成员管理', 'Manager', 'LINK', 6, 3, 'AUTH', '648,649', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (648, '新增/修改', '/api/admin/sys/role/group/save', 'URL', 647, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (649, '查询', '/api/admin/sys/role/group/list', 'URL', 647, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (650, '课程管理', 'baseInfo', 'MENU', NULL, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (651, '电子交卷', 'electronic', 'MENU', NULL, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (652, '电子交卷', 'electronicSubmit', 'MENU', 651, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (653, '下载管理', 'DownloadManage', 'MENU', 112, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (654, '查询', 'Select', 'BUTTON', 653, 1, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (655, '查询条件', 'Condition', 'CONDITION', 653, 2, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (656, '下载文件', 'Download', 'LINK', 653, 3, 'AUTH', '660', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (657, '批量下载文件', 'Download', 'BUTTON', 653, 4, 'AUTH', '661', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (658, '列表', 'List', 'LIST', 653, 5, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (659, '查询', '/api/admin/data/download/page', 'URL', 653, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (660, '下载文件', '/api/admin/data/download/download_one', 'URL', 653, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (661, '批量下载文件', '/api/admin/data/download/download_batch', 'URL', 653, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (662, '教学学生-从课程班级批量添加学生', '/api/admin/teach/student/create_batch', 'URL', 597, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (663, '教学学生-从课程班级添加', 'SaveBatch', 'BUTTON', 597, 7, 'AUTH', '662', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (664, '重新同步', 'Push', 'LINK', 521, 1, 'AUTH', '665', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (665, '重新同步', '/api/admin/data/sync/resync', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (667, '上传试卷标答文件', '/api/admin/exam/structure/upload_answer', 'URL', 488, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (669, '更新客观题答案', '/api/admin/exam/structure/update_objective_answer', 'URL', 488, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (670, '评卷参数设置', 'Submit', 'LINK', 488, 6, 'AUTH', '490,540,667,669,671,672,754,855,882,883', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (671, '提交', '/api/admin/exam/structure/submit', 'URL', 488, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (672, '获取题卡图片', '/api/admin/exam/structure/find_jpg_file', 'URL', 488, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (673, '查询条件', 'Condition', 'CONDITION', 488, 7, 'AUTH', '489', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (674, '查询', 'Select', 'BUTTON', 488, 8, 'AUTH', '489', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (675, '授权配置', 'AuthManage', 'MENU', 1, 6, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (676, '授权配置-导出硬件信息按钮', 'Export', 'BUTTON', 675, 1, 'AUTH', '677,678,679,794', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (677, '导出硬件信息', '/api/admin/auth/export/device/info', 'URL', 676, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (678, '离线激活', '/api/admin/auth/offline/activation', 'URL', 676, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (679, '查询激活信息', '/api/admin/auth/select', 'URL', 676, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (680, '教研分析', 'analyze', 'MENU', NULL, 8, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (681, '分析批次管理', 'AnalysisBatchManage', 'MENU', 680, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (682, '查询', '/api/admin/grade/batch/page', 'URL', 681, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (683, '新建', '/api/admin/grade/batch/save', 'URL', 681, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (684, '删除', '/api/admin/grade/batch/delete', 'URL', 681, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (685, '推送', '/api/admin/grade/batch/push', 'URL', 681, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (686, '查询', 'Select', 'BUTTON', 681, 1, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (687, '列表', 'List', 'LIST', 681, 2, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (688, '新建', 'Add', 'BUTTON', 681, 3, 'AUTH', '683', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (689, '批量删除', 'Delete', 'BUTTON', 681, 4, 'AUTH', '684', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (690, '删除', 'Delete', 'LINK', 681, 5, 'AUTH', '684', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (691, '推送', 'Push', 'LINK', 681, 6, 'AUTH', '685', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (692, '添加分析课程', 'Window', 'LINK', 681, 7, 'AUTH', '693,694,695,696,704,705', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (693, '查询', '/api/admin/grade/batch/paper/page', 'URL', 692, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (694, '新建', '/api/admin/grade/batch/paper/save', 'URL', 692, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (695, '删除', '/api/admin/grade/batch/paper/delete', 'URL', 692, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (696, '查询可分析试卷', '/api/admin/grade/batch/paper/able', 'URL', 692, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (697, '课程查询', 'Select', 'BUTTON', 692, 1, 'AUTH', '693', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (698, '课程列表', 'List', 'LIST', 692, 2, 'AUTH', '693', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (699, '课程添加分析课程', 'Add', 'BUTTON', 692, 3, 'AUTH', '694', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (700, '课程批量删除', 'Delete', 'BUTTON', 692, 4, 'AUTH', '695', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (701, '课程删除', 'Delete', 'BUTTON', 692, 5, 'AUTH', '695', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (702, '可分析课程查询', 'Select', 'BUTTON', 692, 6, 'AUTH', '696', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (703, '可分析课程列表', 'List', 'LIST', 692, 7, 'AUTH', '696', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (704, '推送', '/api/admin/grade/batch/paper/push', 'URL', 692, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (705, '计算', '/api/admin/grade/batch/paper/calculate', 'URL', 692, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (706, '推送', 'Push', 'LINK', 692, 8, 'AUTH', '704', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (707, '计算', 'Calculate', 'LINK', 692, 9, 'AUTH', '705', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (708, '蓝图设置', 'DataInitManage', 'MENU', 680, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (709, '成绩分析课程数据初始化', '/api/admin/grade/paper/initialize', 'URL', 708, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (710, '查询', 'Select', 'BUTTON', 708, 1, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (711, '列表', 'List', 'LIST', 708, 2, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (712, '成绩分析基础配置', 'Window', 'LINK', 708, 3, 'AUTH', '717,718,719,720,726,727,728,729,735,736,737,738,755', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (717, '查询', '/api/admin/grade/paper/dimension/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (718, '新建维度', '/api/admin/grade/paper/dimension/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (719, '模板导出', '/api/admin/grade/paper/dimension/export', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (720, '导入', '/api/admin/grade/paper/dimension/import', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (721, '查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '717', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (722, '列表', 'List', 'LIST', 712, 2, 'AUTH', '717', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (723, '新建', 'Add', 'BUTTON', 712, 3, 'AUTH', '718', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (724, '导入', 'Import', 'BUTTON', 712, 4, 'AUTH', '720', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (725, '模板导出', 'Export', 'BUTTON', 712, 5, 'AUTH', '719', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (726, '查询', '/api/admin/grade/paper/struct/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (727, '新建试卷结构', '/api/admin/grade/paper/struct/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (728, '导入', '/api/admin/grade/paper/struct/import', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (729, '模板导出', '/api/admin/grade/paper/struct/export', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (730, '查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '726', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (731, '列表', 'List', 'LIST', 712, 2, 'AUTH', '726', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (732, '导入', 'Import', 'BUTTON', 712, 3, 'AUTH', '728', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (734, '模板导出', 'Export', 'BUTTON', 712, 4, 'AUTH', '729', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (735, '查询定义', '/api/admin/grade/paper/paper_define', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (736, '新建定义', '/api/admin/grade/paper/save_define', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (737, '查询', '/api/admin/grade/module/evaluation/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (738, '新建模块评价', '/api/admin/grade/module/evaluation/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (739, '查看报告', 'AnalysisReportView', 'MENU', 680, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (740, '教研分析单点登录', '/api/admin/exam/sso/analysis_login', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (741, '考务数据导出', 'Export', 'LINK', 681, 8, 'AUTH', '743', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (742, '考务数据导入', 'Import', 'LINK', 681, 9, 'AUTH', '744', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (743, '考务数据导出', '/api/admin/grade/batch/download', 'URL', 681, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (744, '考务数据导入', '/api/admin/grade/batch/upload', 'URL', 681, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (745, '教研待办接口', '/api/admin/work/grade/task/ready', 'URL', 16, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (746, '教研待办计数接口', '/api/admin/work/grade/task/ready_count', 'URL', 16, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (747, '开始计算', 'StartCalc', 'LINK', 681, 12, 'AUTH', '748', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (748, '开始计算', '/api/admin/grade/batch/start_calc', 'URL', 681, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (749, '查询条件', 'Condition', 'CONDITION', 708, 4, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (750, '查询条件', 'Condition', 'CONDITION', 681, 14, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (751, '课程发布', 'Publish', 'LINK', 681, 6, 'AUTH', '752', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (752, '课程发布', '/api/admin/grade/batch/paper/publish', 'URL', 681, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (754, '绑定科组长', '/api/admin/exam/structure/bind_mark_leader', 'URL', 488, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (755, '更新试卷机构', '/api/admin/grade/paper/struct/change_paper_structure', 'URL', 712, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (756, '查询已审核数量', '/api/admin/exam/task/review_list_audited_count', 'URL', 41, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (757, '批量导入', '/api/admin/sys/org/import', 'URL', 4, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (758, '导入', 'Import', 'BUTTON', 4, 4, 'AUTH', '757', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (759, '考生对象查询', '/api/admin/exam/task/find_exam_task_student_object', 'URL', 40, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (760, '增加考试对象', 'ExamTaskStudentObject', 'BUTTON', 40, 9, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (761, '导入考生对象', '/api/admin/exam/task/exam_task_exam_student_import', 'URL', 40, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (762, '批量导入考生', 'ExamTaskStudentObjectImport', 'BUTTON', 40, 10, 'AUTH', '761', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (763, '查询详情', '/api/admin/basic/template/get_one', 'URL', 11, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (764, '复制', '/api/admin/exam/card/copy', 'URL', 175, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (765, '查询开课学院', '/api/admin/grade/batch/paper/find_teach_college', 'URL', 692, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (766, '查询条件', 'Condition', 'CONDITION', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (767, '同步机构数据', '/api/admin/auth/sync/org', 'URL', 676, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (768, '题库单点登录', '/api/admin/exam/sso/question_library_login', 'URL', 149, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (769, '题库管理', 'questionLibrary', 'MENU', NULL, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (770, '题库管理', 'QuestionLibraryManage', 'MENU', 769, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (771, '用户验证查询', '/api/admin/set/user/select', 'URL', 508, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (772, '用户验证保存', '/api/admin/set/user/save', 'URL', 508, 5, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (773, '试卷规格配置保存', '/api/admin/set/paper/save', 'URL', 508, 6, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (774, '试卷规格配置查询', '/api/admin/set/paper/select', 'URL', 508, 7, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (775, '系统试卷规格配置查询', '/api/admin/set/paper/sys/select', 'URL', 508, 8, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (776, '同步配置保存', '/api/admin/set/sync/save', 'URL', 508, 9, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (777, '同步配置查询', '/api/admin/set/sync/select', 'URL', 508, 10, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (778, '数据还原', '/api/admin/set/backup', 'URL', 508, 11, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (779, '查询自定义角色权限', '/api/admin/set/role/custom/list', 'URL', 508, 12, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (780, '学校新增/修改自定义角色权限', '/api/admin/set/role/custom/save', 'URL', 508, 13, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (781, '学校已绑定自定义角色权限列表', '/api/admin/set/role/custom/get_school_custom_roles', 'URL', 508, 14, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (782, '操作日志', 'ActionLogManage', 'MENU', 112, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (783, '查询', '/api/admin/basic/operation_log/query', 'URL', 782, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (784, '查询条件', 'Condition', 'CONDITION', 782, 1, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (785, '查询', 'Select', 'BUTTON', 782, 2, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (788, '删除', 'Delete', 'LINK', 11, 4, 'AUTH', '789', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (789, '删除', '/api/admin/basic/template/remove', 'URL', 11, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (790, '查询异步结果', '/api/admin/data/task/query_result', 'URL', 113, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (792, '云阅卷考试查询', '/api/admin/basic/stmms_exam/list', 'URL', 392, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (793, '云阅卷考试同步', '/api/admin/basic/stmms_exam/sync', 'URL', 392, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (794, '在线激活', '/api/admin/auth/online/activation', 'URL', 676, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (795, '学校管理', 'SchoolManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (796, '查询', 'Select', 'BUTTON', 795, 1, 'AUTH', '798', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (797, '查询条件', 'Condition', 'CONDITION', 795, 2, 'AUTH', '798', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (798, '查询', '/api/admin/commom/school/list', 'URL', 795, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (799, '机构同步', 'Sync', 'BUTTON', 795, 3, 'AUTH', '800', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (800, '机构同步', '/api/admin/auth/sync/org', 'URL', 795, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (801, '编辑', 'Edit', 'LINK', 795, 1, 'AUTH', '802', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (802, '编辑', '/api/admin/commom/school/update', 'URL', 795, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (803, '列表', 'List', 'LIST', 782, 3, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (804, '删除', 'Delete', 'LINK', 39, 5, 'AUTH', '805', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (805, '删除', '/api/admin/exam/task/delete', 'URL', 39, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (806, '删除', 'Delete', 'LINK', 40, 6, 'AUTH', '807', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (807, '删除', '/api/admin/exam/task/apply_delete', 'URL', 40, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (808, '删除', 'Delete', 'LINK', 44, 2, 'AUTH', '809', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (809, '删除', '/api/admin/exam/print/data_delete', 'URL', 44, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (810, '导出', 'Export', 'BUTTON', 4, 5, 'AUTH', '811', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (811, '导出', '/api/admin/sys/org/export', 'URL', 4, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (812, '导出', 'Export', 'BUTTON', 5, 4, 'AUTH', '813', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (813, '导出', '/api/admin/sys/user/export', 'URL', 5, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (814, '导出', 'Export', 'BUTTON', 358, 3, 'AUTH', '815', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (815, '导出', '/api/admin/basic/student/export', 'URL', 358, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (816, '导出', 'Export', 'BUTTON', 13, 6, 'AUTH', '817', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (817, '导出', '/api/admin/basic/course/export', 'URL', 13, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (818, '成绩归档', 'ScoreSync', 'MENU', 561, 1, NULL, '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (819, '列表', 'List', 'LIST', 818, 1, 'AUTH', '823', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (820, '查询条件', 'Condition', 'CONDITION', 818, 2, 'AUTH', '823', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (821, '查询', 'Select', 'BUTTON', 818, 3, 'AUTH', '823', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (822, '同步', 'Sync', 'LINK', 818, 4, 'AUTH', '824', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (823, '查询接口', '/api/admin/sync/log/list', 'URL', 818, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (824, '同步接口', '/api/admin/sync/log/sync', 'URL', 818, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (825, '命题任务公共卷型查询接口', '/api/admin/exam/print/list_relate_common_types', 'URL', 43, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (826, '批量关联接口', '/api/admin/exam/print/relate_update_batch', 'URL', 43, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (827, '超管-角色管理查询', '/api/admin/sys/role/list_to_admin', 'URL', 5, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (828, '生成图片/预览图片', 'ConvertImage', 'LINK', 624, 7, 'AUTH', '829', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (829, '生成图片', '/api/admin/exam/card/convert_image', 'URL', 624, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (830, '教学课程-分页查询', '/api/admin/teach/course/page', 'URL', 576, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (831, '教学课程-手动添加', '/api/admin/teach/course/manual_create', 'URL', 576, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (832, '教学课程-课程表选择并添加', '/api/admin/teach/course/select_create', 'URL', 576, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (833, '教学课程-批量导入', '/api/admin/teach/course/import', 'URL', 576, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (834, '教学课程-批量删除', '/api/admin/teach/course/delete', 'URL', 576, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (835, '教学课程-列表', 'List', 'LIST', 576, 1, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (836, '查询', 'Select', 'BUTTON', 576, 2, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (837, '删除', 'Delete', 'LINK', 576, 3, 'AUTH', '834', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (838, '查询条件', 'Condition', 'CONDITION', 576, 4, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (839, '基础课程-根据学院查询课程', '/api/admin/basic/course/list_by_org', 'URL', 13, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (840, '新增', 'AddCourse', 'BUTTON', 576, 5, 'AUTH', '831,832,833,839', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (841, '管理', 'Link', 'LINK', 576, 6, 'AUTH', '77,842,843,844,845,846,847,848,849,850,851,852,853,866', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (842, '教学班-分页查询', '/api/admin/teach/clazz/page', 'URL', 841, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (843, '教学班-编辑', '/api/admin/teach/clazz/edit', 'URL', 841, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (844, '教学班-删除', '/api/admin/teach/clazz/delete', 'URL', 841, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (845, '教师团队-分页查询', '/api/admin/teach/course/teacher_team_page', 'URL', 841, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (846, '教师团队-手动添加', '/api/admin/teach/course/manual_create_teacher', 'URL', 841, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (847, '教师团队-教师库添加', '/api/admin/teach/course/select_create_teacher', 'URL', 841, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (848, '教师团队-批量导入教师', '/api/admin/teach/course/import_teacher', 'URL', 841, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (849, '教学学生-创建教学学生', '/api/admin/teach/student/create', 'URL', 841, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (850, '教学学生-查询', '/api/admin/teach/student/page', 'URL', 841, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (851, '教学学生-从课程班级批量添加学生', '/api/admin/teach/student/create_batch', 'URL', 841, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (852, '教学学生-批量导入', '/api/admin/teach/student/import', 'URL', 841, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (853, '教学学生-批量删除', '/api/admin/teach/student/delete', 'URL', 841, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (855, '绑定评卷员', '/api/admin/exam/structure/bind_marker', 'URL', 488, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (856, '查看', 'Preview', 'LINK', 624, 8, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (857, '转办/打回', 'Transfer', 'LINK', 488, 10, 'AUTH', '858,859', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (858, '查询教学秘书', '/api/admin/exam/structure/list_secretary', 'URL', 488, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (859, '转办/打回', '/api/admin/exam/structure/transfer', 'URL', 488, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (860, '共用接口-查询学期', '/api/admin/basic/condition/list_semester', 'URL', 149, 8, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (861, '共用接口-查询考试', '/api/admin/basic/condition/list_exam', 'URL', 149, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (862, '共用接口-查询计划', '/api/admin/basic/condition/list_print_plan', 'URL', 149, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (863, '共用接口-查询课程', '/api/admin/basic/condition/list_course', 'URL', 149, 11, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (864, '共用接口-查询试卷编号', '/api/admin/basic/condition/list_paper_number', 'URL', 149, 12, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (865, '共用接口-查询班级', '/api/admin/basic/condition/list_clazz', 'URL', 149, 13, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (866, '教师团队-根据机构查询', '/api/admin/teach/course/find_teacher_by_org', 'URL', 841, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (867, '下载', 'Download', 'LINK', 624, 9, 'AUTH', '868', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (868, '下载', '/api/admin/exam/card/download_card', 'URL', 624, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (869, '试卷题卡预览日志记录接口', '/api/admin/basic/operation_log/save_preview_log', 'URL', 149, 14, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (872, '批量撤回', 'BatchEnd', 'BUTTON', 47, 3, 'AUTH', '873', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (873, '批量撤回', '/api/admin/exam/print/task_batch_cancel', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (874, '共用接口-查询学院', '/api/admin/basic/condition/list_college', 'URL', 149, 14, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (875, '我教课程选择', 'MyTeachCourse', 'BUTTON', 40, 10, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (876, '学生库选择', 'StudentLibrary', 'BUTTON', 40, 11, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (877, '选择学生（学生库选择）', 'SelectStudent', 'BUTTON', 40, 12, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (878, '作废', 'Cancel', 'LINK', 42, 5, 'AUTH', '879', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (879, '作废', '/api/admin/exam/task/cancel', 'URL', 42, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (880, '重启流程', '/api/admin/exam/task/restart', 'URL', 42, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (881, '打回', 'Restart', 'LINK', 42, 6, 'AUTH', '880', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (882, '查询分班阅班级', '/api/admin/exam/structure/list_user_class', 'URL', 488, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (883, '查询评卷参数状态', '/api/admin/exam/structure/get_structure_status', 'URL', 488, 14, 'AUTH', NULL, 1, 1, 1);

INSERT INTO `sys_user` VALUES (1, NULL, 'sysadmin', 'sysadmin', NULL, 'MTIzNDU2', NULL, NULL, 1, 1, 1629353956868, NULL, NULL, NULL, NULL, 1629353956881, NULL);

INSERT INTO `sys_user_role` VALUES (1, 1, 1, NULL, 1);

INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, NULL, NULL, 'sys.user.initPassword', '初始密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (2, NULL, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (3, NULL, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (4, NULL, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (5, NULL, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (6, NULL, NULL, 'sys.sync.enable', '是否开启数据同步云阅卷', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (8, NULL, NULL, 'sys.pdf.size.list', '全局pdf格式清单', '[A3, A4, A5, B3, B4, B5, 8K]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (9, NULL, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (10, NULL, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (11, NULL, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (12, NULL, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (13, NULL, NULL, 'auto.create.pdf.reset.max.count', '自动重试生成pdf失败最大次数', '5', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (14, NULL, NULL, 'thread.pool.core.size', '自定义线程池大小', '500', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (15, NULL, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (16, NULL, NULL, 'session.active', 'session会话时长', '4h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (18, NULL, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (19, NULL, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (20, NULL, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (21, NULL, NULL, 'teachcloud.print.open.callback.pwd', '分布式印刷回调密码', 'qmth87863577-online', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (24, NULL, NULL, 'sms.sign.name', '短信签名', '知学知考', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (25, NULL, NULL, 'sms.tpl.code', '登录短信验证码', 'SMS_147416565', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (26, NULL, NULL, 'sms.audit.pass.code', '命题任务审核通过', 'SMS_216425141', NULL, 1, 3, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (27, NULL, NULL, 'sms.audit.not.pass.code', '命题任务审核不通过', 'SMS_216275156', NULL, 1, 4, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (28, NULL, NULL, 'sms.exam.task.create.code', '命题任务待办生成通知', 'SMS_217436292', NULL, 1, 5, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (29, NULL, NULL, 'sms.exam.task.will.expire.code', '命题任务待办到期预警通知', 'SMS_217436295', NULL, 1, 6, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (30, NULL, NULL, 'sms.exam.task.overdue.code', '命题任务待办逾期通知', 'SMS_217426313', NULL, 1, 7, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (31, NULL, NULL, 'sms.allocation.will.expire.code', '命题分配待办到期预警通知', 'SMS_217406305', NULL, 1, 8, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (32, NULL, NULL, 'sms.allocation.overdue.code', '命题分配待办逾期通知', 'SMS_217406308', NULL, 1, 9, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (33, NULL, NULL, 'sms.audit.create.code', '审核待办生成通知', 'SMS_237216000', NULL, 1, 10, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (34, NULL, NULL, 'sms.audit.review.code', '审核待办修改申请通知', 'SMS_237216002', NULL, 1, 11, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (35, NULL, NULL, 'sms.audit.will.expire.code', '审核待办到期预警通知', 'SMS_217436302', NULL, 1, 12, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (36, NULL, NULL, 'sms.audit.overdue.code', '审核待办逾期通知', 'SMS_217416271', NULL, 1, 13, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (37, NULL, NULL, 'sms.audit.reject.code', '审核驳回通知', 'SMS_237206063', NULL, 1, 14, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (38, NULL, NULL, 'sms.upload.structure.code', '试卷结构推送通知', 'SMS_237201068', NULL, 1, 15, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (39, NULL, NULL, 'sms.audit.copy.user.code', '用户', 'SMS_237206065', NULL, 1, 16, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (40, NULL, NULL, 'sys.txt.charset', 'txt文件编码', 'UTF-8', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (401752651068346368, 135, NULL, 'open.flow.message.push', '是否开启消息推送', 'true', NULL, 1, 5, 1, 1686726096298, NULL, 1686726096298);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (401752651068346369, 135, NULL, 'flow.message.type', '消息类型', 'OFF_STANDARD', NULL, 1, 6, 1, 1686726096303, NULL, 1686726096303);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (401752651068346370, 135, NULL, 'mq.host.url', '消息中心地址', 'http://exam.gdpu.edu.cn:7400?code=gdpu', NULL, 1, 7, 1, 1686726096309, NULL, 1686726096309);

INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (1, NULL, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, NULL, NULL);
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (2, NULL, NULL, '教务处老师', 1, 'OFFICE_TEACHER', NULL, NULL, 1, 1691113883889, 1, 'ANALYSIS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (3, NULL, NULL, '学院院长', 1, 'PRESIDENT', NULL, NULL, 1, 1691113890328, 1, 'ANALYSIS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (4, NULL, NULL, '任课老师', 1, 'TEACHER', NULL, NULL, 1, 1691113896037, 1, 'ANALYSIS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (5, NULL, NULL, '评卷员', 1, 'MARKER', NULL, NULL, 1, 1691113902940, 1, 'CLOUD_MARK', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (6, NULL, NULL, '科组长', 1, 'MARKER_LEADER', NULL, NULL, 1, 1691113910873, 1, 'CLOUD_MARK', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (7, NULL, NULL, '学科老师', 1, 'SUBJECT_TEACHER', NULL, NULL, 1, 1691113918169, 1, 'QUESTION_LIBRARY', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (8, NULL, NULL, '命题老师', 1, 'ASSIGN_TEACHER', NULL, NULL, 1, 1691114135409, 1, 'QUESTION_LIBRARY', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (9, NULL, NULL, '学校管理员', 1, 'SCHOOL_ADMIN', NULL, NULL, 1, 1691114168057, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (10, NULL, NULL, '老师', 1, 'SCHOOL_TEACHER', NULL, NULL, 1, 1691114224321, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (11, NULL, NULL, '学院秘书', 1, 'ASSISTANT', NULL, NULL, 1, 1691114261182, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (12, NULL, NULL, '印刷员', 1, 'PRINT', NULL, NULL, 1, 1691114261182, 1, 'SYS', '系统内置');

INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1, 1, 1, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (2, 1, 2, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (3, 1, 4, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (4, 1, 5, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (5, 1, 6, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (6, 1, 92, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (7, 1, 93, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (8, 1, 94, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (9, 1, 95, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (10, 1, 224, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (11, 1, 225, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (12, 1, 226, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (13, 1, 227, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (14, 1, 228, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (15, 1, 234, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (16, 1, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (17, 1, 78, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (18, 1, 79, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (19, 1, 80, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (20, 1, 81, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (21, 1, 82, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (22, 1, 87, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (23, 1, 184, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (24, 1, 217, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (25, 1, 218, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (26, 1, 219, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (27, 1, 220, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (28, 1, 221, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (29, 1, 222, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (30, 1, 223, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (31, 1, 243, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (32, 1, 83, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (33, 1, 84, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (34, 1, 85, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (35, 1, 86, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (36, 1, 91, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (37, 1, 180, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (38, 1, 229, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (39, 1, 230, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (40, 1, 231, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (41, 1, 232, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (42, 1, 233, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (43, 1, 235, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (44, 1, 390, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (45, 1, 402, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (46, 1, 407, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (47, 1, 408, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (48, 1, 409, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (49, 1, 410, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (50, 1, 411, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (53, 1, 527, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (54, 1, 528, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (55, 1, 529, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (56, 1, 638, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (57, 1, 643, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (58, 1, 675, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (59, 1, 676, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (60, 1, 677, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (61, 1, 678, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (62, 1, 679, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (63, 2, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (64, 2, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (65, 3, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (66, 3, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (67, 4, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (68, 4, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (69, 5, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (70, 5, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (71, 5, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (72, 6, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (73, 6, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (74, 6, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (75, 5, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (76, 5, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (77, 5, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (78, 6, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (79, 6, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (80, 6, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (81, 1, 767, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (88, 1, 771, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (89, 1, 772, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (90, 1, 773, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (91, 1, 774, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (92, 1, 775, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (93, 1, 776, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (94, 1, 777, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (95, 1, 778, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (96, 1, 779, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (97, 1, 780, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (98, 1, 781, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687280181941567488, 7, 768, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687280181941567489, 7, 769, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687280181941567490, 7, 770, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281093103779841, 8, 768, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281093103779842, 8, 769, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281093107974145, 8, 770, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942786, 9, 1, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942787, 9, 2, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942788, 9, 3, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942789, 9, 4, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942790, 9, 5, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942791, 9, 6, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942792, 9, 8, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942793, 9, 9, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942794, 9, 11, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942795, 9, 13, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942796, 9, 14, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942797, 9, 38, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942798, 9, 39, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942799, 9, 40, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942800, 9, 41, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942801, 9, 42, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942802, 9, 43, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942803, 9, 44, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942804, 9, 45, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942805, 9, 47, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942806, 9, 48, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942807, 9, 49, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942808, 9, 50, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942809, 9, 51, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942810, 9, 52, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942811, 9, 53, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942812, 9, 54, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942813, 9, 55, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942814, 9, 56, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942815, 9, 57, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942816, 9, 58, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942817, 9, 59, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942818, 9, 60, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942819, 9, 61, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942820, 9, 62, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230093942821, 9, 63, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137090, 9, 64, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137091, 9, 65, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137092, 9, 66, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137093, 9, 67, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137094, 9, 68, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137095, 9, 70, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137096, 9, 71, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137097, 9, 72, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137098, 9, 73, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137099, 9, 74, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137100, 9, 76, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137101, 9, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137102, 9, 78, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137103, 9, 79, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137104, 9, 80, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137105, 9, 81, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137106, 9, 83, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137107, 9, 84, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137108, 9, 85, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137109, 9, 87, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137110, 9, 88, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137111, 9, 89, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137112, 9, 90, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137113, 9, 91, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137114, 9, 92, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137115, 9, 93, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137116, 9, 94, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137117, 9, 96, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137118, 9, 97, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137119, 9, 99, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137120, 9, 100, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137121, 9, 101, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137122, 9, 102, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137123, 9, 103, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137124, 9, 104, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137125, 9, 105, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137126, 9, 112, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137127, 9, 113, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137128, 9, 114, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137129, 9, 123, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137130, 9, 124, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137131, 9, 125, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137132, 9, 157, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137133, 9, 158, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137134, 9, 159, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137135, 9, 161, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137136, 9, 162, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137137, 9, 163, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137138, 9, 164, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137139, 9, 165, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137140, 9, 166, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137141, 9, 167, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137142, 9, 168, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137143, 9, 169, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137144, 9, 174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137145, 9, 184, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137146, 9, 188, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137147, 9, 190, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137148, 9, 191, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137149, 9, 192, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137150, 9, 217, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137151, 9, 218, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137152, 9, 219, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137153, 9, 220, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137154, 9, 221, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230098137155, 9, 222, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331393, 9, 223, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331394, 9, 224, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331395, 9, 225, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331396, 9, 226, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331397, 9, 227, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331398, 9, 228, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331399, 9, 229, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331400, 9, 230, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331401, 9, 231, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331402, 9, 232, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331403, 9, 233, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331404, 9, 235, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331405, 9, 236, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331406, 9, 237, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331407, 9, 238, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331408, 9, 239, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331409, 9, 240, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331410, 9, 243, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331411, 9, 249, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331412, 9, 250, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331413, 9, 251, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331414, 9, 252, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331415, 9, 253, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331416, 9, 254, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331417, 9, 255, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331418, 9, 263, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331419, 9, 268, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331420, 9, 269, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331421, 9, 270, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331422, 9, 271, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331423, 9, 272, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331424, 9, 273, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331425, 9, 274, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331426, 9, 275, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331427, 9, 277, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331428, 9, 278, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331429, 9, 279, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331430, 9, 280, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331431, 9, 281, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331432, 9, 282, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331433, 9, 283, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331434, 9, 284, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331435, 9, 285, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331436, 9, 288, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331437, 9, 289, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331438, 9, 292, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331439, 9, 293, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331440, 9, 294, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331441, 9, 295, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331442, 9, 296, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331443, 9, 297, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331444, 9, 298, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331445, 9, 299, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331446, 9, 300, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331447, 9, 301, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331448, 9, 302, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331449, 9, 303, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331450, 9, 304, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331451, 9, 305, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331452, 9, 309, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331453, 9, 310, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331454, 9, 311, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331455, 9, 312, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331456, 9, 313, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331457, 9, 314, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331458, 9, 315, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331459, 9, 316, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331460, 9, 317, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230102331461, 9, 318, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525698, 9, 319, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525699, 9, 320, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525700, 9, 321, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525701, 9, 322, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525702, 9, 323, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525703, 9, 324, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525704, 9, 325, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525705, 9, 326, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525706, 9, 330, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525707, 9, 331, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525708, 9, 332, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525709, 9, 333, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525710, 9, 334, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525711, 9, 335, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525712, 9, 336, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525713, 9, 337, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525714, 9, 338, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525715, 9, 339, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525716, 9, 340, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525717, 9, 341, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525718, 9, 342, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525719, 9, 343, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525720, 9, 345, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525721, 9, 356, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525722, 9, 358, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525723, 9, 359, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525724, 9, 362, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525725, 9, 363, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525726, 9, 364, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525727, 9, 365, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525728, 9, 367, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525729, 9, 368, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525730, 9, 369, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525731, 9, 371, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525732, 9, 377, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525733, 9, 378, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525734, 9, 379, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525735, 9, 380, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525736, 9, 381, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525737, 9, 382, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525738, 9, 383, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525739, 9, 384, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525740, 9, 385, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525741, 9, 388, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525742, 9, 390, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525743, 9, 392, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525744, 9, 393, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525745, 9, 394, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525746, 9, 395, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525747, 9, 396, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525748, 9, 397, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525749, 9, 398, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525750, 9, 399, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525751, 9, 400, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525752, 9, 402, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525753, 9, 403, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525754, 9, 404, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525755, 9, 405, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525756, 9, 406, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525757, 9, 407, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525758, 9, 408, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525759, 9, 409, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525760, 9, 410, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525761, 9, 411, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525762, 9, 413, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525763, 9, 414, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525764, 9, 415, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525765, 9, 416, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525766, 9, 417, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525767, 9, 418, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525768, 9, 419, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525769, 9, 420, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525770, 9, 421, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525771, 9, 422, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525772, 9, 424, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525773, 9, 427, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525774, 9, 431, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525775, 9, 435, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525776, 9, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525777, 9, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525778, 9, 445, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525779, 9, 446, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525780, 9, 447, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525781, 9, 448, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525782, 9, 450, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525783, 9, 451, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525784, 9, 452, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525785, 9, 453, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525786, 9, 455, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525787, 9, 456, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525788, 9, 457, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525789, 9, 458, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525790, 9, 459, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525791, 9, 461, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525792, 9, 462, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525793, 9, 463, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525794, 9, 464, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525795, 9, 465, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525796, 9, 466, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525797, 9, 467, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525798, 9, 468, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525799, 9, 469, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525800, 9, 470, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525801, 9, 471, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525802, 9, 472, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525803, 9, 473, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525804, 9, 474, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525805, 9, 475, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525806, 9, 476, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525807, 9, 477, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525808, 9, 478, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525809, 9, 479, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525810, 9, 480, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525811, 9, 481, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525812, 9, 482, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525813, 9, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525814, 9, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525815, 9, 487, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525816, 9, 488, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525817, 9, 489, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525818, 9, 490, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525819, 9, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525820, 9, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525821, 9, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525822, 9, 494, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525823, 9, 495, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525824, 9, 498, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525825, 9, 499, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525826, 9, 500, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525827, 9, 502, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525828, 9, 503, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525829, 9, 504, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525830, 9, 505, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525831, 9, 506, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525832, 9, 510, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525833, 9, 511, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525834, 9, 513, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525835, 9, 514, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525836, 9, 515, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525837, 9, 516, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525838, 9, 517, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525839, 9, 518, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525840, 9, 519, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525841, 9, 520, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525842, 9, 521, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525843, 9, 522, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525844, 9, 523, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525845, 9, 524, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525846, 9, 525, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525847, 9, 526, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525848, 9, 533, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525849, 9, 534, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525850, 9, 535, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525851, 9, 536, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525852, 9, 537, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525853, 9, 538, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525854, 9, 540, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525855, 9, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525856, 9, 542, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525857, 9, 543, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525858, 9, 544, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525859, 9, 545, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525860, 9, 549, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525861, 9, 550, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525862, 9, 551, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525863, 9, 552, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525864, 9, 553, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525865, 9, 554, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525866, 9, 555, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525867, 9, 556, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525868, 9, 557, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525869, 9, 559, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525870, 9, 560, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525871, 9, 561, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525872, 9, 562, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525873, 9, 563, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525874, 9, 567, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525875, 9, 568, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525876, 9, 569, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525877, 9, 570, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525878, 9, 571, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525879, 9, 572, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525880, 9, 573, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525881, 9, 574, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525882, 9, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525883, 9, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525884, 9, 609, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525885, 9, 610, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525886, 9, 611, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525887, 9, 612, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525888, 9, 613, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525889, 9, 614, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525890, 9, 615, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525891, 9, 616, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525892, 9, 617, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525893, 9, 618, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525894, 9, 619, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525895, 9, 620, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525896, 9, 621, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525897, 9, 622, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525898, 9, 623, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525899, 9, 624, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525900, 9, 625, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525901, 9, 626, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525902, 9, 627, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525903, 9, 628, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525904, 9, 629, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525905, 9, 630, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525906, 9, 631, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525907, 9, 632, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525908, 9, 633, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525909, 9, 635, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525910, 9, 636, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525911, 9, 638, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525912, 9, 639, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525913, 9, 640, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525914, 9, 641, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525915, 9, 642, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525916, 9, 643, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525917, 9, 647, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525918, 9, 648, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525919, 9, 649, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525920, 9, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525921, 9, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525922, 9, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525923, 9, 653, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525924, 9, 654, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525925, 9, 655, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525926, 9, 656, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525927, 9, 657, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525928, 9, 658, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525929, 9, 659, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525930, 9, 660, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525931, 9, 661, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525932, 9, 664, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525933, 9, 665, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525934, 9, 667, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525935, 9, 669, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525936, 9, 670, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525937, 9, 671, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525938, 9, 672, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525939, 9, 673, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525940, 9, 674, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525941, 9, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525942, 9, 681, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525943, 9, 682, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525944, 9, 683, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525945, 9, 684, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525946, 9, 685, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525947, 9, 686, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525948, 9, 687, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525949, 9, 688, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525950, 9, 689, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525951, 9, 690, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525952, 9, 691, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525953, 9, 692, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525954, 9, 693, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525955, 9, 694, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230106525956, 9, 695, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691521, 9, 696, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691522, 9, 704, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691523, 9, 705, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691524, 9, 708, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691525, 9, 709, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691526, 9, 710, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691527, 9, 711, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691528, 9, 712, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691529, 9, 717, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691530, 9, 718, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691531, 9, 719, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691532, 9, 720, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691533, 9, 726, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691534, 9, 727, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691535, 9, 728, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691536, 9, 729, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691537, 9, 735, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691538, 9, 736, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691539, 9, 737, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691540, 9, 738, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691541, 9, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691542, 9, 741, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691543, 9, 742, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691544, 9, 743, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691545, 9, 744, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691546, 9, 747, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691547, 9, 748, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691548, 9, 749, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691549, 9, 750, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691550, 9, 751, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691551, 9, 752, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691552, 9, 754, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691553, 9, 755, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691554, 9, 757, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691555, 9, 758, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691556, 9, 759, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691557, 9, 760, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691558, 9, 761, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691559, 9, 762, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691560, 9, 763, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691561, 9, 766, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691562, 9, 769, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691563, 9, 770, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691564, 9, 782, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691565, 9, 783, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691566, 9, 784, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691567, 9, 785, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691568, 9, 788, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691569, 9, 789, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691570, 9, 792, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691571, 9, 793, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691572, 9, 803, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691573, 9, 804, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691574, 9, 805, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691575, 9, 806, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691576, 9, 807, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691577, 9, 808, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691578, 9, 809, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691579, 9, 810, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691580, 9, 811, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691581, 9, 812, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691582, 9, 813, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691583, 9, 814, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691584, 9, 815, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691585, 9, 816, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691586, 9, 817, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691587, 9, 818, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691588, 9, 819, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691589, 9, 820, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691590, 9, 821, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691591, 9, 822, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691592, 9, 823, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691593, 9, 824, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691594, 9, 825, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691595, 9, 826, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691596, 9, 828, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691597, 9, 829, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691598, 9, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691599, 9, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691600, 9, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691601, 9, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691602, 9, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691603, 9, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691604, 9, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230131691605, 9, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885825, 9, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885826, 9, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885827, 9, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885828, 9, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885829, 9, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885830, 9, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885831, 9, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885832, 9, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885833, 9, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885834, 9, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885835, 9, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885836, 9, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885837, 9, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885838, 9, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230135885839, 9, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274434, 9, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274435, 9, 855, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274436, 9, 856, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274437, 9, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274438, 9, 867, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274439, 9, 868, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274440, 9, 872, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274441, 9, 873, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274442, 9, 880, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274443, 9, 881, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274444, 9, 882, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281230144274445, 9, 883, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466031931393, 10, 280, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125698, 10, 281, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125699, 10, 282, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125700, 10, 283, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125701, 10, 284, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125702, 10, 540, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125703, 10, 285, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125704, 10, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125705, 10, 542, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125706, 10, 543, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125707, 10, 544, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125708, 10, 545, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125709, 10, 806, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125710, 10, 807, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125711, 10, 40, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125712, 10, 42, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125713, 10, 299, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125714, 10, 300, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125715, 10, 301, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125716, 10, 302, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125717, 10, 303, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125718, 10, 304, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125719, 10, 305, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125720, 10, 561, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125721, 10, 49, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125722, 10, 563, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125723, 10, 568, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125724, 10, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125725, 10, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125726, 10, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125727, 10, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125728, 10, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125729, 10, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125730, 10, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125731, 10, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125732, 10, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125733, 10, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125734, 10, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125735, 10, 70, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125736, 10, 71, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125737, 10, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125738, 10, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125739, 10, 72, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125740, 10, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125741, 10, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125742, 10, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125743, 10, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125744, 10, 76, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125745, 10, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125746, 10, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125747, 10, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125748, 10, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125749, 10, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125750, 10, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125751, 10, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125752, 10, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125753, 10, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125754, 10, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125755, 10, 855, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125756, 10, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466036125757, 10, 875, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320001, 10, 876, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320002, 10, 882, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320003, 10, 883, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320004, 10, 371, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320005, 10, 640, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320006, 10, 641, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320007, 10, 642, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320008, 10, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320009, 10, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320010, 10, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320011, 10, 667, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320012, 10, 669, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320013, 10, 670, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320014, 10, 671, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320015, 10, 672, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320016, 10, 673, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320017, 10, 417, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320018, 10, 674, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320019, 10, 420, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320020, 10, 164, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320021, 10, 421, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320022, 10, 165, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320023, 10, 167, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320024, 10, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320025, 10, 168, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320026, 10, 424, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320027, 10, 427, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320028, 10, 174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320029, 10, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320030, 10, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320031, 10, 191, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320032, 10, 708, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320033, 10, 709, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320034, 10, 710, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320035, 10, 711, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320036, 10, 712, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320037, 10, 717, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320038, 10, 718, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320039, 10, 463, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320040, 10, 719, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320041, 10, 464, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320042, 10, 720, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320043, 10, 465, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320044, 10, 466, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320045, 10, 467, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320046, 10, 468, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320047, 10, 726, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320048, 10, 727, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320049, 10, 728, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320050, 10, 729, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320051, 10, 477, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320052, 10, 735, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320053, 10, 736, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320054, 10, 737, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320055, 10, 738, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320056, 10, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320057, 10, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320058, 10, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320059, 10, 487, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320060, 10, 488, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320061, 10, 489, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320062, 10, 490, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320063, 10, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320064, 10, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320065, 10, 749, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320066, 10, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320067, 10, 494, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320068, 10, 498, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320069, 10, 754, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320070, 10, 499, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320071, 10, 755, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320072, 10, 500, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466040320073, 10, 502, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466044514305, 10, 503, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466044514306, 10, 759, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466044514307, 10, 760, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466044514308, 10, 504, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466044514309, 10, 505, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281466044514310, 10, 506, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620629782529, 11, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976833, 11, 667, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976834, 11, 540, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976835, 11, 669, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976836, 11, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976837, 11, 670, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976838, 11, 542, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976839, 11, 671, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976840, 11, 543, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976841, 11, 672, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976842, 11, 544, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976843, 11, 673, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976844, 11, 545, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976845, 11, 674, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976846, 11, 561, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976847, 11, 563, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976848, 11, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976849, 11, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976850, 11, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976851, 11, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976852, 11, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976853, 11, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976854, 11, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976855, 11, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976856, 11, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976857, 11, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976858, 11, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976859, 11, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976860, 11, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976861, 11, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976862, 11, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976863, 11, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976864, 11, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976865, 11, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976866, 11, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976867, 11, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976868, 11, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976869, 11, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976870, 11, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976871, 11, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976872, 11, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976873, 11, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976874, 11, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976875, 11, 855, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976876, 11, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976877, 11, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976878, 11, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976879, 11, 487, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976880, 11, 488, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976881, 11, 489, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976882, 11, 490, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976883, 11, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976884, 11, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976885, 11, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976886, 11, 494, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976887, 11, 495, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976888, 11, 498, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976889, 11, 754, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976890, 11, 882, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976891, 11, 499, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976892, 11, 883, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976893, 11, 500, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976894, 11, 502, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976895, 11, 503, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976896, 11, 504, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976897, 11, 505, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1687281620633976898, 11, 506, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (600, 12, 199, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (601, 12, 200, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (602, 12, 201, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (603, 12, 202, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (604, 12, 203, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (605, 12, 204, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (606, 12, 205, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (607, 12, 206, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (608, 12, 207, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (609, 12, 208, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (610, 12, 209, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (611, 12, 210, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (612, 12, 211, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (613, 12, 212, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (614, 12, 213, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (615, 12, 214, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (616, 12, 215, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (617, 12, 216, 1, NULL);
